@@ -28,7 +28,7 @@ var switchLZ; //Counter for incrementing index every third landing
 function extraVictoryCondition()
 {
 	var enemies = enumArea(0, 0, mapWidth, mapHeight, ENEMIES, false);
-	if(index === 5 && enemies.length === 0)
+	if (index === 5 && enemies.length === 0)
 	{
 		return true;
 	}
@@ -47,7 +47,7 @@ function checkForGroundForces()
 		var droidGroup2 = []; //Sensor and heavy mortar units
 		var templates = [ cTempl.nphct, cTempl.npmsens, cTempl.npmorb ];
 
-		for (var i = 0; i <= maxTanks; ++i)
+		for (let i = 0; i <= maxTanks; ++i)
 		{
 			if (i <= firstAmount)
 			{
@@ -86,7 +86,7 @@ function sendTransport()
 	var templates = [ cTempl.npcybc, cTempl.npcybf, cTempl.npcybm ];
 
 	var droids = [];
-	for (var i = 0; i < count; ++i)
+	for (let i = 0; i < count; ++i)
 	{
 		var t = templates[camRand(templates.length)];
 		// two droids of each template
@@ -94,7 +94,6 @@ function sendTransport()
 		droids[droids.length] = t;
 	}
 
-
 	camSendReinforcement(NEW_PARADIGM, position, droids, CAM_REINFORCE_TRANSPORT, {
 		entry: { x: 126, y: 36 },
 		exit: { x: 126, y: 76 },
@@ -102,9 +101,9 @@ function sendTransport()
 		order: CAM_ORDER_PATROL,
 		data: {
 			pos:[
-				camMakePos( cyborgPatrolList[(3 * index)] ),
-				camMakePos( cyborgPatrolList[(3 * index) + 1] ),
-				camMakePos( cyborgPatrolList[(3 * index) + 2] ),
+				camMakePos(cyborgPatrolList[(3 * index)]),
+				camMakePos(cyborgPatrolList[(3 * index) + 1]),
+				camMakePos(cyborgPatrolList[(3 * index) + 2]),
 			],
 			radius: 8,
 			interval: camMinutesToMilliseconds(1),
@@ -125,7 +124,6 @@ function sendTransport()
 	if (index === 5)
 	{
 		removeTimer("sendTransport");
-		return;
 	}
 }
 
@@ -151,10 +149,10 @@ function eventStartLevel()
 	setMissionTime(camChangeOnDiff(camHoursToSeconds(1)));
 
 	// make sure player doesn't build on enemy LZs
-	for (var i = 6; i <= 10; ++i)
+	for (let i = 6; i <= 10; ++i)
 	{
 		var ph = getObject("NPLZ" + i);
-		setNoGoArea(ph.x, ph.y, ph.x2, ph.y2, NEW_PARADIGM);
+		setNoGoArea(ph.x, ph.y, ph.x2, ph.y2, i - 4);
 	}
 
 	camCompleteRequiredResearch(NEW_PARADIGM_RES, NEW_PARADIGM);
