@@ -5,26 +5,35 @@
 
 //;; ## camDef(something)
 //;;
-//;; Returns false if something is JavaScript-undefined, true otherwise.
+//;; Returns `false` if something is JavaScript-undefined, `true` otherwise.
+//;;
+//;; @param {*} something
+//;; @returns {boolean}
 //;;
 function camDef(something)
 {
-	return typeof(something) !== "undefined";
+	return typeof something !== "undefined";
 }
 
 //;; ## camIsString(something)
 //;;
-//;; Returns true if something is a string, false otherwise.
+//;; Returns `true` if something is a string, `false` otherwise.
+//;;
+//;; @param {*} something
+//;; @returns {boolean}
 //;;
 function camIsString(something)
 {
-	return typeof(something) === "string";
+	return typeof something === "string";
 }
 
 //;; ## camRand(max)
 //;;
 //;; A non-synchronous random integer in range [0, max - 1].
 //;;
+//;; @param {number} max
+//;; @returns {number}
+//;;
 function camRand(max)
 {
 	if (max > 0)
@@ -34,25 +43,32 @@ function camRand(max)
 	camDebug("Max should be positive");
 }
 
-//;; ## camCallOnce(function name)
+//;; ## camCallOnce(functionName)
 //;;
 //;; Call a function by name, but only if it has not been called yet.
 //;;
-function camCallOnce(callback)
+//;; @param {string} functionName
+//;; @returns {void}
+//;;
+function camCallOnce(functionName)
 {
-	if (camDef(__camCalledOnce[callback]) && __camCalledOnce[callback])
+	if (camDef(__camCalledOnce[functionName]) && __camCalledOnce[functionName])
 	{
 		return;
 	}
-	__camCalledOnce[callback] = true;
-	__camGlobalContext()[callback]();
+	__camCalledOnce[functionName] = true;
+	__camGlobalContext()[functionName]();
 }
 
-//;; ## camSafeRemoveObject(obj[, special effects?])
+//;; ## camSafeRemoveObject(obj[, specialEffects])
 //;;
 //;; Remove a game object (by value or label) if it exists, do nothing otherwise.
 //;;
-function camSafeRemoveObject(obj, flashy)
+//;; @param {string|Object} obj
+//;; @param {boolean} [specialEffects]
+//;; @returns {void}
+//;;
+function camSafeRemoveObject(obj, specialEffects)
 {
 	if (__camLevelEnded)
 	{
@@ -64,36 +80,39 @@ function camSafeRemoveObject(obj, flashy)
 	}
 	if (camDef(obj) && obj)
 	{
-		removeObject(obj, flashy);
+		removeObject(obj, specialEffects);
 	}
 }
 
-//;; ## camMakePos(x, y | label | object)
+//;; ## camMakePos(label|object|x[, y])
 //;;
-//;; Make a POSITION-like object, unless already done. Often useful
-//;; for making functions that would accept positions in both xx,yy and {x:xx,y:yy} forms.
-//;; Also accepts labels. If label of AREA is given, returns the center of the area.
-//;; If an existing object or label of such is given, returns a safe JavaScript
-//;; object containing its x, y and id.
+//;; Make a `POSITION`-like object, unless already done.
+//;; Often useful for making functions that would accept positions in both `x,y` and `{x: x, y: y}` forms.
+//;; Also accepts labels. If label of `AREA` is given, returns the center of the area.
+//;; If an existing object or label of such is given, returns a safe JavaScript object containing its `x`, `y` and `id`.
 //;;
-function camMakePos(xx, yy)
+//;; @param {number|string|Object|undefined} x
+//;; @param {number} [y]
+//;; @returns {Object|undefined}
+//;;
+function camMakePos(x, y)
 {
-	if (camDef(yy))
+	if (camDef(y))
 	{
-		return { x : xx, y : yy };
+		return { x: x, y: y };
 	}
-	if (!camDef(xx))
+	if (!camDef(x))
 	{
 		return undefined;
 	}
-	var obj = xx;
-	if (camIsString(xx))
+	var obj = x;
+	if (camIsString(x))
 	{
-		obj = getObject(xx);
+		obj = getObject(x);
 	}
 	if (!camDef(obj) || !obj)
 	{
-		camDebug("Failed at", xx);
+		camDebug("Failed at", x);
 		return undefined;
 	}
 	switch (obj.type)
@@ -125,7 +144,13 @@ function camMakePos(xx, yy)
 
 //;; ## camDist(x1, y1, x2, y2 | pos1, x2, y2 | x1, y1, pos2 | pos1, pos2)
 //;;
-//;; A wrapper for ```distBetweenTwoPoints()```.
+//;; A wrapper for `distBetweenTwoPoints()`.
+//;;
+//;; @param {number|Object} x1
+//;; @param {number|Object} y1
+//;; @param {number} [x2]
+//;; @param {number} [y2]
+//;; @returns {number}
 //;;
 function camDist(x1, y1, x2, y2)
 {
@@ -133,11 +158,11 @@ function camDist(x1, y1, x2, y2)
 	{
 		return distBetweenTwoPoints(x1, y1, x2, y2);
 	}
-	var pos2 = camMakePos(x2, y2);
-	if (!camDef(pos2)) // pos1, pos2
+	if (!camDef(x2)) // pos1, pos2
 	{
 		return distBetweenTwoPoints(x1.x, x1.y, y1.x, y1.y);
 	}
+	var pos2 = camMakePos(x2);
 	if (camDef(pos2.x)) // x2 is pos2
 	{
 		return distBetweenTwoPoints(x1, y1, pos2.x, pos2.y);
@@ -148,35 +173,41 @@ function camDist(x1, y1, x2, y2)
 	}
 }
 
-//;; ## camPlayerMatchesFilter(player, filter)
+//;; ## camPlayerMatchesFilter(playerId, playerFilter)
 //;;
-//;; A function to handle player filters in a way similar to
-//;; how JS API functions (eg. ```enumDroid(filter, ...)```) handle them.
+//;; A function to handle player filters in a way similar to how JS API functions (eg. `enumDroid(filter, ...)`) handle them.
 //;;
-function camPlayerMatchesFilter(player, filter)
+//;; @param {number} playerId
+//;; @param {number} playerFilter
+//;; @returns {boolean}
+//;;
+function camPlayerMatchesFilter(playerId, playerFilter)
 {
-	switch(filter) {
+	switch (playerFilter) {
 		case ALL_PLAYERS:
 			return true;
 		case ALLIES:
-			return player === CAM_HUMAN_PLAYER;
+			return playerId === CAM_HUMAN_PLAYER;
 		case ENEMIES:
-			return player >= 0 && player < CAM_MAX_PLAYERS && player !== CAM_HUMAN_PLAYER;
+			return playerId >= 0 && playerId < CAM_MAX_PLAYERS && playerId !== CAM_HUMAN_PLAYER;
 		default:
-			return player === filter;
+			return playerId === playerFilter;
 	}
 }
 
-//;; ## camRemoveDuplicates(array)
+//;; ## camRemoveDuplicates(items)
 //;;
 //;; Remove duplicate items from an array.
 //;;
-function camRemoveDuplicates(array)
+//;; @param {*[]} items
+//;; @returns {*[]}
+//;;
+function camRemoveDuplicates(items)
 {
 	var prims = {"boolean":{}, "number":{}, "string":{}};
 	var objs = [];
 
-	return array.filter(function(item) {
+	return items.filter((item) => {
 		var type = typeof item;
 		if (type in prims)
 		{
@@ -189,19 +220,23 @@ function camRemoveDuplicates(array)
 	});
 }
 
-//;; ## camCountStructuresInArea(label, [player])
+//;; ## camCountStructuresInArea(label[, playerFilter])
 //;;
-//;; Mimics wzscript's numStructsButNotWallsInArea().
+//;; Mimics wzscript's `numStructsButNotWallsInArea()`.
 //;;
-function camCountStructuresInArea(lab, player)
+//;; @param {string} label
+//;; @param {number} [playerFilter]
+//;; @returns {number}
+//;;
+function camCountStructuresInArea(label, playerFilter)
 {
-	if (!camDef(player))
+	if (!camDef(playerFilter))
 	{
-		player = CAM_HUMAN_PLAYER;
+		playerFilter = CAM_HUMAN_PLAYER;
 	}
-	var list = enumArea(lab, player, false);
+	var list = enumArea(label, playerFilter, false);
 	var ret = 0;
-	for (var i = 0, l = list.length; i < l; ++i)
+	for (let i = 0, l = list.length; i < l; ++i)
 	{
 		var object = list[i];
 		if (object.type === STRUCTURE && object.stattype !== WALL && object.status === BUILT)
@@ -212,69 +247,81 @@ function camCountStructuresInArea(lab, player)
 	return ret;
 }
 
-//;; ## camChangeOnDiff(numeric value)
+//;; ## camChangeOnDiff(numericValue)
 //;;
 //;; Change a numeric value based on campaign difficulty.
 //;;
-function camChangeOnDiff(num)
+//;; @param {number} numericValue
+//;; @returns {number}
+//;;
+function camChangeOnDiff(numericValue)
 {
 	var modifier = 0;
 
 	switch (difficulty)
 	{
+		case SUPEREASY:
+			modifier = 2;
+			break;
 		case EASY:
-			modifier = 1.25;
+			modifier = 1.5;
 			break;
 		case MEDIUM:
-			modifier = 1.00;
+			modifier = 1;
 			break;
 		case HARD:
-			modifier = 0.80;
+			modifier = 0.85;
 			break;
 		case INSANE:
 			modifier = 0.70;
 			break;
 		default:
-			modifier = 1.00;
+			modifier = 1;
 			break;
 	}
 
-	return Math.floor(num * modifier);
+	return Math.floor(numericValue * modifier);
 }
 
-//;; ## camIsSystemDroid(game object)
+//;; ## camIsSystemDroid(gameObject)
 //;;
 //;; Determine if the passed in object is a non-weapon based droid.
 //;;
-function camIsSystemDroid(obj)
+//;; @param {Object} gameObject
+//;; @returns {boolean}
+//;;
+function camIsSystemDroid(gameObject)
 {
-	if(!camDef(obj) || !obj)
+	if (!camDef(gameObject) || !gameObject)
 	{
 		return false;
 	}
 
-	if (obj.type !== DROID)
+	if (gameObject.type !== DROID)
 	{
-		camTrace("Non-droid: " + obj.type + " pl: " + obj.name);
+		camTrace("Non-droid: " + gameObject.type + " pl: " + gameObject.name);
 		return false;
 	}
 
-	return (obj.droidType === DROID_SENSOR || obj.droidType === DROID_CONSTRUCT || obj.droidType === DROID_REPAIR);
+	return (gameObject.droidType === DROID_SENSOR || gameObject.droidType === DROID_CONSTRUCT || gameObject.droidType === DROID_REPAIR);
 }
 
-//;; ## camMakeGroup(what, filter)
+//;; ## camMakeGroup(what[, playerFilter])
 //;;
-//;; Make a new group out of array of droids, single game object,
-//;; or label string, with fuzzy auto-detection of argument type.
-//;; Only droids would be added to the group. ```filter``` can be one of
-//;; a player index, ```ALL_PLAYERS```, ```ALLIES``` or ```ENEMIES```;
-//;; defaults to ```ENEMIES```.
+//;; Make a new group out of array of droids, single game object, or label string,
+//;; with fuzzy auto-detection of argument type.
+//;; Only droids would be added to the group. `playerFilter` can be one of a
+//;; player index, `ALL_PLAYERS`, `ALLIES` or `ENEMIES`; defaults to `ENEMIES`.
 //;;
-function camMakeGroup(what, filter)
+//;; @param {string|Object|Object[]} what
+//;; @param {number} [playerFilter]
+//;; @returns {number|void}
+//;;
+function camMakeGroup(what, playerFilter)
 {
-	if (!camDef(filter))
+	if (!camDef(playerFilter))
 	{
-		filter = ENEMIES;
+		playerFilter = ENEMIES;
 	}
 	var array;
 	var obj;
@@ -292,7 +339,7 @@ function camMakeGroup(what, filter)
 	}
 	if (camDef(obj))
 	{
-		switch(obj.type) {
+		switch (obj.type) {
 			case POSITION:
 				obj = getObject(obj.x, obj.y);
 				// fall-through
@@ -302,12 +349,10 @@ function camMakeGroup(what, filter)
 				array = [ obj ];
 				break;
 			case AREA:
-				array = enumArea(obj.x, obj.y, obj.x2, obj.y2,
-				                 ALL_PLAYERS, false);
+				array = enumArea(obj.x, obj.y, obj.x2, obj.y2, ALL_PLAYERS, false);
 				break;
 			case RADIUS:
-				array = enumRange(obj.x, obj.y, obj.radius,
-				                 ALL_PLAYERS, false);
+				array = enumRange(obj.x, obj.y, obj.radius, ALL_PLAYERS, false);
 				break;
 			case GROUP:
 				array = enumGroup(obj.id);
@@ -319,7 +364,7 @@ function camMakeGroup(what, filter)
 	if (camDef(array))
 	{
 		var group = camNewGroup();
-		for (var i = 0, l = array.length; i < l; ++i)
+		for (let i = 0, l = array.length; i < l; ++i)
 		{
 			var o = array[i];
 			if (!camDef(o) || !o)
@@ -327,7 +372,7 @@ function camMakeGroup(what, filter)
 				camDebug("Trying to add", o);
 				continue;
 			}
-			if (o.type === DROID && o.droidType !== DROID_CONSTRUCT && camPlayerMatchesFilter(o.player, filter))
+			if (o.type === DROID && o.droidType !== DROID_CONSTRUCT && camPlayerMatchesFilter(o.player, playerFilter))
 			{
 				groupAdd(group, o);
 			}
@@ -341,11 +386,13 @@ function camMakeGroup(what, filter)
 //;;
 //;; Break alliances between all players.
 //;;
+//;; @returns {void}
+//;;
 function camBreakAlliances()
 {
-	for (var i = 0; i < CAM_MAX_PLAYERS; ++i)
+	for (let i = 0; i < CAM_MAX_PLAYERS; ++i)
 	{
-		for (var c = 0; c < CAM_MAX_PLAYERS; ++c)
+		for (let c = 0; c < CAM_MAX_PLAYERS; ++c)
 		{
 			if (i !== c && allianceExistsBetween(i, c) === true)
 			{
@@ -355,7 +402,13 @@ function camBreakAlliances()
 	}
 }
 
-// Picks a random coordinate anywhere on the edge of the map.
+//;; ## camGenerateRandomMapEdgeCoordinate(reachPosition)
+//;;
+//;; Returns a random coordinate anywhere on the edge of the map that reachs a position.
+//;;
+//;; @param {Object} reachPosition
+//;; @returns {Object}
+//;;
 function camGenerateRandomMapEdgeCoordinate(reachPosition)
 {
 	let limits = getScrollLimits();
@@ -404,11 +457,16 @@ function camGenerateRandomMapEdgeCoordinate(reachPosition)
 		loc = location;
 	} while (camDef(reachPosition) && reachPosition && !propulsionCanReach("wheeled01", reachPosition.x, reachPosition.y, loc.x, loc.y));
 
-
 	return loc;
 }
 
-// Picks a random coordinate anywhere on the map.
+//;; ## camGenerateRandomMapCoordinate(reachPosition)
+//;;
+//;; Returns a random coordinate anywhere on the map
+//;;
+//;; @param {Object} reachPosition
+//;; @returns {Object}
+//;;
 function camGenerateRandomMapCoordinate(reachPosition, distFromReach, scanObjectRadius)
 {
 	if (!camDef(distFromReach))
@@ -450,11 +508,44 @@ function camGenerateRandomMapCoordinate(reachPosition, distFromReach, scanObject
 		reachPosition &&
 		!propulsionCanReach("wheeled01", reachPosition.x, reachPosition.y, pos.x, pos.y) &&
 		(camDist(pos, reachPosition) < distFromReach) &&
-		(enumRange(pos.x, pos.y, scanObjectRadius, ALL_PLAYERS, false).length !== 0));
+		(enumRange(pos.x, pos.y, scanObjectRadius, ALL_PLAYERS, false).length > 0));
 
 	return pos;
 }
 
+//;; ## camDiscoverCampaign()
+//;;
+//;; Figures out what campaign we are in without reliance on the source at all.
+//;;
+//;; @returns {number}
+//;;
+function camDiscoverCampaign()
+{
+	for (let i = 0, len = ALPHA_LEVELS.length; i < len; ++i)
+	{
+		if (__camNextLevel === ALPHA_LEVELS[i] || __camNextLevel === BETA_LEVELS[0])
+		{
+			return ALPHA_CAMPAIGN_NUMBER;
+		}
+	}
+	for (let i = 0, len = BETA_LEVELS.length; i < len; ++i)
+	{
+		if (__camNextLevel === BETA_LEVELS[i] || __camNextLevel === GAMMA_LEVELS[0])
+		{
+			return BETA_CAMPAIGN_NUMBER;
+		}
+	}
+	for (let i = 0, len = GAMMA_LEVELS.length; i < len; ++i)
+	{
+		if (__camNextLevel === GAMMA_LEVELS[i] || __camNextLevel === CAM_GAMMA_OUT)
+		{
+			return GAMMA_CAMPAIGN_NUMBER;
+		}
+	}
+
+	return UNKNOWN_CAMPAIGN_NUMBER;
+}
+
 //////////// privates
 
 function __camGlobalContext()
@@ -466,13 +557,13 @@ function __camFindClusters(list, size)
 {
 	// The good old cluster analysis algorithm taken from NullBot AI.
 	var ret = { clusters: [], xav: [], yav: [], maxIdx: 0, maxCount: 0 };
-	for (var i = list.length - 1; i >= 0; --i)
+	for (let i = list.length - 1; i >= 0; --i)
 	{
 		var x = list[i].x;
 		var y = list[i].y;
 		var found = false;
 		var n = 0;
-		for (var j = 0; j < ret.clusters.length; ++j)
+		for (let j = 0; j < ret.clusters.length; ++j)
 		{
 			if (camDist(ret.xav[j], ret.yav[j], x, y) < size)
 			{
@@ -518,7 +609,7 @@ function __camTick()
 //Reset AI power back to highest storage possible.
 function __camAiPowerReset()
 {
-	for (var i = 1; i < CAM_MAX_PLAYERS; ++i)
+	for (let i = 1; i < CAM_MAX_PLAYERS; ++i)
 	{
 		setPower(AI_POWER, i);
 	}
