@@ -1,5 +1,6 @@
 include("script/campaign/libcampaign.js");
 include("script/campaign/templates.js");
+
 const SCAVENGER_PLAYER = 7;
 
 camAreaEvent("removeObjectiveBlip", function()
@@ -40,7 +41,7 @@ function grantStartTech()
 	];
 
 	camCompleteRequiredResearch(TECH, CAM_HUMAN_PLAYER);
-	for (var i = 0, l = STRUCTS.length; i < l; ++i)
+	for (let i = 0, l = STRUCTS.length; i < l; ++i)
 	{
 		enableStructure(STRUCTS[i], CAM_HUMAN_PLAYER);
 	}
@@ -165,10 +166,11 @@ function eventStartLevel()
 	camSafeRemoveObject("flamerArti", false);
 	camSetArtifacts({
 		"base1Factory": { tech: "R-Defense-Tower01" },
+		"base1PowerGenerator": { tech: "R-Struc-PowerModuleMk1" },
 		"artifactPos": { tech: "R-Wpn-Flamer01Mk1" },
 		"radarTower": { tech: "R-Sys-Sensor-Turret01" },
 		"base2Factory": { tech: "R-Vehicle-Prop-Halftracks" },
-		"bunkerArti": { tech: "R-Sys-Engineering01" },
+		"bunkerArti": { tech: ["R-Sys-Engineering01", "R-Sys-MobileRepairTurret01" ]},
 	});
 
 	camSetFactories({
@@ -191,7 +193,7 @@ function eventStartLevel()
 			order: CAM_ORDER_ATTACK,
 			groupSize: 5,
 			throttle: camChangeOnDiff(camSecondsToMilliseconds(15)),
-			templates: [cTempl.rbjeep, cTempl.trike, cTempl.buggy, cTempl.rbjeep]
+			templates: [cTempl.rbjeep, cTempl.trike, cTempl.buggy, cTempl.rbjeep, cTempl.bjeep]
 		}
 	});
 
