@@ -15,7 +15,7 @@ function reticuleManufactureCheck()
 	var structureComplete = false;
 	var facs = [FACTORY, CYBORG_FACTORY, VTOL_FACTORY,];
 
-	for (var i = 0, len = facs.length; i < len; ++i)
+	for (let i = 0, len = facs.length; i < len; ++i)
 	{
 		var facType = facs[i];
 		var offWorldFacs = enumStructOffWorld(selectedPlayer, facType);
@@ -23,7 +23,7 @@ function reticuleManufactureCheck()
 
 		if (offWorldFacs !== null)
 		{
-			for (var j = 0, len2 = offWorldFacs.length; j < len2; ++j)
+			for (let j = 0, len2 = offWorldFacs.length; j < len2; ++j)
 			{
 				if (offWorldFacs[j].status === BUILT)
 				{
@@ -34,7 +34,7 @@ function reticuleManufactureCheck()
 		}
 		if (!structureComplete)
 		{
-			for (var j = 0, len2 = onMapFacs.length; j < len2; ++j)
+			for (let j = 0, len2 = onMapFacs.length; j < len2; ++j)
 			{
 				if (onMapFacs[j].status === BUILT)
 				{
@@ -65,7 +65,7 @@ function reticuleResearchCheck()
 	var structureComplete = false;
 	var labs = [RESEARCH_LAB,];
 
-	for (var i = 0, len = labs.length; i < len; ++i)
+	for (let i = 0, len = labs.length; i < len; ++i)
 	{
 		var resType = labs[i];
 		var offWorldLabs = enumStructOffWorld(selectedPlayer, resType);
@@ -73,7 +73,7 @@ function reticuleResearchCheck()
 
 		if (offWorldLabs !== null)
 		{
-			for (var j = 0, len2 = offWorldLabs.length; j < len2; ++j)
+			for (let j = 0, len2 = offWorldLabs.length; j < len2; ++j)
 			{
 				if (offWorldLabs[j].status === BUILT)
 				{
@@ -84,7 +84,7 @@ function reticuleResearchCheck()
 		}
 		if (!structureComplete)
 		{
-			for (var j = 0, len2 = onMapLabs.length; j < len2; ++j)
+			for (let j = 0, len2 = onMapLabs.length; j < len2; ++j)
 			{
 				if (onMapLabs[j].status === BUILT)
 				{
@@ -127,7 +127,7 @@ function reticuleDesignCheck()
 	var structureComplete = false;
 	var hqs = [HQ,];
 
-	for (var i = 0, len = hqs.length; i < len; ++i)
+	for (let i = 0, len = hqs.length; i < len; ++i)
 	{
 		var hqType = hqs[i];
 		var offWorldHq = enumStructOffWorld(selectedPlayer, hqType);
@@ -135,7 +135,7 @@ function reticuleDesignCheck()
 
 		if (offWorldHq !== null)
 		{
-			for (var j = 0, len2 = offWorldHq.length; j < len2; ++j)
+			for (let j = 0, len2 = offWorldHq.length; j < len2; ++j)
 			{
 				if (offWorldHq[j].status === BUILT)
 				{
@@ -146,7 +146,7 @@ function reticuleDesignCheck()
 		}
 		if (!structureComplete)
 		{
-			for (var j = 0, len2 = onMapHq.length; j < len2; ++j)
+			for (let j = 0, len2 = onMapHq.length; j < len2; ++j)
 			{
 				if (onMapHq[j].status === BUILT)
 				{
@@ -287,7 +287,7 @@ function setLimits()
 	setDroidLimit(selectedPlayer, 10, DROID_COMMAND);
 	setDroidLimit(selectedPlayer, 15, DROID_CONSTRUCT);
 
-	for (var i = 0; i < maxPlayers; ++i)
+	for (let i = 0; i < maxPlayers; ++i)
 	{
 		setStructureLimits("A0PowerGenerator", 5, i);
 		setStructureLimits("A0ResourceExtractor", 200, i);
@@ -310,7 +310,7 @@ function resetPower()
 	var powerProductionRate = 100;
 
 	// set income modifier/power storage for player 0 (human)
-	if (difficulty === EASY)
+	if (difficulty <= EASY)
 	{
 		powerProductionRate = 115;
 	}
@@ -321,11 +321,11 @@ function resetPower()
 
 		if (tilesetType === "URBAN")
 		{
-			powerLimit = powerLimit + 5000;
+			powerLimit += 5000;
 		}
 		else if (tilesetType === "ROCKIES")
 		{
-			powerLimit = powerLimit + 10000;
+			powerLimit += 10000;
 		}
 	}
 	else if (difficulty === INSANE)
@@ -335,11 +335,11 @@ function resetPower()
 
 		if (tilesetType === "URBAN")
 		{
-			powerLimit = powerLimit + 2000;
+			powerLimit += 2000;
 		}
 		else if (tilesetType === "ROCKIES")
 		{
-			powerLimit = powerLimit + 4000;
+			powerLimit += 4000;
 		}
 	}
 
