@@ -1,6 +1,6 @@
 function completeResearchOnTime(time, playnum)
 {
-	for (var tech in allRes)
+	for (const tech in allRes)
 	{
 		if (allRes[tech] <= time)
 		{
@@ -43,22 +43,20 @@ var allRes = {
 	"R-Wpn-Cannon-Damage01": 262,
 	"R-Defense-Pillbox04": 284,
 	"R-Defense-WallTower02": 284,
-	"R-Struc-Materials01": 301,
 	"R-Comp-CommandTurret01": 301,
 	"R-Struc-Research-Module": 301,
 	"R-Sys-MobileRepairTurretHvy": 304,
 	"R-Wpn-Flamer-ROF01": 315,
 	"R-Vehicle-Engine02": 329,
 	"R-Wpn-Rocket-Damage01": 329,
+	"R-Struc-Materials01": 358,
 	"R-Struc-Research-Upgrade01": 358,
 	"R-Defense-WallUpgrade02": 360,
 	"R-Wpn-MG-Damage03": 360,
 	"R-Struc-Factory-Module": 384,
-	"R-Wpn-Flamer-Damage03": 385,
 	"R-Wpn-Cannon-Damage02": 385,
+	"R-Wpn-Flamer-Damage03": 385,
 	"R-Defense-Tower06": 395,
-	"R-Struc-Materials02": 404,
-	"R-Wpn-AAGun03": 407,
 	"R-Wpn-MG3Mk1": 407,
 	"R-Wpn-Rocket-Damage02": 427,
 	"R-Cyborg-Metals01": 430,
@@ -69,30 +67,27 @@ var allRes = {
 	"R-Struc-Factory-Upgrade01": 472,
 	"R-Struc-RepairFacility": 472,
 	"R-Wpn-Mortar01Lt": 472,
-	"R-Defense-AASite-QuadMg1": 491,
 	"R-Vehicle-Body04": 492,
-	"R-Defense-WallUpgrade03": 509,
 	"R-Wpn-Rocket02-MRL": 509,
+	"R-Defense-WallUpgrade03": 511,
 	"R-Vehicle-Prop-Hover": 511,
 	"R-Wpn-Cannon2Mk1": 511,
-	"R-Cyborg-Metals02": 511,
 	"R-Defense-MortarPit": 511,
-	"R-Wpn-Cannon-Accuracy01": 520,
-	"R-Wpn-Cannon-Damage03": 520,
+	"R-Cyborg-Metals02": 511,
+	"R-Wpn-Cannon-Accuracy01": 512,
+	"R-Wpn-Cannon-Damage03": 512,
 	"R-Vehicle-Engine03": 528,
-	"R-Struc-Materials03": 528,
 	"R-Wpn-Mortar-Damage01": 530,
 	"R-Wpn-Rocket-Accuracy01": 547,
 	"R-Wpn-Rocket-Damage03": 547,
 	"R-Defense-MRL": 548,
-	"R-Wpn-Rocket-ROF01": 550,
 	"R-Struc-Research-Upgrade03": 567,
 	"R-Struc-RprFac-Upgrade01": 569,
+	"R-Wpn-Rocket-ROF01": 569,
 	"R-Vehicle-Metals02": 569,
 	"R-Defense-WallTower03": 569,
-	"R-Wpn-MG-ROF01": 585,
 	"R-Wpn-MG-Damage04": 602,
-	"R-Wpn-AAGun-Damage01": 602,
+	"R-Wpn-MG-ROF01": 602,
 	"R-Cyborg-Metals03": 619,
 	"R-Wpn-Flamer-Damage04": 620,
 	"R-Wpn-Mortar-Acc01": 636,
@@ -101,39 +96,37 @@ var allRes = {
 	"R-Vehicle-Body11": 636,
 	"R-Vehicle-Prop-Tracks": 636,
 	"R-Wpn-Rocket01-LtAT": 650,
-	"R-Wpn-Cannon4AMk1": 667,
-	"R-Wpn-Rocket-ROF02": 686,
+	"R-Wpn-Cannon4AMk1": 669,
+	"R-Wpn-Cannon-Damage04": 686,
 	"R-Defense-Pillbox06": 692,
 	"R-Defense-WallTower06": 692,
-	"R-Wpn-Cannon-Damage04": 694,
 	"R-Struc-VTOLFactory": 700,
 	"R-Struc-Research-Upgrade04": 701,
-	"R-Wpn-AAGun-ROF01": 702,
-	"R-Wpn-MG-ROF02": 717,
 	"R-Vehicle-Metals03": 717,
 	"R-Cyborg-Armor-Heat01": 717,
+	"R-Wpn-MG-ROF02": 731,
 	"R-Vehicle-Body12": 732,
 	"R-Sys-Sensor-Upgrade01": 745,
-	"R-Defense-Emplacement-HPVcannon": 750,
+	"R-Wpn-Rocket-ROF02": 746,
+	"R-Defense-Emplacement-HPVcannon": 751,
+	"R-Wpn-MG-Damage05": 761,
+	"R-Wpn-Mortar-Acc02": 774,
 	"R-Wpn-Rocket-Damage04": 774,
 	"R-Wpn-Rocket03-HvAT": 774,
-	"R-Wpn-Mortar-Acc02": 774,
+	"R-Wpn-Cannon-ROF01": 786,
 	"R-Cyborg-Transport": 789,
-	"R-Defense-WallTower-HPVcannon": 789,
-	"R-Wpn-AAGun-Damage02": 790,
-	"R-Wpn-Cannon-ROF01": 792,
+	"R-Defense-WallTower-HPVcannon": 790,
 	"R-Sys-RadarDetector01": 794,
-	"R-Wpn-MG-Damage05": 803,
 	"R-Wpn-Sunburst": 803,
 	"R-Wpn-Flame2": 805,
 	"R-Struc-Factory-Upgrade04": 808,
-	"R-Wpn-AAGun01": 818,
 	"R-Sys-Engineering02": 818,
 	"R-Wpn-Cannon-Accuracy02": 818,
-	"R-Wpn-AAGun-ROF02": 819,
+	"R-Wpn-Mortar-Damage03": 819,
 	"R-Wpn-Mortar02Hvy": 819,
 	"R-Wpn-Mortar3": 819,
-	"R-Wpn-Mortar-Damage03": 832,
+	"R-Wpn-AAGun01": 819,
+	"R-Wpn-AAGun03": 820,
 	"R-Vehicle-Armor-Heat01": 834,
 	"R-Cyborg-Armor-Heat02": 834,
 	"R-Cyborg-Metals04": 838,
@@ -143,159 +136,150 @@ var allRes = {
 	"R-Wpn-Flamer-Damage05": 853,
 	"R-Defense-Sunburst": 860,
 	"R-Sys-CBSensor-Turret01": 861,
-	"R-Wpn-Cannon-Damage05": 868,
+	"R-Wpn-Mortar-ROF01": 862,
+	"R-Wpn-Cannon-Damage05": 862,
 	"R-Vehicle-Engine04": 869,
-	"R-Wpn-Mortar-ROF01": 873,
-	"R-Defense-AASite-QuadBof": 873,
-	"R-Wpn-MG-ROF03": 875,
-	"R-Defense-RotMor": 875,
-	"R-Defense-HvyMor": 878,
+	"R-Defense-AASite-QuadBof": 875,
+	"R-Defense-AASite-QuadMg1": 875,
+	"R-Wpn-MG4": 875,
 	"R-SuperTransport": 882,
 	"R-Wpn-Rocket-Accuracy02": 886,
+	"R-Defense-HvyMor": 888,
+	"R-Defense-RotMor": 888,
 	"R-Vehicle-Metals04": 896,
-	"R-Defense-HvyFlamer": 900,
+	"R-Defense-HvyFlamer": 897,
 	"R-Wpn-Rocket-Damage05": 912,
-	"R-Wpn-Mortar-Incendiary": 912,
-	"R-Wpn-Rocket-ROF03": 914,
+	"R-Wpn-Mortar-Incendiary": 914,
 	"R-Wpn-Mortar-Acc03": 923,
 	"R-Struc-VTOLPad": 926,
-	"R-Defense-WallTower-DoubleAAgun": 926,
 	"R-Cyborg-Hvywpn-Mcannon": 926,
 	"R-Sys-Sensor-Upgrade02": 926,
+	"R-Defense-WallTower-DoubleAAgun": 927,
 	"R-Sys-CBSensor-Tower01": 939,
+	"R-Wpn-MG-Damage06": 940,
 	"R-Wpn-Flamer-ROF02": 940,
 	"R-Struc-RprFac-Upgrade04": 943,
 	"R-Wpn-Cannon-ROF02": 943,
 	"R-Defense-WallUpgrade04": 952,
+	"R-Defense-RotMG": 953,
+	"R-Wpn-Rocket-ROF03": 966,
 	"R-Cyborg-Armor-Heat03": 966,
-	"R-Defense-MortarPit-Incendiary": 966,
+	"R-Wpn-Cannon5": 967,
+	"R-Defense-MortarPit-Incendiary": 967,
 	"R-Wpn-Bomb01": 967,
-	"R-Wpn-Cannon5": 972,
-	"R-Wpn-AAGun-Damage03": 979,
-	"R-Wpn-MG4": 979,
-	"R-Struc-Power-Upgrade01b": 982,
+	"R-Struc-Power-Upgrade01b": 984,
 	"R-Vehicle-Armor-Heat02": 988,
-	"R-Wpn-Mortar-ROF02": 1004,
+	"R-Wpn-Mortar-ROF02": 1000,
 	"R-Cyborg-Metals05": 1004,
-	"R-Wpn-MG-Damage06": 1008,
+	"R-Wpn-MG-ROF03": 1005,
 	"R-Cyborg-Hvywpn-HPV": 1013,
 	"R-Wpn-RocketSlow-Accuracy01": 1017,
 	"R-Wpn-Rocket06-IDF": 1017,
 	"R-Struc-Research-Upgrade06": 1022,
-	"R-Vehicle-Body02": 1027,
+	"R-Vehicle-Body02": 1026,
 	"R-Struc-VTOLPad-Upgrade01": 1029,
-	"R-Wpn-Mortar-Damage04": 1033,
-	"R-Wpn-Cannon-Damage06": 1048,
-	"R-Defense-RotMG": 1054,
-	"R-Wpn-Cannon3Mk1": 1056,
-	"R-Cyborg-Hvywpn-Acannon": 1056,
+	"R-Defense-Wall-RotMg": 1031,
+	"R-Wpn-Mortar-Damage04": 1034,
+	"R-Wpn-Cannon-Damage06": 1043,
+	"R-Wpn-Cannon3Mk1": 1050,
 	"R-Vehicle-Engine05": 1061,
+	"R-Cyborg-Hvywpn-Acannon": 1062,
 	"R-Wpn-Rocket-Damage06": 1065,
-	"R-Defense-Wall-VulcanCan": 1072,
+	"R-Defense-Wall-VulcanCan": 1066,
 	"R-Sys-VTOLStrike-Turret01": 1076,
-	"R-Struc-Materials04": 1077,
+	"R-Struc-Materials02": 1077,
 	"R-Wpn-Flamer-Damage06": 1090,
 	"R-Vehicle-Metals05": 1097,
-	"R-Defense-WallTower04": 1103,
-	"R-Wpn-Flamer-ROF03": 1104,
-	"R-Struc-Power-Upgrade01c": 1104,
+	"R-Defense-WallTower04": 1098,
+	"R-Wpn-Flamer-ROF03": 1105,
+	"R-Struc-Power-Upgrade01c": 1106,
 	"R-Wpn-Cannon-ROF03": 1108,
 	"R-Defense-IDFRocket": 1112,
 	"R-Wpn-Bomb-Damage01": 1114,
 	"R-Wpn-Bomb03": 1114,
 	"R-Defense-WallUpgrade05": 1116,
-	"R-Defense-Wall-RotMg": 1125,
-	"R-Wpn-Mortar-ROF03": 1133,
+	"R-Wpn-MG-Damage07": 1121,
+	"R-Wpn-Mortar-ROF03": 1126,
 	"R-Wpn-Rocket07-Tank-Killer": 1135,
 	"R-Struc-VTOLPad-Upgrade02": 1147,
+	"R-Wpn-AAGun04": 1148,
 	"R-Wpn-RocketSlow-Accuracy02": 1159,
-	"R-Wpn-Cannon6TwinAslt": 1166,
-	"R-Vehicle-Armor-Heat03": 1168,
+	"R-Wpn-Cannon6TwinAslt": 1160,
 	"R-Vehicle-Body06": 1168,
-	"R-Wpn-AAGun04": 1174,
+	"R-Vehicle-Armor-Heat03": 1168,
 	"R-Defense-HvyA-Trocket": 1183,
 	"R-Defense-WallTower-HvyA-Trocket": 1183,
 	"R-Cyborg-Metals06": 1194,
 	"R-Wpn-MG5": 1198,
-	"R-Wpn-MG-Damage07": 1206,
-	"R-Wpn-HowitzerMk1": 1210,
+	"R-Wpn-HowitzerMk1": 1211,
 	"R-Cyborg-Hvywpn-TK": 1214,
-	"R-Wpn-Mortar-Damage05": 1215,
+	"R-Wpn-Mortar-Damage05": 1217,
 	"R-Sys-VTOLStrike-Tower01": 1218,
 	"R-Struc-Research-Upgrade07": 1218,
+	"R-Wpn-MG-Damage08": 1219,
 	"R-Wpn-Rocket-Damage07": 1228,
-	"R-Struc-Materials05": 1232,
-	"R-Wpn-Cannon-Damage07": 1235,
+	"R-Wpn-Cannon-Damage07": 1230,
 	"R-Sys-VTOLCBS-Turret01": 1239,
+	"R-Defense-AASite-QuadRotMg": 1241,
 	"R-Wpn-Bomb02": 1252,
+	"R-Wpn-Mortar-ROF04": 1252,
 	"R-Wpn-PlasmaCannon": 1254,
-	"R-Wpn-Mortar-ROF04": 1259,
-	"R-Defense-AASite-QuadRotMg": 1264,
 	"R-Vehicle-Engine06": 1271,
-	"R-Wpn-AAGun-Accuracy01": 1271,
 	"R-Defense-Super-Cannon": 1275,
-	"R-Defense-Cannon6": 1285,
-	"R-Wpn-AAGun-ROF03": 1285,
+	"R-Defense-Cannon6": 1280,
 	"R-Defense-WallTower-TwinAGun": 1286,
 	"R-Wpn-Bomb04": 1287,
 	"R-Wpn-Bomb-Damage02": 1289,
-	"R-Defense-Howitzer": 1296,
 	"R-Struc-VTOLPad-Upgrade03": 1297,
+	"R-Defense-Howitzer": 1298,
 	"R-Defense-WallUpgrade06": 1304,
 	"R-Comp-CommandTurret02": 1307,
 	"R-Vehicle-Metals06": 1315,
-	"R-Wpn-Howitzer-Damage01": 1318,
+	"R-Wpn-Howitzer-Damage01": 1319,
 	"R-Vehicle-Body09": 1323,
-	"R-Wpn-Howitzer-Accuracy01": 1339,
 	"R-Cyborg-Armor-Heat04": 1340,
+	"R-Wpn-Howitzer-Accuracy01": 1340,
 	"R-Sys-Sensor-Upgrade03": 1347,
 	"R-Wpn-Plasmite-Flamer": 1347,
 	"R-Sys-VTOLCBS-Tower01": 1368,
+	"R-Defense-WallTower-QuadRotAA": 1369,
 	"R-Struc-Power-Upgrade02": 1379,
-	"R-Wpn-Mortar-Damage06": 1384,
-	"R-Wpn-Howitzer-Incendiary": 1389,
-	"R-Wpn-Howitzer03-Rot": 1389,
-	"R-Defense-WallTower-QuadRotAA": 1393,
+	"R-Wpn-Mortar-Damage06": 1385,
+	"R-Wpn-Howitzer-Incendiary": 1390,
+	"R-Wpn-Howitzer03-Rot": 1390,
 	"R-Wpn-AAGun02": 1397,
 	"R-Wpn-Rocket-Damage08": 1400,
 	"R-Cyborg-Metals07": 1404,
-	"R-Wpn-AAGun-Damage04": 1406,
-	"R-Struc-Materials06": 1411,
-	"R-Wpn-Cannon-ROF04": 1414,
-	"R-Wpn-MG-Damage08": 1415,
+	"R-Wpn-Cannon-ROF04": 1409,
 	"R-Defense-PlasmiteFlamer": 1418,
-	"R-Wpn-Cannon-Damage08": 1428,
+	"R-Wpn-Cannon-Damage08": 1423,
 	"R-Struc-Research-Upgrade08": 1432,
-	"R-Wpn-Howitzer-Damage02": 1458,
-	"R-Wpn-AAGun-Accuracy02": 1465,
+	"R-Wpn-Howitzer-Damage02": 1460,
 	"R-Struc-VTOLPad-Upgrade04": 1472,
 	"R-Wpn-Laser01": 1472,
-	"R-Defense-RotHow": 1475,
 	"R-Defense-Howitzer-Incendiary": 1475,
+	"R-Defense-RotHow": 1476,
 	"R-Defense-AASite-QuadBof02": 1479,
+	"R-Wpn-RailGun01": 1483,
 	"R-Wpn-Bomb-Damage03": 1485,
-	"R-Wpn-RailGun01": 1487,
 	"R-Sys-Sensor-WS": 1491,
-	"R-Wpn-Howitzer-Accuracy02": 1498,
+	"R-Wpn-Howitzer-Accuracy02": 1499,
 	"R-Defense-PlasmaCannon": 1505,
 	"R-Vehicle-Armor-Heat04": 1505,
 	"R-Wpn-Missile2A-T": 1508,
 	"R-Cyborg-Armor-Heat05": 1511,
 	"R-Defense-WallTower-DoubleAAgun02": 1518,
-	"R-Wpn-AAGun-ROF04": 1546,
 	"R-Vehicle-Metals07": 1548,
 	"R-Wpn-Flamer-Damage07": 1570,
 	"R-Wpn-Rocket-Damage09": 1580,
 	"R-Struc-Power-Upgrade03": 1580,
-	"R-Wpn-AAGun-Damage05": 1586,
 	"R-Cyborg-Hvywpn-A-T": 1590,
 	"R-Sys-Engineering03": 1590,
 	"R-Defense-Super-Rocket": 1600,
-	"R-Wpn-AAGun-Accuracy03": 1603,
-	"R-Defense-GuardTower-Rail1": 1606,
-	"R-Wpn-Cannon-ROF05": 1612,
-	"R-Wpn-Howitzer-Damage03": 1623,
-	"R-Wpn-Cannon-Damage09": 1625,
+	"R-Defense-GuardTower-Rail1": 1601,
+	"R-Wpn-Cannon-ROF05": 1608,
+	"R-Wpn-Cannon-Damage09": 1621,
+	"R-Wpn-Howitzer-Damage03": 1624,
 	"R-Defense-GuardTower-ATMiss": 1626,
 	"R-Defense-WallTower-A-Tmiss": 1626,
 	"R-Cyborg-Metals08": 1629,
@@ -304,46 +288,42 @@ var allRes = {
 	"R-Sys-Autorepair-General": 1668,
 	"R-Wpn-EMPCannon": 1668,
 	"R-Struc-VTOLPad-Upgrade05": 1671,
-	"R-Wpn-Howitzer-Accuracy03": 1680,
 	"R-Wpn-Bomb05": 1680,
-	"R-Wpn-HvyHowitzer": 1692,
+	"R-Wpn-Howitzer-Accuracy03": 1681,
+	"R-Wpn-HvyHowitzer": 1693,
 	"R-Defense-PrisLas": 1705,
 	"R-Wpn-Energy-Accuracy01": 1705,
 	"R-Cyborg-Armor-Heat06": 1705,
-	"R-Wpn-AAGun-ROF05": 1707,
 	"R-Vehicle-Armor-Heat05": 1715,
-	"R-Wpn-Rail-Damage01": 1719,
-	"R-Wpn-MdArtMissile": 1738,
+	"R-Wpn-Rail-Damage01": 1715,
 	"R-Wpn-Missile-LtSAM": 1738,
 	"R-Wpn-Missile-ROF01": 1738,
 	"R-Struc-RprFac-Upgrade06": 1747,
 	"R-Sys-Sensor-WSTower": 1753,
 	"R-Vehicle-Engine07": 1754,
 	"R-Struc-Power-Upgrade03a": 1768,
-	"R-Wpn-AAGun-Damage06": 1774,
-	"R-Defense-EMPCannon": 1778,
 	"R-Defense-WallUpgrade07": 1778,
-	"R-Defense-HvyHowitzer": 1783,
+	"R-Defense-EMPCannon": 1778,
+	"R-Defense-HvyHowitzer": 1785,
 	"R-Vehicle-Metals08": 1793,
 	"R-Wpn-Flamer-Damage08": 1808,
+	"R-Wpn-Cannon-ROF06": 1824,
 	"R-Struc-Factory-Upgrade07": 1826,
-	"R-Wpn-Cannon-ROF06": 1828,
+	"R-Wpn-MdArtMissile": 1853,
 	"R-Defense-SamSite1": 1857,
 	"R-Cyborg-Metals09": 1868,
 	"R-Sys-SpyTurret": 1881,
 	"R-Struc-VTOLPad-Upgrade06": 1883,
 	"R-Wpn-MortarEMP": 1887,
-	"R-Wpn-AAGun-ROF06": 1889,
-	"R-Wpn-Howitzer-Damage04": 1911,
-	"R-Wpn-Howitzer-ROF01": 1911,
+	"R-Wpn-Howitzer-Damage04": 1912,
+	"R-Wpn-Howitzer-ROF01": 1912,
 	"R-Cyborg-Armor-Heat07": 1917,
 	"R-Wpn-Laser02": 1923,
-	"R-Wpn-Energy-Damage01": 1925,
-	"R-Wpn-Rail-Accuracy01": 1937,
-	"R-Vehicle-Armor-Heat06": 1942,
+	"R-Wpn-Rail-Accuracy01": 1933,
+	"R-Wpn-Energy-Damage01": 1956,
 	"R-Wpn-Missile-Damage01": 1956,
-	"R-Struc-Materials07": 1960,
-	"R-Defense-MdArtMissile": 1966,
+	"R-Struc-Materials03": 1960,
+	"R-Vehicle-Armor-Heat06": 1975,
 	"R-Sys-SpyTower": 1990,
 	"R-Defense-WallUpgrade08": 1990,
 	"R-Defense-EMPMortar": 1996,
@@ -354,66 +334,65 @@ var allRes = {
 	"R-Wpn-Bomb06": 2047,
 	"R-Vehicle-Metals09": 2060,
 	"R-Wpn-Flamer-Damage09": 2069,
+	"R-Defense-MdArtMissile": 2071,
 	"R-Sys-Resistance-Circuits": 2099,
 	"R-Struc-Factory-Upgrade09": 2117,
 	"R-Defense-PulseLas": 2141,
 	"R-Defense-WallTower-PulseLas": 2141,
-	"R-Wpn-Energy-ROF01": 2144,
-	"R-Wpn-Rail-Damage02": 2156,
-	"R-Wpn-Rail-ROF01": 2156,
+	"R-Wpn-Rail-Damage02": 2151,
+	"R-Wpn-Rail-ROF01": 2152,
 	"R-Cyborg-Armor-Heat08": 2160,
-	"R-Struc-Materials08": 2172,
+	"R-Wpn-Energy-ROF01": 2174,
 	"R-Wpn-Missile-ROF02": 2175,
 	"R-Wpn-Missile-Accuracy01": 2175,
-	"R-Vehicle-Armor-Heat07": 2200,
+	"R-Vehicle-Armor-Heat07": 2232,
 	"R-Defense-WallUpgrade09": 2233,
 	"R-Vehicle-Engine08": 2239,
 	"R-Wpn-HeavyPlasmaLauncher": 2244,
-	"R-Wpn-Howitzer-ROF02": 2347,
+	"R-Wpn-Howitzer-ROF02": 2349,
 	"R-Wpn-Howitzer-Damage05": 2354,
-	"R-Wpn-Energy-Damage02": 2362,
+	"R-Wpn-Energy-Damage02": 2393,
 	"R-Wpn-Missile-Damage02": 2393,
-	"R-Struc-Materials09": 2415,
 	"R-Cyborg-Armor-Heat09": 2433,
-	"R-Wpn-HvyLaser": 2471,
-	"R-Vehicle-Armor-Heat08": 2488,
 	"R-Vehicle-Body10": 2497,
+	"R-Wpn-HvyLaser": 2502,
 	"R-Defense-WallUpgrade10": 2506,
-	"R-Wpn-AALaser": 2580,
-	"R-Wpn-Energy-ROF02": 2580,
-	"R-Wpn-RailGun02": 2592,
-	"R-Wpn-Rail-ROF02": 2592,
+	"R-Vehicle-Armor-Heat08": 2520,
+	"R-Wpn-Rail-ROF02": 2588,
+	"R-Wpn-RailGun02": 2596,
+	"R-Wpn-Energy-ROF02": 2611,
 	"R-Wpn-Missile-Accuracy02": 2611,
-	"R-Cyborg-Hvywpn-RailGunner": 2676,
+	"R-Wpn-AALaser": 2669,
+	"R-Cyborg-Hvywpn-RailGunner": 2680,
 	"R-Defense-HeavyPlasmaLauncher": 2681,
-	"R-Defense-Rail2": 2744,
+	"R-Defense-Rail2": 2748,
 	"R-Vehicle-Engine09": 2755,
-	"R-Defense-HeavyLas": 2780,
-	"R-Vehicle-Armor-Heat09": 2806,
-	"R-Defense-AA-Laser": 2811,
-	"R-Defense-WallTower-Rail2": 2811,
+	"R-Defense-HeavyLas": 2811,
+	"R-Defense-WallTower-Rail2": 2815,
+	"R-Wpn-Missile-ROF03": 2829,
 	"R-Wpn-HvArtMissile": 2830,
+	"R-Vehicle-Armor-Heat09": 2839,
 	"R-Defense-WallUpgrade11": 2839,
-	"R-Wpn-Missile-ROF03": 2854,
+	"R-Defense-AA-Laser": 2887,
 	"R-Wpn-LasSat": 2890,
-	"R-Wpn-Howitzer-ROF03": 3002,
+	"R-Wpn-Howitzer-ROF03": 3003,
 	"R-Wpn-Howitzer-Damage06": 3009,
-	"R-Wpn-Energy-Damage03": 3017,
+	"R-Wpn-Energy-Damage03": 3047,
 	"R-Wpn-Missile-Damage03": 3048,
 	"R-Vehicle-Body13": 3191,
 	"R-Defense-WallUpgrade12": 3203,
-	"R-Wpn-Energy-ROF03": 3235,
-	"R-Wpn-Rail-Damage03": 3247,
-	"R-Wpn-Rail-ROF03": 3247,
+	"R-Wpn-Rail-ROF03": 3243,
+	"R-Wpn-Rail-Damage03": 3251,
+	"R-Wpn-Energy-ROF03": 3266,
 	"R-Defense-HvyArtMissile": 3266,
 	"R-Wpn-Missile-HvSAM": 3484,
-	"R-Defense-Super-Missile": 3612,
+	"R-Defense-Super-Missile": 3596,
 	"R-Defense-SamSite2": 3703,
 	"R-Defense-WallTower-SamHvy": 3757,
 	"R-Vehicle-Body14": 3846,
-	"R-Wpn-Howitzer-ROF04": 3860,
-	"R-Wpn-RailGun03": 3902,
-	"R-Defense-Rail3": 4054,
-	"R-Defense-WallTower-Rail3": 4120,
-	"R-Defense-MassDriver": 4660
+	"R-Wpn-Howitzer-ROF04": 3861,
+	"R-Wpn-RailGun03": 3906,
+	"R-Defense-Rail3": 4058,
+	"R-Defense-WallTower-Rail3": 4124,
+	"R-Defense-MassDriver": 4664
 };
