@@ -3,10 +3,10 @@
 // Base Under Attack
 function eventAttacked(victimObj, attackerObj)
 {
-	if (gameTime > lastHitTime + 5000 && victimObj.player == selectedPlayer)
+	if (gameTime > lastHitTime + 5000 && victimObj.player === selectedPlayer)
 	{
 		lastHitTime = gameTime;
-		if (victimObj.type == STRUCTURE)
+		if (victimObj.type === STRUCTURE)
 		{
 			playSound("pcv337.ogg", victimObj.x, victimObj.y, victimObj.z);	// show position if still alive
 		}
