@@ -26,6 +26,7 @@
 #include <cmath>
 #include <functional>
 #include <typeindex>
+#include <array>
 
 namespace gfx_api
 {
@@ -59,16 +60,23 @@ struct gl_texture final : public gfx_api::texture
 private:
 	friend struct gl_context;
 	GLuint _id;
-	size_t mip_count;
+	size_t mip_count = 0;
+	gfx_api::pixel_format internal_format = gfx_api::pixel_format::invalid;
+	bool gles = false;
+#if defined(WZ_DEBUG_GFX_API_LEAKS)
+	std::string debugName;
+#endif
 
 	gl_texture();
 	virtual ~gl_texture();
 public:
 	virtual void bind() override;
 	void unbind();
-	virtual void upload(const size_t& mip_level, const size_t& offset_x, const size_t& offset_y, const size_t & width, const size_t & height, const gfx_api::pixel_format & buffer_format, const void * data) override;
-	virtual void upload_and_generate_mipmaps(const size_t& offset_x, const size_t& offset_y, const size_t& width, const size_t& height, const  gfx_api::pixel_format& buffer_format, const void* data) override;
+	virtual bool upload(const size_t& mip_level, const iV_BaseImage& image) override;
+	virtual bool upload_sub(const size_t& mip_level, const size_t& offset_x, const size_t& offset_y, const iV_Image& image) override;
 	virtual unsigned id() override;
+private:
+	virtual bool upload_internal(const size_t& mip_level, const size_t& offset_x, const size_t& offset_y, const iV_BaseImage& image);
 };
 
 struct gl_buffer final : public gfx_api::buffer
@@ -105,7 +113,7 @@ struct gl_pipeline_state_object final : public gfx_api::pipeline_state_object
 	template<typename T>
 	typename std::pair<std::type_index, std::function<void(const void*, size_t)>> uniform_setting_func();
 
-	gl_pipeline_state_object(bool gles, bool fragmentHighpFloatAvailable, bool fragmentHighpIntAvailable, const gfx_api::state_description& _desc, const SHADER_MODE& shader, const std::vector<std::type_index>& uniform_blocks, const std::vector<gfx_api::vertex_buffer>& vertex_buffer_desc);
+	gl_pipeline_state_object(bool gles, bool fragmentHighpFloatAvailable, bool fragmentHighpIntAvailable, bool patchFragmentShaderMipLodBias, const gfx_api::state_description& _desc, const SHADER_MODE& shader, const std::vector<std::type_index>& uniform_blocks, const std::vector<gfx_api::vertex_buffer>& vertex_buffer_desc, optional<float> mipLodBias);
 	void set_constants(const void* buffer, const size_t& size);
 	void set_uniforms(const size_t& first, const std::vector<std::tuple<const void*, size_t>>& uniform_blocks);
 
@@ -123,11 +131,11 @@ private:
 
 	void getLocs();
 
-	void build_program(bool fragmentHighpFloatAvailable, bool fragmentHighpIntAvailable,
+	void build_program(bool fragmentHighpFloatAvailable, bool fragmentHighpIntAvailable, bool patchFragmentShaderMipLodBias,
 					   const std::string& programName,
 					   const char * vertex_header, const std::string& vertexPath,
 					   const char * fragment_header, const std::string& fragmentPath,
-					   const std::vector<std::string> &uniformNames);
+					   const std::vector<std::string> &uniformNames, optional<float> mipLodBias);
 
 	void fetch_uniforms(const std::vector<std::string>& uniformNames, const std::vector<std::string>& duplicateFragmentUniforms);
 
@@ -138,8 +146,8 @@ private:
 	 */
 	void setUniforms(size_t uniformIdx, const ::glm::vec4 &v);
 	void setUniforms(size_t uniformIdx, const ::glm::mat4 &m);
-	void setUniforms(size_t uniformIdx, const Vector2i &v);
-	void setUniforms(size_t uniformIdx, const Vector2f &v);
+	void setUniforms(size_t uniformIdx, const ::glm::ivec2 &v);
+	void setUniforms(size_t uniformIdx, const ::glm::vec2 &v);
 	void setUniforms(size_t uniformIdx, const int32_t &v);
 	void setUniforms(size_t uniformIdx, const float &v);
 
@@ -171,6 +179,7 @@ struct gl_context final : public gfx_api::context
 	GLuint scratchbuffer = 0;
 	size_t scratchbuffer_size = 0;
 	bool khr_debug = false;
+	optional<float> mipLodBias;
 
 	bool gles = false;
 	bool fragmentHighpFloatAvailable = true;
@@ -195,7 +204,7 @@ struct gl_context final : public gfx_api::context
 	virtual void unbind_vertex_buffers(const std::size_t& first, const std::vector<std::tuple<gfx_api::buffer*, std::size_t>>& vertex_buffers_offset) override;
 	virtual void disable_all_vertex_buffers() override;
 	virtual void bind_streamed_vertex_buffers(const void* data, const std::size_t size) override;
-	virtual void bind_textures(const std::vector<gfx_api::texture_input>& texture_descriptions, const std::vector<gfx_api::texture*>& textures) override;
+	virtual void bind_textures(const std::vector<gfx_api::texture_input>& texture_descriptions, const std::vector<gfx_api::abstract_texture*>& textures) override;
 	virtual void set_constants(const void* buffer, const size_t& size) override;
 	virtual void set_uniforms(const size_t& first, const std::vector<std::tuple<const void*, size_t>>& uniform_blocks) override;
 	virtual void draw(const size_t& offset, const size_t &count, const gfx_api::primitive_type &primitive) override;
@@ -204,7 +213,8 @@ struct gl_context final : public gfx_api::context
 	virtual void set_depth_range(const float& min, const float& max) override;
 	virtual int32_t get_context_value(const context_value property) override;
 
-	virtual void flip(int clearMode) override;
+	virtual void beginRenderPass() override;
+	virtual void endRenderPass() override;
 	virtual void debugStringMarker(const char *str) override;
 	virtual void debugSceneBegin(const char *descr) override;
 	virtual void debugSceneEnd(const char *descr) override;
@@ -218,12 +228,18 @@ struct gl_context final : public gfx_api::context
 	virtual const std::string& getFormattedRendererInfoString() const override;
 	virtual bool getScreenshot(std::function<void (std::unique_ptr<iV_Image>)> callback) override;
 	virtual void handleWindowSizeChange(unsigned int oldWidth, unsigned int oldHeight, unsigned int newWidth, unsigned int newHeight) override;
+	virtual std::pair<uint32_t, uint32_t> getDrawableDimensions() override;
+	virtual bool shouldDraw() override;
 	virtual void shutdown() override;
 	virtual const size_t& current_FrameNum() const override;
 	virtual bool setSwapInterval(gfx_api::context::swap_interval_mode mode) override;
 	virtual gfx_api::context::swap_interval_mode getSwapInterval() const override;
+	virtual bool textureFormatIsSupported(gfx_api::pixel_format_target target, gfx_api::pixel_format format, gfx_api::pixel_format_usage::flags usage) override;
+	virtual bool supportsMipLodBias() const override;
 private:
-	virtual bool _initialize(const gfx_api::backend_Impl_Factory& impl, int32_t antialiasing, swap_interval_mode mode) override;
+	virtual bool _initialize(const gfx_api::backend_Impl_Factory& impl, int32_t antialiasing, swap_interval_mode mode, optional<float> mipLodBias) override;
+	void initPixelFormatsSupport();
+	void _beginRenderPassImpl();
 private:
 	bool initGLContext();
 	void enableVertexAttribArray(GLuint index);
@@ -231,7 +247,14 @@ private:
 	std::string calculateFormattedRendererInfoString() const;
 	bool isBlocklistedGraphicsDriver() const;
 
+	uint32_t viewportWidth = 0;
+	uint32_t viewportHeight = 0;
 	std::vector<bool> enabledVertexAttribIndexes;
 	size_t frameNum = 0;
 	std::string formattedRendererInfoString;
+	std::array<std::vector<gfx_api::pixel_format_usage::flags>, gfx_api::PIXEL_FORMAT_TARGET_COUNT> textureFormatsSupport;
+	bool has2DTextureArraySupport = false;
+	int32_t maxArrayTextureLayers = 0;
+	GLfloat maxTextureAnisotropy = 0.f;
+	GLuint vaoId = 0;
 };
