@@ -96,14 +96,12 @@ static bool tryLoad(const WzString &path, const WzString &filename)
 
 const std::string &modelName(iIMDShape *model)
 {
-	for (const auto &pair : models)
+	if (model != nullptr)
 	{
-		if (&pair.second == model)
-		{
-			return pair.first;
-		}
+		ASSERT(!model->modelName.empty(), "An IMD pointer could not be backtraced to a filename!");
+		return model->modelName;
 	}
-	ASSERT(false, "An IMD pointer could not be backtraced to a filename!");
+	ASSERT(false, "Null IMD pointer??");
 	static std::string error;
 	return error;
 }
@@ -729,6 +727,7 @@ void finishTangentsGeneration()
  * \pre ppFileData loaded
  * \post s allocated
  */
+static_assert(PATH_MAX >= 255, "PATH_MAX is insufficient!");
 static iIMDShape *_imd_load_level(const WzString &filename, const char **ppFileData, const char *FileDataEnd, int nlevels, int pieVersion, int level)
 {
 	const char *pFileData = *ppFileData;
@@ -749,6 +748,8 @@ static iIMDShape *_imd_load_level(const WzString &filename, const char **ppFileD
 	}
 	ASSERT(models.count(key) == 0, "Duplicate model load for %s!", key.c_str());
 	iIMDShape &s = models[key]; // create entry and return reference
+	s.modelName = key;
+	s.modelLevel = level;
 	s.pShadowPoints = &s.points;
 	s.pShadowPolys = &s.polys;
 
@@ -869,9 +870,9 @@ static iIMDShape *_imd_load_level(const WzString &filename, const char **ppFileD
 				s.objanimdata[i].pos.x = pos.x / INT_SCALE;
 				s.objanimdata[i].pos.y = pos.z / INT_SCALE;
 				s.objanimdata[i].pos.z = pos.y / INT_SCALE;
-				s.objanimdata[i].rot.pitch = static_cast<uint16_t>(-(rot.x * DEG_1 / INT_SCALE));
-				s.objanimdata[i].rot.direction = static_cast<uint16_t>(-(rot.z * DEG_1 / INT_SCALE));
-				s.objanimdata[i].rot.roll = static_cast<uint16_t>(-(rot.y * DEG_1 / INT_SCALE));
+				s.objanimdata[i].rot.pitch = static_cast<uint16_t>(static_cast<int32_t>(-(rot.x * DEG_1 / INT_SCALE)));
+				s.objanimdata[i].rot.direction = static_cast<uint16_t>(static_cast<int32_t>(-(rot.z * DEG_1 / INT_SCALE)));
+				s.objanimdata[i].rot.roll = static_cast<uint16_t>(static_cast<int32_t>(-(rot.y * DEG_1 / INT_SCALE)));
 				pFileData += cnt;
 			}
 		}
@@ -1249,7 +1250,7 @@ static void iV_ProcessIMD(const WzString &filename, const char **ppFileData, con
 	// load texture page if specified
 	if (bTextured)
 	{
-		optional<size_t> texpage = iV_GetTexture(texfile);
+		optional<size_t> texpage = iV_GetTexture(texfile, gfx_api::texture_type::game_texture);
 		optional<size_t> normalpage;
 		optional<size_t> specpage;
 
@@ -1258,42 +1259,14 @@ static void iV_ProcessIMD(const WzString &filename, const char **ppFileData, con
 		if (normalfile[0] != '\0')
 		{
 			debug(LOG_TEXTURE, "Loading normal map %s for %s", normalfile, filename.toUtf8().c_str());
-			normalpage = iV_GetTexture(normalfile, false);
+			normalpage = iV_GetTexture(normalfile, gfx_api::texture_type::normal_map);
 			ASSERT_OR_RETURN(, normalpage.has_value(), "%s could not load tex page %s", filename.toUtf8().c_str(), normalfile);
 		}
 
 		if (specfile[0] != '\0')
 		{
 			debug(LOG_TEXTURE, "Loading specular map %s for %s", specfile, filename.toUtf8().c_str());
-			specpage = iV_GetTransformTexture(specfile, [filename, specfile](iV_Image& sSprite){
-				if (sSprite.depth == 1) { return; }
-				// Otherwise, expecting 3 or 4-channel (RGB/RGBA)
-				ASSERT_OR_RETURN(, sSprite.depth == 3 || sSprite.depth == 4, "(%s): Does not have 1, 3 or 4 channels", specfile);
-				auto originalBmpData = sSprite.bmp;
-				const size_t numPixels = static_cast<size_t>(sSprite.height) * static_cast<size_t>(sSprite.width);
-				sSprite.bmp = (unsigned char *)malloc(numPixels);
-				for (size_t pixelIdx = 0; pixelIdx < numPixels; pixelIdx++)
-				{
-					uint32_t red = originalBmpData[(pixelIdx * sSprite.depth)];
-					uint32_t green = originalBmpData[(pixelIdx * sSprite.depth) + 1];
-					uint32_t blue = originalBmpData[(pixelIdx * sSprite.depth) + 2];
-					if (red == green && red == blue)
-					{
-						// all channels are the same - just use the first channel
-						sSprite.bmp[pixelIdx] = static_cast<unsigned char>(red);
-					}
-					else
-					{
-						// quick approximation of a weighted RGB -> Luma method
-						// (R+R+B+G+G+G)/6
-						uint32_t lum = (red+red+blue+green+green+green) / 6;
-						sSprite.bmp[pixelIdx] = static_cast<unsigned char>(std::min<uint32_t>(lum, 255));
-					}
-				}
-				sSprite.depth = 1;
-				// free the original bitmap data
-				free(originalBmpData);
-			});
+			specpage = iV_GetTexture(specfile, gfx_api::texture_type::specular_map);
 			ASSERT_OR_RETURN(, specpage.has_value(), "%s could not load tex page %s", filename.toUtf8().c_str(), specfile);
 		}
 
@@ -1312,20 +1285,7 @@ static void iV_ProcessIMD(const WzString &filename, const char **ppFileData, con
 		{
 			std::string tcmask_name = pie_MakeTexPageTCMaskName(texfile);
 			tcmask_name += ".png";
-			optional<size_t> texpage_mask = iV_GetTransformTexture(tcmask_name.c_str(), [filename, tcmask_name](iV_Image& sSprite){
-				ASSERT_OR_RETURN(, sSprite.depth == 4, "(%s) tcmask png (%s) does not have 4 channels, as expected", filename.toUtf8().c_str(), tcmask_name.c_str());
-				auto originalBmpData = sSprite.bmp;
-				const size_t numPixels = static_cast<size_t>(sSprite.height) * static_cast<size_t>(sSprite.width);
-				// copy just the alpha channel over
-				sSprite.bmp = (unsigned char *)malloc(numPixels);
-				for (size_t pixelIdx = 0; pixelIdx < numPixels; pixelIdx++)
-				{
-					sSprite.bmp[pixelIdx] = originalBmpData[(pixelIdx * 4) + 3];
-				}
-				sSprite.depth = 1;
-				// free the original bitmap data
-				free(originalBmpData);
-			});
+			optional<size_t> texpage_mask = iV_GetTexture(tcmask_name.c_str(), gfx_api::texture_type::alpha_mask);
 
 			ASSERT_OR_RETURN(, texpage_mask.has_value(), "%s could not load tcmask %s", filename.toUtf8().c_str(), tcmask_name.c_str());
 
