@@ -26,7 +26,11 @@
 #include "lib/ivis_opengl/tex.h"
 #include "src/warzoneconfig.h"
 #include "cursors_sdl.h"
-#include <SDL.h>
+#include <SDL_mouse.h>
+#include <SDL_surface.h>
+#include <SDL_hints.h>
+#include "sdl_backend_private.h"
+#include "cursor_sdl_helpers.h"
 
 static CURSOR currentCursor = CURSOR_MAX;
 static CURSOR lastAppliedCursor = CURSOR_MAX;
@@ -1259,24 +1263,34 @@ static const struct
 	{ cursor_select,        CURSOR_SELECT },
 };
 
-/**
-	init_system_ColorCursor()-- Create a colored mouse cursor image
- */
-SDL_Cursor *init_system_ColorCursor(CURSOR cur, const char *fileName)
+static void scaleCursorImageForUpload(iV_Image& cursorImage, int& hot_x, int& hot_y)
 {
-
-	iV_Image *psSprite = (iV_Image *)malloc(sizeof(iV_Image));
-	if (!psSprite)
+#if defined(WZ_OS_WIN)
+	if (SDL_GetHintBoolean(SDL_HINT_WINDOWS_DPI_SCALING, SDL_FALSE) == SDL_TRUE)
 	{
-		debug(LOG_FATAL, "Could not allocate memory for cursor sprite. Exiting.");
-		exit(-1);
+		// Must manually scale the cursor image based on the window scale factor (in Windows w/ SDL 2.24.0+)
+		float horizWindowScaleFactor = 0.f, vertWindowScaleFactor = 0.f;
+		wzGetWindowToRendererScaleFactor(&horizWindowScaleFactor, &vertWindowScaleFactor);
+		assert(horizWindowScaleFactor != 0.f);
+		assert(vertWindowScaleFactor != 0.f);
+		unsigned int scaledWidth = static_cast<unsigned int>(cursorImage.width() * horizWindowScaleFactor);
+		unsigned int scaledHeight = static_cast<unsigned int>(cursorImage.height() * vertWindowScaleFactor);
+		if (scaledWidth > cursorImage.width() && scaledHeight > cursorImage.height())
+		{
+			debug(LOG_GUI, "Scaling cursor image from (%u x %u) to (%u x %u)", cursorImage.width(), cursorImage.height(), scaledWidth, scaledHeight);
+			cursorImage.resize(scaledWidth, scaledHeight);
+			hot_x = static_cast<int>(hot_x * horizWindowScaleFactor);
+			hot_y = static_cast<int>(hot_y * vertWindowScaleFactor);
+		}
 	}
+#else
+	// no-op
+#endif
+}
 
-	if (!iV_loadImage_PNG(fileName, psSprite))
-	{
-		debug(LOG_FATAL, "Could not load cursor sprite. Exiting.");
-		exit(-1);
-	}
+SDL_Cursor* init_cursor_from_image(iV_Image&& sprite, int hot_x, int hot_y)
+{
+	scaleCursorImageForUpload(sprite, hot_x, hot_y);
 
 #if SDL_BYTEORDER == SDL_BIG_ENDIAN
 	uint32_t rmask = 0xff000000;
@@ -1290,13 +1304,13 @@ SDL_Cursor *init_system_ColorCursor(CURSOR cur, const char *fileName)
 	uint32_t amask = 0xff000000;
 #endif
 
-	SDL_Surface *surface = SDL_CreateRGBSurfaceFrom(psSprite->bmp, psSprite->width, psSprite->height, psSprite->depth * 8, psSprite->width * 4, rmask, gmask, bmask, amask);
+	SDL_Surface *surface = SDL_CreateRGBSurfaceFrom(sprite.bmp_w(), sprite.width(), sprite.height(), sprite.depth(), sprite.width() * 4, rmask, gmask, bmask, amask);
 	if (!surface)
 	{
 		debug(LOG_FATAL, "Failed to create cursor because: %s", SDL_GetError());
 		return nullptr;
 	}
-	SDL_Cursor *pointer = SDL_CreateColorCursor(surface, psSprite->width / 2, psSprite->height / 2);	// We center the hotspot for all (FIXME ?)
+	SDL_Cursor *pointer = SDL_CreateColorCursor(surface, hot_x, hot_y);
 	if (!pointer)
 	{
 		debug(LOG_FATAL, "Could not create cursor because %s", SDL_GetError());
@@ -1305,12 +1319,30 @@ SDL_Cursor *init_system_ColorCursor(CURSOR cur, const char *fileName)
 
 	// free up surface & image data
 	SDL_FreeSurface(surface);
-	free(psSprite->bmp);
-	free(psSprite);
+	sprite.clear();
 
 	return pointer;
 }
 
+/**
+	init_system_ColorCursor()-- Create a colored mouse cursor image
+ */
+SDL_Cursor *init_system_ColorCursor(CURSOR cur, const char *fileName)
+{
+	iV_Image sprite;
+	if (!iV_loadImage_PNG2(fileName, sprite, true))
+	{
+		debug(LOG_FATAL, "Could not load cursor sprite. Exiting.");
+		exit(-1);
+	}
+
+	// We center the hotspot for all (FIXME ?)
+	int hot_x = sprite.width() / 2;
+	int hot_y = sprite.height() / 2;
+
+	return init_cursor_from_image(std::move(sprite), hot_x, hot_y);
+}
+
 /**
 	init_system_cursor32()-- Create a monochrome mouse cursor image
  */
@@ -1358,7 +1390,15 @@ SDL_Cursor *init_system_cursor32(CURSOR cur)
 	}
 
 	sscanf(image[4 + row], "%d,%d", &hot_x, &hot_y);
-	return SDL_CreateCursor(data, mask, 32, 32, hot_x, hot_y);
+
+	iV_Image cursorImage;
+	if (!ivImageFromMonoCursorDataSDLCompat(data, mask, 32, 32, cursorImage))
+	{
+		debug(LOG_FATAL, "Failed to convert data to cursor!");
+		abort();
+	}
+
+	return init_cursor_from_image(std::move(cursorImage), hot_x, hot_y);
 }
 
 /**
