@@ -51,6 +51,7 @@
 #include <unordered_set>
 #include <deque>
 
+static	bool	bWidgetsInitialized = false;
 static	bool	bWidgetsActive = true;
 
 /* The widget the mouse is over this update */
@@ -134,6 +135,7 @@ static inline void _widgDebugAssertIfRunningScreen(const char *function)
 /* Initialise the widget module */
 bool widgInitialise()
 {
+	bWidgetsInitialized = true;
 	tipInitialise();
 	return true;
 }
@@ -169,6 +171,7 @@ void widgShutDown(void)
 		ASSERT(debugLiveWidgets.empty(), "There are %zu widgets that were not cleaned up.", debugLiveWidgets.size());
 	}
 #endif
+	bWidgetsInitialized = false;
 }
 
 void widgScheduleTask(std::function<void ()> task)
@@ -602,7 +605,10 @@ W_SCREEN::~W_SCREEN()
 	if (auto focusedWidget = psFocus.lock())
 	{
 		// must trigger a resignation of focus
-		focusedWidget->focusLost();
+		if (bWidgetsInitialized) // do not call focusLost if widgets have already shutdown / are not initialized
+		{
+			focusedWidget->focusLost();
+		}
 	}
 	psFocus.reset();
 }
@@ -1405,6 +1411,8 @@ void widgDisplayScreen(const std::shared_ptr<W_SCREEN> &psScreen)
 	debugLoc = debugLoc[1] == -1 ? debugSequence : debugLoc[0] == debugCode ? debugLoc : debugLoc[1] == debugCode ? debugLoc + 1 : debugSequence;
 	debugBoundingBoxes = debugBoundingBoxes ^ (debugLoc[1] == -1);
 
+	bool skipDrawing = !gfx_api::context::get().shouldDraw();
+
 	cleanupDeletedOverlays();
 
 	/* Process any user callback functions */
@@ -1413,23 +1421,32 @@ void widgDisplayScreen(const std::shared_ptr<W_SCREEN> &psScreen)
 	sContext.my = mouseY();
 	psScreen->psForm->processCallbacksRecursive(&sContext);
 
-	// Display the widgets.
-	psScreen->psForm->displayRecursive();
+	if (!skipDrawing)
+	{
+		// Display the widgets.
+		psScreen->psForm->displayRecursive();
+	}
 
 	// Always overlays on-top (i.e. draw them last)
-	forEachOverlayScreenBottomUp([&sContext](const OverlayScreen& overlay) -> bool
+	forEachOverlayScreenBottomUp([&sContext, skipDrawing](const OverlayScreen& overlay) -> bool
 	{
 		overlay.psScreen->psForm->processCallbacksRecursive(&sContext);
-		overlay.psScreen->psForm->displayRecursive();
+		if (!skipDrawing)
+		{
+			overlay.psScreen->psForm->displayRecursive();
+		}
 		return true;
 	}); // <- enumerate in *increasing* z-order for drawing
 
 	deleteOldWidgets();  // Delete any widgets that called deleteLater() while being displayed.
 
-	/* Display the tool tip if there is one */
-	tipDisplay();
+	if (!skipDrawing)
+	{
+		/* Display the tool tip if there is one */
+		tipDisplay();
+	}
 
-	if (debugBoundingBoxes)
+	if (debugBoundingBoxes && !skipDrawing)
 	{
 		debugBoundingBoxesOnly = true;
 		psScreen->psForm->displayRecursive();
@@ -1441,7 +1458,10 @@ void W_SCREEN::setFocus(const std::shared_ptr<WIDGET> &widget)
 {
 	if (auto locked = psFocus.lock())
 	{
-		locked->focusLost();
+		if (bWidgetsInitialized) // do not call focusLost if widgets have already shutdown / are not initialized
+		{
+			locked->focusLost();
+		}
 	}
 	psFocus = widget;
 }
