@@ -25,6 +25,7 @@
 
 #include "lib/framework/wzconfig.h"
 #include "lib/framework/input.h"
+#include "lib/framework/physfs_ext.h"
 #include "lib/netplay/netplay.h"
 #include "lib/sound/mixer.h"
 #include "lib/sound/sounddefs.h"
@@ -39,6 +40,7 @@
 #include "ingameop.h"
 #include "multiint.h"
 #include "multiplay.h"
+#include "multistat.h"
 #include "radar.h"
 #include "seqdisp.h"
 #include "texture.h"
@@ -63,6 +65,8 @@
 
 #define MASTERSERVERPORT	9990
 #define GAMESERVERPORT		2100
+#define BASECONFVERSION		1
+#define CURRCONFVERSION		2
 
 static const char *fileName = "config";
 
@@ -116,6 +120,24 @@ public:
 		}
 		return PHYSFS_exists(utf8Path.c_str());
 	}
+	optional<uint64_t> fileSize() const
+	{
+#if defined(WZ_PHYSFS_2_1_OR_GREATER)
+		PHYSFS_Stat metaData;
+		if (PHYSFS_stat(utf8Path.c_str(), &metaData) == 0)
+		{
+			return nullopt; // failed to get file info
+		}
+		if (metaData.filesize < 0)
+		{
+			// unknown filesize
+			return nullopt;
+		}
+		return static_cast<uint64_t>(metaData.filesize);
+#else
+		return nullopt; // unknown
+#endif
+	}
 	std::string realPath() const
 	{
 		std::string fullPath = WZ_PHYSFS_getRealDir_String(utf8Path.c_str());
@@ -230,6 +252,8 @@ bool saveIniFile(mINI::INIFile &file, mINI::INIStructure &ini)
 	return true;
 }
 
+constexpr uint64_t MAX_CONFIG_FILE_SIZE = 1024 * 1024 * 2; // 2 MB seems like enough...
+
 // ////////////////////////////////////////////////////////////////////////////
 bool loadConfig()
 {
@@ -239,20 +263,31 @@ bool loadConfig()
 
 	// next, create a structure that will hold data
 	mINI::INIStructure ini;
+	bool createdConfigFile = false;
 
 	// now we can read the file
 	try
 	{
-		if (!file.read(ini))
+		uint64_t fileSize = fileStreamGenerator->fileSize().value_or(0);
+		if (fileSize > MAX_CONFIG_FILE_SIZE)
 		{
+			createdConfigFile = true;
+			debug(LOG_ERROR, "Could not read existing configuration file \"%s\"; filesize (%" PRIu64 ") exceeds max", fileName, fileSize);
+			// will just proceed with an empty ini structure
+		}
+		if (!createdConfigFile && !file.read(ini))
+		{
+			createdConfigFile = true;
 			debug(LOG_WZ, "Could not read existing configuration file \"%s\"", fileName);
 			// will just proceed with an empty ini structure
 		}
 	}
 	catch (const std::exception& e)
 	{
+		createdConfigFile = true;
 		debug(LOG_ERROR, "Ini read failed with exception: %s", e.what());
-		return false;
+		ini.clear();
+		// will just proceed with an empty ini structure
 	}
 
 	auto& iniGeneral = ini["General"];
@@ -327,7 +362,7 @@ bool loadConfig()
 	{
 		setMouseWarp(value.value());
 	}
-	wz_texture_compression = false; // currently unsupported
+	wz_texture_compression = iniGetBool("textureCompression", true).value();
 	showFPS = iniGetBool("showFPS", false).value();
 	showUNITCOUNT = iniGetBool("showUNITCOUNT", false).value();
 	if (auto value = iniGetIntegerOpt("cameraSpeed"))
@@ -346,11 +381,16 @@ bool loadConfig()
 	{
 		war_SetRadarJump(value.value());
 	}
+	if (auto value = iniGetIntegerOpt("lodDistanceBias"))
+	{
+		war_setLODDistanceBiasPercentage(value.value());
+	}
 	rotateRadar = iniGetBool("rotateRadar", true).value();
 	radarRotationArrow = iniGetBool("radarRotationArrow", true).value();
 	hostQuitConfirmation = iniGetBool("hostQuitConfirmation", true).value();
 	war_SetPauseOnFocusLoss(iniGetBool("PauseOnFocusLoss", false).value());
-	setAutoratingUrl(iniGetString("autoratingUrl", "").value());
+	setAutoratingUrl(iniGetString("autoratingUrlV2", WZ_DEFAULT_PUBLIC_RATING_LOOKUP_SERVICE_URL).value());
+	setAutoratingEnable(iniGetBool("autorating", false).value());
 	NETsetMasterserverName(iniGetString("masterserver_name", "lobby.wz2100.net").value().c_str());
 	mpSetServerName(iniGetString("server_name", "").value().c_str());
 //	iV_font(ini.value("fontname", "DejaVu Sans").toString().toUtf8().constData(),
@@ -368,6 +408,19 @@ bool loadConfig()
 	war_setScanlineMode((SCANLINE_MODE)iniGetInteger("scanlines", SCANLINES_OFF).value());
 	seq_SetSubtitles(iniGetBool("subtitles", true).value());
 	setDifficultyLevel((DIFFICULTY_LEVEL)iniGetInteger("difficulty", DL_NORMAL).value());
+	if (!createdConfigFile && iniGetInteger("configVersion", BASECONFVERSION).value() < CURRCONFVERSION)
+	{
+		int level = (int)getDifficultyLevel() + 1;
+		if (level > static_cast<int>(AIDifficulty::INSANE))
+		{
+			level = static_cast<int>(AIDifficulty::INSANE);
+		}
+		else if (level < static_cast<int>(AIDifficulty::SUPEREASY))
+		{
+			level = static_cast<int>(AIDifficulty::SUPEREASY);
+		}
+		setDifficultyLevel(static_cast<DIFFICULTY_LEVEL>(level));
+	}
 	war_SetSPcolor(iniGetInteger("colour", 0).value());	// default is green (0)
 	war_setMPcolour(iniGetInteger("colourMP", -1).value());  // default is random (-1)
 	sstrcpy(game.name, iniGetString("gameName", _("My Game")).value().c_str());
@@ -437,11 +490,23 @@ bool loadConfig()
 	war_SetHeight(height);
 	war_SetScreen(screen);
 
+	int fullscreenWidth = iniGetInteger("fullscreenWidth", war_GetFullscreenModeWidth()).value();
+	int fullscreenHeight = iniGetInteger("fullscreenHeight", war_GetFullscreenModeHeight()).value();
+	int fullscreenScreen = iniGetInteger("fullscreenScreen", 0).value();
+	if ((fullscreenWidth != 0 && fullscreenWidth < 640) || (fullscreenHeight != 0 && fullscreenHeight < 480))	// sanity check
+	{
+		// set to special value (0x0) that reverts to the default for this display
+		fullscreenWidth = 0;
+		fullscreenHeight = 0;
+	}
+	war_SetFullscreenModeWidth(fullscreenWidth);
+	war_SetFullscreenModeHeight(fullscreenHeight);
+	war_SetFullscreenModeScreen(fullscreenScreen);
+
 	if (auto value = iniGetIntegerOpt("bpp"))
 	{
 		war_SetVideoBufferDepth(value.value());
 	}
-	setFavoriteStructs(WzString::fromUtf8(iniGetString("favoriteStructs", "").value()));
 
 	video_backend gfxBackend;
 	if (iniGeneral.has("gfxbackend"))
@@ -458,6 +523,14 @@ bool loadConfig()
 		gfxBackend = wzGetDefaultGfxBackendForCurrentSystem();
 	}
 	war_setGfxBackend(gfxBackend);
+	if (auto value = iniGetIntegerOpt("minimizeOnFocusLoss"))
+	{
+		war_setMinimizeOnFocusLoss(value.value());
+	}
+	if (auto value = iniGetIntegerOpt("altEnterToggleMode"))
+	{
+		war_setToggleFullscreenMode(value.value());
+	}
 	auto js_backend = war_getJSBackend();
 	if (iniGeneral.has("jsbackend"))
 	{
@@ -483,8 +556,12 @@ bool loadConfig()
 	}
 	war_setAutoLagKickSeconds(iniGetInteger("hostAutoLagKickSeconds", war_getAutoLagKickSeconds()).value());
 	war_setDisableReplayRecording(iniGetBool("disableReplayRecord", war_getDisableReplayRecording()).value());
+	war_setMaxReplaysSaved(iniGetInteger("maxReplaysSaved", war_getMaxReplaysSaved()).value());
+	war_setOldLogsLimit(iniGetInteger("oldLogsLimit", war_getOldLogsLimit()).value());
 	int openSpecSlotsIntValue = iniGetInteger("openSpectatorSlotsMP", war_getMPopenSpectatorSlots()).value();
 	war_setMPopenSpectatorSlots(static_cast<uint16_t>(std::max<int>(0, std::min<int>(openSpecSlotsIntValue, MAX_SPECTATOR_SLOTS))));
+	war_setFogEnd(iniGetInteger("fogEnd", 8000).value());
+	war_setFogStart(iniGetInteger("fogStart", 4000).value());
 	ActivityManager::instance().endLoadingSettings();
 	return true;
 }
@@ -547,12 +624,16 @@ bool saveConfig()
 	iniSetInteger("width", war_GetWidth());
 	iniSetInteger("height", war_GetHeight());
 	iniSetInteger("screen", war_GetScreen());
+	iniSetInteger("fullscreenWidth", war_GetFullscreenModeWidth());
+	iniSetInteger("fullscreenHeight", war_GetFullscreenModeHeight());
+	iniSetInteger("fullscreenScreen", war_GetFullscreenModeScreen());
 	iniSetInteger("bpp", war_GetVideoBufferDepth());
 	iniSetInteger("fullscreen", static_cast<typename std::underlying_type<WINDOW_MODE>::type>(war_getWindowMode()));
 	iniSetString("language", getLanguage());
 	iniSetInteger("difficulty", getDifficultyLevel());		// level
 	iniSetInteger("cameraSpeed", war_GetCameraSpeed());	// camera speed
 	iniSetBool("radarJump", war_GetRadarJump());		// radar jump
+	iniSetInteger("lodDistanceBias", war_getLODDistanceBiasPercentage());
 	iniSetBool("cameraAccel", getCameraAccel());		// camera acceleration
 	iniSetInteger("shake", (int)getShakeStatus());		// screenshake
 	iniSetInteger("mouseflip", (int)(getInvertMouseStatus()));	// flipmouse
@@ -560,6 +641,7 @@ bool saveConfig()
 	iniSetInteger("coloredCursor", (int)war_GetColouredCursor());
 	iniSetInteger("RightClickOrders", (int)(getRightClickOrders()));
 	iniSetInteger("MiddleClickRotate", (int)(getMiddleClickRotate()));
+	iniSetInteger("textureCompression", (wz_texture_compression) ? 1 : 0);
 	iniSetInteger("showFPS", (int)showFPS);
 	iniSetInteger("showUNITCOUNT", (int)showUNITCOUNT);
 	iniSetInteger("shadows", (int)(getDrawShadows()));	// shadows
@@ -580,7 +662,8 @@ bool saveConfig()
 	iniSetBool("radarRotationArrow", radarRotationArrow);
 	iniSetBool("hostQuitConfirmation", hostQuitConfirmation);
 	iniSetBool("PauseOnFocusLoss", war_GetPauseOnFocusLoss());
-	iniSetString("autoratingUrl", getAutoratingUrl());
+	iniSetString("autoratingUrlV2", getAutoratingUrl());
+	iniSetBool("autorating", getAutoratingEnable());
 	iniSetString("masterserver_name", NETgetMasterserverName());
 	iniSetInteger("masterserver_port", (int)NETgetMasterserverPort());
 	iniSetString("server_name", mpGetServerName());
@@ -623,8 +706,9 @@ bool saveConfig()
 	iniSetInteger("colourMP", war_getMPcolour());
 	iniSetInteger("inactivityMinutesMP", war_getMPInactivityMinutes());
 	iniSetInteger("openSpectatorSlotsMP", war_getMPopenSpectatorSlots());
-	iniSetString("favoriteStructs", getFavoriteStructs().toUtf8());
 	iniSetString("gfxbackend", to_string(war_getGfxBackend()));
+	iniSetInteger("minimizeOnFocusLoss", war_getMinimizeOnFocusLoss());
+	iniSetInteger("altEnterToggleMode", war_getToggleFullscreenMode());
 	iniSetString("jsbackend", to_string(war_getJSBackend()));
 	iniSetInteger("BlueprintTrackAnimationSpeed", BlueprintTrackAnimationSpeed);
 	iniSetBool("lockCameraScrollWhileRotating", lockCameraScrollWhileRotating);
@@ -632,6 +716,11 @@ bool saveConfig()
 	iniSetBool("fog", pie_GetFogEnabled());
 	iniSetInteger("hostAutoLagKickSeconds", war_getAutoLagKickSeconds());
 	iniSetBool("disableReplayRecord", war_getDisableReplayRecording());
+	iniSetInteger("maxReplaysSaved", war_getMaxReplaysSaved());
+	iniSetInteger("oldLogsLimit", war_getOldLogsLimit());
+	iniSetInteger("fogEnd", war_getFogEnd());
+	iniSetInteger("fogStart", war_getFogStart());
+	iniSetInteger("configVersion", CURRCONFVERSION);
 
 	// write out ini file changes
 	bool result = saveIniFile(file, ini);
