@@ -83,9 +83,10 @@ void startMouseOptionsMenu();
 void startGameOptionsMenu();
 void startMultiplayOptionsMenu();
 void refreshCurrentVideoOptionsValues();
+void frontendIsShuttingDown();
 
 void addTopForm(bool wide);
-void addBottomForm();
+void addBottomForm(bool wide = false);
 W_FORM *addBackdrop();
 W_FORM *addBackdrop(const std::shared_ptr<W_SCREEN> &screen);
 void addTextButton(UDWORD id, UDWORD PosX, UDWORD PosY, const std::string &txt, unsigned int style);
@@ -130,6 +131,13 @@ void seqScrollEvent();
 
 struct DisplayTextOptionCache
 {
+	enum class OverflowBehavior
+	{
+		None,
+		ShrinkFont
+	};
+	OverflowBehavior overflowBehavior = OverflowBehavior::None;
+	int lastWidgetWidth = 0;
 	WzText wzText;
 };
 
@@ -155,9 +163,18 @@ struct DisplayTextOptionCache
 #define FRONTEND_BOTFORMH		305				// keep Y+H < 480 (minimum display height)
 
 
+#define FRONTEND_BOTFORM_WIDEX		30
+#define FRONTEND_BOTFORM_WIDEY		FRONTEND_BOTFORMY
+#define FRONTEND_BOTFORM_WIDEW		580
+#define FRONTEND_BOTFORM_WIDEH		FRONTEND_BOTFORMH	// keep Y+H < 480 (minimum display height)
+
+
 #define FRONTEND_BUTWIDTH		FRONTEND_BOTFORMW-40 // text button sizes.
 #define FRONTEND_BUTHEIGHT		35
 
+#define FRONTEND_BUTWIDTH_WIDE	FRONTEND_BOTFORM_WIDEW-40 // text button sizes.
+#define FRONTEND_BUTHEIGHT_WIDE	FRONTEND_BUTHEIGHT
+
 #define FRONTEND_POS1X			20				// button positions
 #define FRONTEND_POS1Y			(0*FRONTEND_BUTHEIGHT)
 #define FRONTEND_POS1M			340
@@ -273,6 +290,8 @@ enum
 	FRONTEND_RADAR_R,
 	FRONTEND_RADAR_JUMP,
 	FRONTEND_RADAR_JUMP_R,
+	FRONTEND_LOD_DISTANCE,
+	FRONTEND_LOD_DISTANCE_R,
 	FRONTEND_SSHAKE,
 	FRONTEND_SSHAKE_R,
 
@@ -314,6 +333,10 @@ enum
 	FRONTEND_DISPLAYSCALE_R,
 	FRONTEND_GFXBACKEND,
 	FRONTEND_GFXBACKEND_R,
+	FRONTEND_MINIMIZE_ON_FOCUS_LOSS,
+	FRONTEND_MINIMIZE_ON_FOCUS_LOSS_DROPDOWN,
+	FRONTEND_ALTENTER_TOGGLE_MODE,
+	FRONTEND_ALTENTER_TOGGLE_MODE_DROPDOWN,
 
 	FRONTEND_MOUSEOPTIONS = 25000,          // Mouse Options Menu
 	FRONTEND_CURSORMODE,
@@ -344,6 +367,8 @@ enum
 	FRONTEND_LAG_KICK_DROPDOWN,
 	FRONTEND_SPECTATOR_SLOTS,
 	FRONTEND_SPECTATOR_SLOTS_DROPDOWN,
+	FRONTEND_AUTORATING,
+	FRONTEND_AUTORATING_R,
 
 	FRONTEND_NOGAMESAVAILABLE = 31666	// Used when no games are available in lobby
 
