@@ -33,7 +33,7 @@ struct IMAGEFILE;
 #define FILE_LOAD_BUFFER_SIZE (1024*1024*4)
 extern char fileLoadBuffer[];
 
-bool systemInitialise(float horizScaleFactor, float vertScaleFactor);
+bool systemInitialise(unsigned int horizScalePercentage, unsigned int vertScalePercentage);
 void systemShutdown();
 bool frontendInitialise(const char *ResourceFile);
 bool frontendShutdown();
@@ -59,7 +59,7 @@ struct wzSearchPath
 enum searchPathMode { mod_clean, mod_campaign, mod_multiplay, mod_override };
 
 void registerSearchPath(const char path[], unsigned int priority);
-bool rebuildSearchPath(searchPathMode mode, bool force, const char *current_map = NULL);
+bool rebuildSearchPath(searchPathMode mode, bool force, const char *current_map = NULL, const char* current_map_mount_point = NULL);
 
 bool buildMapList();
 bool CheckForMod(char const *mapFile);
