@@ -408,8 +408,8 @@ bool intAddMessageView(MESSAGE *psMessage)
 	sFormInit.pUserData = psMessage;
 	auto form3dView = std::make_shared<W_CLICKFORM>(&sFormInit);
 
-	/*Add the Flic box if videos are installed */
-	if (PHYSFS_exists("sequences/devastation.ogg"))
+	/*Add the Flic box if videos are installed, or on-demand video streaming is available */
+	if (seq_hasVideos())
 	{
 		sFormInit = W_FORMINIT();
 		sFormInit.id = IDINTMAP_FLICVIEW;
@@ -608,7 +608,7 @@ void intIntelButtonPressed(bool proxMsg, UDWORD id)
 					intAddMessageView(psMessage);
 				}
 				// only attempt to show videos if they are installed
-				if (PHYSFS_exists("sequences/devastation.ogg"))
+				if (seq_hasVideos())
 				{
 					StartMessageSequences(psMessage, true);
 				}
@@ -653,7 +653,12 @@ void intIntelButtonPressed(bool proxMsg, UDWORD id)
 
 					if (audio != nullptr)
 					{
-						playing = audio_PlayStream(audio, sound_GetUIVolume(), [](const void *) { playing = nullptr; }, nullptr);
+						playing = audio_PlayStream(audio, sound_GetUIVolume(), [](const AUDIO_STREAM *stream, const void *) {
+							if (stream == playing)
+							{
+								playing = nullptr;
+							}
+						}, nullptr);
 					}
 				}
 
@@ -793,7 +798,7 @@ void IntMessageButton::display(int xOffset, int yOffset)
 	RESEARCH		*pResearch = nullptr;
 	bool MovieButton = false;
 	ImdObject object;
-	Image image;
+	AtlasImage image;
 
 	initDisplay();
 
@@ -839,11 +844,11 @@ void IntMessageButton::display(int xOffset, int yOffset)
 		}
 		break;
 	case MSG_CAMPAIGN:
-		image = Image(IntImages, IMAGE_INTEL_CAMPAIGN);
+		image = AtlasImage(IntImages, IMAGE_INTEL_CAMPAIGN);
 		MovieButton = true;
 		break;
 	case MSG_MISSION:
-		image = Image(IntImages, IMAGE_INTEL_MISSION);
+		image = AtlasImage(IntImages, IMAGE_INTEL_MISSION);
 		MovieButton = true;
 		break;
 	default:
@@ -856,7 +861,7 @@ void IntMessageButton::display(int xOffset, int yOffset)
 	{
 		if (pResearch->iconID != NO_RESEARCH_ICON)
 		{
-			image = Image(IntImages, pResearch->iconID);
+			image = AtlasImage(IntImages, pResearch->iconID);
 		}
 
 		//do we have the same icon for the top right hand corner?
@@ -866,7 +871,7 @@ void IntMessageButton::display(int xOffset, int yOffset)
 		//draw buttons for mission and general messages
 	{
 		// Draw the button with the relevant image, don't add isDown() to the image ID if it's a movie button.
-		displayImage(MovieButton ? image : Image(image.images, image.id + isDown()), xOffset, yOffset);
+		displayImage(MovieButton ? image : AtlasImage(image.images, image.id + isDown()), xOffset, yOffset);
 	}
 	displayIfHighlight(xOffset, yOffset);
 }
@@ -1046,7 +1051,7 @@ void displayImmediateMessage(MESSAGE *psMessage)
 	*/
 
 	// only attempt to show videos if they are installed
-	if (PHYSFS_exists("sequences/devastation.ogg"))
+	if (seq_hasVideos())
 	{
 	    psCurrentMsg = psMessage;
 	    StartMessageSequences(psMessage, true);
