@@ -52,6 +52,7 @@
 #include "qtscript.h"
 #include "wrappers.h"
 #include "activity.h"
+#include <wzmaplib/map_package.h>
 
 #include <unordered_set>
 
@@ -76,7 +77,7 @@ static LEVEL_DATASET	*psBaseData = nullptr;
 static LEVEL_DATASET	*psCurrLevel = nullptr;
 
 // dummy level data for single WRF loads
-static LEVEL_DATASET	sSingleWRF = { LEVEL_TYPE::LDS_COMPLETE, 0, 0, nullptr, mod_clean, {nullptr}, nullptr, nullptr, nullptr, {{0}}};
+static LEVEL_DATASET	sSingleWRF = { LEVEL_TYPE::LDS_COMPLETE, 0, 0, nullptr, mod_clean, {nullptr}, nullptr, nullptr, nullptr, {{0}}, nullptr};
 
 // return values from the lexer
 char *pLevToken;
@@ -125,6 +126,7 @@ static inline void freeLevel(LEVEL_DATASET* toDelete)
 
 	free(toDelete->pName);
 	free(toDelete->realFileName);
+	free(toDelete->customMountPoint);
 	free(toDelete);
 }
 
@@ -263,6 +265,106 @@ Sha256 levGetMapNameHash(char const *mapName)
 	return levGetFileHash(level);
 }
 
+LEVEL_DATASET* levFindBaseTileset(MAP_TILESET tileset)
+{
+	switch (tileset)
+	{
+		case MAP_TILESET::ARIZONA:
+			return levFindDataSet("MULTI_CAM_1");
+		case MAP_TILESET::URBAN:
+			return levFindDataSet("MULTI_CAM_2");
+		case MAP_TILESET::ROCKIES:
+			return levFindDataSet("MULTI_CAM_3");
+	}
+	return nullptr;
+}
+
+bool levParse_JSON(const std::string& mountPoint, const std::string& filename, searchPathMode pathMode, char const *realFileName)
+{
+	// start a new level data set
+	LEVEL_DATASET *psDataSet = (LEVEL_DATASET *)malloc(sizeof(LEVEL_DATASET));
+	if (!psDataSet)
+	{
+		debug(LOG_FATAL, "Out of memory");
+		abort();
+		return false;
+	}
+	memset(psDataSet, 0, sizeof(LEVEL_DATASET));
+	
+	psDataSet->players = 1;
+	psDataSet->game = -1;
+	psDataSet->dataDir = pathMode;
+	psDataSet->realFileName = realFileName != nullptr ? strdup(realFileName) : nullptr;
+	psDataSet->realFileHash.setZero();  // The hash is only calculated on demand; for example, if the map name matches.
+
+	WzMapPhysFSIO mapIO(mountPoint);
+	auto levelDetails = WzMap::loadLevelDetails_JSON(filename, mapIO);
+	if (!levelDetails.has_value())
+	{
+		debug(LOG_ERROR, "Level File JSON load error: Failed to load JSON: %s", filename.c_str());
+		free(psDataSet);
+		return false;
+	}
+
+	const auto& mapFolderPath = levelDetails.value().mapFolderPath;
+	std::string customMountPoint;
+	if (mapFolderPath.empty())
+	{
+		// support "flattened" plain maps
+		// must be mounted within the virtual filesystem at the appropriate location (not at the root)
+		if (levelDetails.value().name.empty())
+		{
+			debug(LOG_ERROR, "Level File JSON load error: Map has empty name??: %s", filename.c_str());
+			free(psDataSet);
+			return false;
+		}
+		switch (levelDetails.value().type)
+		{
+			case WzMap::MapType::CAMPAIGN:
+			case WzMap::MapType::SAVEGAME:
+				// FUTURE TODO: Support other map types
+				debug(LOG_ERROR, "Level File JSON load error: Unsupported map type: %d", (int)levelDetails.value().type);
+				free(psDataSet);
+				return false;
+			case WzMap::MapType::SKIRMISH:
+				customMountPoint = std::string("multiplay/maps/") + levelDetails.value().name;
+				break;
+		}
+	}
+
+	switch (levelDetails.value().type)
+	{
+		case WzMap::MapType::CAMPAIGN:
+		case WzMap::MapType::SAVEGAME:
+			// FUTURE TODO: Support other map types
+			debug(LOG_ERROR, "Level File JSON load error: Unsupported map type: %d", (int)levelDetails.value().type);
+			free(psDataSet);
+			return false;
+		case WzMap::MapType::SKIRMISH:
+			psDataSet->type = LEVEL_TYPE::SKIRMISH;
+			break;
+	}
+	psDataSet->players = static_cast<SWORD>(levelDetails.value().players);
+	psDataSet->pName = strdup(levelDetails.value().name.c_str());
+	auto gamFilePath = mapIO.pathJoin(mapIO.pathDirName(customMountPoint), levelDetails.value().gamFilePath());
+	psDataSet->apDataFiles[0] = strdup(gamFilePath.c_str());
+	psDataSet->game = 0;
+	psDataSet->psBaseData = levFindBaseTileset(levelDetails.value().tileset);
+	if (psDataSet->psBaseData == nullptr)
+	{
+		debug(LOG_ERROR, "Level File JSON load error: Failed to find base tileset for: %d", (int)levelDetails.value().tileset);
+		free(psDataSet);
+		return false;
+	}
+	if (!customMountPoint.empty())
+	{
+		psDataSet->customMountPoint = strdup(customMountPoint.c_str());
+	}
+
+	psLevels.push_back(psDataSet);
+	return true;
+}
+
 // parse a level description data file
 // the ignoreWrf hack is for compatibility with old maps that try to link in various
 // data files that we have removed
@@ -569,6 +671,8 @@ bool levReleaseMissionData()
 			return false;
 		}
 
+		resDoResLoadCallback();		// do callback.
+
 		// free up the old data
 		for (int i = LEVEL_MAXFILES - 1; i >= 0; i--)
 		{
@@ -602,12 +706,16 @@ bool levReleaseAll()
 	// release old data if any was loaded
 	if (psCurrLevel != nullptr)
 	{
+		resDoResLoadCallback();		// do callback.
+
 		if (!levReleaseMissionData())
 		{
 			debug(LOG_ERROR, "Failed to unload mission data");
 			return false;
 		}
 
+		resDoResLoadCallback();		// do callback.
+
 		// release the game data
 		if (psCurrLevel->psBaseData != nullptr)
 		{
@@ -779,7 +887,7 @@ bool levLoadData(char const *name, Sha256 const *hash, char *pSaveName, GAME_TYP
 		}
 	}
 
-	if (!rebuildSearchPath(psNewLevel->dataDir, true, psNewLevel->realFileName))
+	if (!rebuildSearchPath(psNewLevel->dataDir, true, psNewLevel->realFileName, psNewLevel->customMountPoint))
 	{
 		debug(LOG_ERROR, "Failed to rebuild search path");
 		return false;
@@ -788,7 +896,7 @@ bool levLoadData(char const *name, Sha256 const *hash, char *pSaveName, GAME_TYP
 	// reset the old mission data if necessary
 	if (psCurrLevel != nullptr)
 	{
-		debug(LOG_WZ, "Reseting old mission data");
+		debug(LOG_WZ, "Resetting old mission data");
 		if (!levReleaseMissionData())
 		{
 			debug(LOG_ERROR, "Failed to unload old mission data");
@@ -839,7 +947,11 @@ bool levLoadData(char const *name, Sha256 const *hash, char *pSaveName, GAME_TYP
 	std::unordered_set<FactionID> enabledNonNormalFactions = getEnabledFactions(true);
 	if (!enabledNonNormalFactions.empty())
 	{
-		enumerateLoadedModels([enabledNonNormalFactions](const std::string &modelName, iIMDShape &){
+		enumerateLoadedModels([enabledNonNormalFactions](const std::string &modelName, iIMDShape &s){
+			if (s.modelLevel != 0)
+			{
+				return; // skip
+			}
 			for (const auto& faction : enabledNonNormalFactions)
 			{
 				auto factionModel = getFactionModelName(faction, WzString::fromUtf8(modelName));
@@ -848,6 +960,15 @@ bool levLoadData(char const *name, Sha256 const *hash, char *pSaveName, GAME_TYP
 					iIMDShape *retval = modelGet(factionModel.value());
 					ASSERT(retval != nullptr, "Cannot find the faction PIE model %s (for normal model: %s)",
 						   factionModel.value().toUtf8().c_str(), modelName.c_str());
+					for (const iIMDShape *pIMD = retval, *pNormalIMD = &s; pIMD != nullptr && pNormalIMD != nullptr; pIMD = pIMD->next, pNormalIMD = pNormalIMD->next)
+					{
+						if (pIMD->modelLevel <= 0)
+						{
+							continue;
+						}
+						// Must add mapping to faction IMD lookup table for all additional level modelNames (these have _<level> appended)
+						addFactionModelNameMapping(faction, WzString::fromUtf8(pNormalIMD->modelName), WzString::fromUtf8(pIMD->modelName));
+					}
 				}
 			}
 		});
@@ -1127,14 +1248,14 @@ bool levLoadData(char const *name, Sha256 const *hash, char *pSaveName, GAME_TYP
 	ssprintf(buf, "Current Level/map is %s", psCurrLevel->pName);
 	addDumpInfo(buf);
 
-	if (autogame_enabled())
+	if (autogame_enabled() && getHostLaunch() != HostLaunch::LoadReplay)
 	{
 		gameTimeSetMod(Rational(500));
 		if (getHostLaunch() != HostLaunch::Skirmish) // tests will specify the AI manually
 		{
 			if (selectedPlayer < MAX_PLAYERS && !NetPlay.players[selectedPlayer].isSpectator)
 			{
-				jsAutogameSpecific("multiplay/skirmish/semperfi.js", selectedPlayer);
+				jsAutogameSpecific("multiplay/skirmish/semperfi.js", selectedPlayer, AIDifficulty::DEFAULT);
 			}
 			else
 			{
@@ -1152,11 +1273,7 @@ std::string mapNameWithoutTechlevel(const char *mapName)
 {
 	ASSERT_OR_RETURN("", mapName != nullptr, "null mapName provided");
 	std::string result(mapName);
-	size_t len = result.length();
-	if (len > 2 && result[len - 3] == '-' && result[len - 2] == 'T' && isdigit(result[len - 1]))
-	{
-		result.resize(len - 3);
-	}
+	WzMap::trimTechLevelFromMapName(result);
 	return result;
 }
 
