@@ -100,6 +100,7 @@
 #include "map.h"
 #include "keybind.h"
 #include "random.h"
+#include "urlhelpers.h"
 #include "urlrequest.h"
 #include <time.h>
 #include <LaunchInfo.h>
@@ -183,6 +184,7 @@ char	ScreenDumpPath[PATH_MAX];
 char	MultiCustomMapsPath[PATH_MAX];
 char	MultiPlayersPath[PATH_MAX];
 char	KeyMapPath[PATH_MAX];
+char	FavoriteStructuresPath[PATH_MAX];
 // Start game in title mode:
 static GS_GAMEMODE gameStatus = GS_TITLE_SCREEN;
 // Status of the gameloop
@@ -757,16 +759,6 @@ static void scanDataDirs()
 	registerSearchPath("/Library/Application Support/Warzone 2100/", 1);
 #endif
 
-	// Commandline supplied datadir
-	if (strlen(datadir) != 0)
-	{
-		registerSearchPath(datadir, 1);
-	}
-
-	// User's home dir
-	registerSearchPath(PHYSFS_getWriteDir(), 2);
-	rebuildSearchPath(mod_multiplay, true);
-
 #if !defined(WZ_OS_MAC)
 	// Check PREFIX-based paths
 	std::string tmpstr;
@@ -838,7 +830,7 @@ static void scanDataDirs()
 		{
 			WzString resourceDataPath(resourcePath);
 			resourceDataPath += "/data";
-			registerSearchPath(resourceDataPath.toUtf8().c_str(), 8);
+			registerSearchPath(resourceDataPath.toUtf8().c_str(), 3);
 			rebuildSearchPath(mod_multiplay, true);
 		}
 		else
@@ -853,6 +845,16 @@ static void scanDataDirs()
 	}
 #endif
 
+	// Commandline supplied datadir
+	if (strlen(datadir) != 0)
+	{
+		registerSearchPath(datadir, 1);
+	}
+
+	// User's home dir
+	registerSearchPath(PHYSFS_getWriteDir(), 2);
+	rebuildSearchPath(mod_multiplay, true);
+
 	/** Debugging and sanity checks **/
 
 	printSearchPath();
@@ -970,7 +972,8 @@ static void startGameLoop()
 	if (!levLoadData(aLevelName, &game.hash, nullptr, GTYPE_SCENARIO_START))
 	{
 		debug(LOG_FATAL, "Shutting down after failure");
-		exit(EXIT_FAILURE);
+		wzQuit(EXIT_FAILURE);
+		return;
 	}
 
 	screen_StopBackDrop();
@@ -1018,7 +1021,7 @@ static void startGameLoop()
 			if (!war_getDisableReplayRecording())
 			{
 				WZGameReplayOptionsHandler replayOptions;
-				NETreplaySaveStart((currentGameMode == ActivitySink::GameMode::MULTIPLAYER) ? "multiplay" : "skirmish", replayOptions, (currentGameMode == ActivitySink::GameMode::MULTIPLAYER));
+				NETreplaySaveStart((currentGameMode == ActivitySink::GameMode::MULTIPLAYER) ? "multiplay" : "skirmish", replayOptions, war_getMaxReplaysSaved(), (currentGameMode == ActivitySink::GameMode::MULTIPLAYER));
 			}
 			break;
 		}
@@ -1029,8 +1032,16 @@ static void startGameLoop()
 	setMaxFastForwardTicks(WZ_DEFAULT_MAX_FASTFORWARD_TICKS, true); // default value / spectator "catch-up" behavior
 	if (NETisReplay())
 	{
-		// for replays, ensure we don't start off fast-forwarding
-		setMaxFastForwardTicks(0, true);
+		if (!headlessGameMode() && !autogame_enabled())
+		{
+			// for replays, ensure we don't start off fast-forwarding
+			setMaxFastForwardTicks(0, true);
+		}
+		else
+		{
+			// when loading replays in headless / autogame mode, set to fast-forward
+			setMaxFastForwardTicks(10, false);
+		}
 	}
 }
 
@@ -1136,6 +1147,35 @@ static bool initSaveGameLoad()
  */
 static void runGameLoop()
 {
+	// Run the second half of a queued gameloopstatus change
+	// (This is needed so that the main loop completes once between initializing the loading screen and actually loading)
+	switch (gameLoopStatus)
+	{
+	case GAMECODE_QUITGAME:
+		debug(LOG_MAIN, "GAMECODE_QUITGAME");
+		ActivityManager::instance().quitGame(collectEndGameStatsData(), Cheated);
+		cdAudio_SetGameMode(MusicGameMode::MENUS);
+		stopGameLoop();
+		startTitleLoop(); // Restart into titleloop
+		gameLoopStatus = GAMECODE_CONTINUE;
+		return;
+	case GAMECODE_LOADGAME:
+		debug(LOG_MAIN, "GAMECODE_LOADGAME");
+		stopGameLoop();
+		initSaveGameLoad(); // Restart and load a savegame
+		gameLoopStatus = GAMECODE_CONTINUE;
+		return;
+	case GAMECODE_NEWLEVEL:
+		debug(LOG_MAIN, "GAMECODE_NEWLEVEL");
+		stopGameLoop();
+		startGameLoop(); // Restart gameloop
+		gameLoopStatus = GAMECODE_CONTINUE;
+		return;
+	default:
+		// ignore other values, and proceed with gameLoop
+		break;
+	}
+
 	gameLoopStatus = gameLoop();
 	switch (gameLoopStatus)
 	{
@@ -1144,20 +1184,15 @@ static void runGameLoop()
 		break;
 	case GAMECODE_QUITGAME:
 		debug(LOG_MAIN, "GAMECODE_QUITGAME");
-		ActivityManager::instance().quitGame(collectEndGameStatsData(), Cheated);
-		cdAudio_SetGameMode(MusicGameMode::MENUS);
-		stopGameLoop();
-		startTitleLoop(); // Restart into titleloop
+		initLoadingScreen(true);
 		break;
 	case GAMECODE_LOADGAME:
 		debug(LOG_MAIN, "GAMECODE_LOADGAME");
-		stopGameLoop();
-		initSaveGameLoad(); // Restart and load a savegame
+		initLoadingScreen(true);
 		break;
 	case GAMECODE_NEWLEVEL:
 		debug(LOG_MAIN, "GAMECODE_NEWLEVEL");
-		stopGameLoop();
-		startGameLoop(); // Restart gameloop
+		initLoadingScreen(true);
 		break;
 	// Never thrown:
 	case GAMECODE_FASTEXIT:
@@ -1170,11 +1205,48 @@ static void runGameLoop()
 }
 
 
+static optional<TITLECODE> queuedTilecodeChange = nullopt;
+
 /*!
  * Run the code inside the titleloop
  */
 static void runTitleLoop()
 {
+	if (queuedTilecodeChange.has_value())
+	{
+		// Run the second half of a queued titlecode change
+		// (This is needed so that the main loop completes once between initializing the loading screen and actually loading)
+		TITLECODE toProcess = queuedTilecodeChange.value();
+		queuedTilecodeChange.reset();
+		switch (toProcess)
+		{
+		case TITLECODE_SAVEGAMELOAD:
+			{
+				debug(LOG_MAIN, "TITLECODE_SAVEGAMELOAD");
+				// Restart into gameloop and load a savegame, ONLY on a good savegame load!
+				stopTitleLoop();
+				if (!initSaveGameLoad())
+				{
+					// we had a error loading savegame (corrupt?), so go back to title screen?
+					stopGameLoop();
+					startTitleLoop();
+					changeTitleMode(TITLE);
+				}
+				closeLoadingScreen();
+				return;
+			}
+		case TITLECODE_STARTGAME:
+			debug(LOG_MAIN, "TITLECODE_STARTGAME");
+			stopTitleLoop();
+			startGameLoop(); // Restart into gameloop
+			closeLoadingScreen();
+			return;
+		default:
+			// ignore unexpected value
+			break;
+		}
+	}
+
 	switch (titleLoop())
 	{
 	case TITLECODE_CONTINUE:
@@ -1188,24 +1260,13 @@ static void runTitleLoop()
 		{
 			debug(LOG_MAIN, "TITLECODE_SAVEGAMELOAD");
 			initLoadingScreen(true);
-			// Restart into gameloop and load a savegame, ONLY on a good savegame load!
-			stopTitleLoop();
-			if (!initSaveGameLoad())
-			{
-				// we had a error loading savegame (corrupt?), so go back to title screen?
-				stopGameLoop();
-				startTitleLoop();
-				changeTitleMode(TITLE);
-			}
-			closeLoadingScreen();
+			queuedTilecodeChange = TITLECODE_SAVEGAMELOAD;
 			break;
 		}
 	case TITLECODE_STARTGAME:
 		debug(LOG_MAIN, "TITLECODE_STARTGAME");
 		initLoadingScreen(true);
-		stopTitleLoop();
-		startGameLoop(); // Restart into gameloop
-		closeLoadingScreen();
+		queuedTilecodeChange = TITLECODE_STARTGAME;
 		break;
 	case TITLECODE_SHOWINTRO:
 		debug(LOG_MAIN, "TITLECODE_SHOWINTRO");
@@ -1228,6 +1289,8 @@ void mainLoop()
 {
 	frameUpdate(); // General housekeeping
 
+	pie_ScreenFrameRenderBegin();
+
 	// Screenshot key is now available globally
 	if (keyPressed(KEY_F10))
 	{
@@ -1242,14 +1305,17 @@ void mainLoop()
 		if (loop_GetVideoStatus())
 		{
 			videoLoop(); // Display the video if necessary
+			pie_ScreenFrameRenderEnd();
 		}
 		else switch (GetGameMode())
 			{
 			case GS_NORMAL: // Run the gameloop code
 				runGameLoop();
+				// gameLoop handles pie_ScreenFrameRenderEnd()
 				break;
 			case GS_TITLE_SCREEN: // Run the titleloop code
 				runTitleLoop();
+				pie_ScreenFrameRenderEnd();
 				break;
 			default:
 				break;
@@ -1264,7 +1330,7 @@ void mainLoop()
 #endif
 }
 
-bool getUTF8CmdLine(int *const utfargc WZ_DECL_UNUSED, char *** const utfargv WZ_DECL_UNUSED) // explicitely pass by reference
+bool getUTF8CmdLine(int *const _utfargc WZ_DECL_UNUSED, char *** const _utfargv WZ_DECL_UNUSED) // explicitely pass by reference
 {
 #ifdef WZ_OS_WIN
 	int wargc;
@@ -1285,8 +1351,8 @@ bool getUTF8CmdLine(int *const utfargc WZ_DECL_UNUSED, char *** const utfargv WZ
 		return false;
 	}
 	// the following malloc and UTF16toUTF8 will be cleaned up in realmain().
-	*utfargv = (char **)malloc(sizeof(char *) * wargc);
-	if (!*utfargv)
+	*_utfargv = (char **)malloc(sizeof(char *) * wargc);
+	if (!*_utfargv)
 	{
 		debug(LOG_FATAL, "Out of memory!");
 		abort();
@@ -1295,16 +1361,16 @@ bool getUTF8CmdLine(int *const utfargc WZ_DECL_UNUSED, char *** const utfargv WZ
 	for (int i = 0; i < wargc; ++i)
 	{
 		STATIC_ASSERT(sizeof(wchar_t) == sizeof(utf_16_char)); // Should be true on windows
-		(*utfargv)[i] = UTF16toUTF8((const utf_16_char *)wargv[i], NULL); // only returns null when memory runs out
-		if ((*utfargv)[i] == NULL)
+		(*_utfargv)[i] = UTF16toUTF8((const utf_16_char *)wargv[i], NULL); // only returns null when memory runs out
+		if ((*_utfargv)[i] == NULL)
 		{
-			*utfargc = i;
+			*_utfargc = i;
 			LocalFree(wargv);
 			abort();
 			return false;
 		}
 	}
-	*utfargc = wargc;
+	*_utfargc = wargc;
 	LocalFree(wargv);
 #endif
 	return true;
@@ -1412,58 +1478,95 @@ static bool winCheckIfRunningUnderWine(std::string* output_wineinfostr = nullptr
 	typedef const char* (CDECL *WineGetVersionFunction)(void);
 	typedef void (CDECL *WineGetHostVersionFunction)(const char **sysname, const char **release);
 
-	HMODULE hntdll = GetModuleHandleW(L"ntdll.dll");
-	if (!hntdll)
-	{
-		return false;
-	}
-
-	WineGetVersionFunction pWineGetVersion = reinterpret_cast<WineGetVersionFunction>(reinterpret_cast<void*>(GetProcAddress(hntdll, "wine_get_version")));
-	WineGetHostVersionFunction pWineGetHostVersion = reinterpret_cast<WineGetHostVersionFunction>(reinterpret_cast<void*>(GetProcAddress(hntdll, "wine_get_host_version")));
-
-	if (pWineGetVersion == nullptr)
-	{
-		return false;
-	}
-
+	bool bResult = false;
 	std::string resultWineVersionInfo = "Wine";
-	const char* pWineVer = pWineGetVersion();
-	if (pWineVer != nullptr)
-	{
-		resultWineVersionInfo += std::string(" ") + pWineVer;
-	}
 
-	const char* pSysname = nullptr;
-	const char* pSysversion = nullptr;
-	if (pWineGetHostVersion != nullptr)
+	HMODULE hntdll = GetModuleHandleW(L"ntdll.dll");
+	if (hntdll != NULL)
 	{
-		pWineGetHostVersion(&pSysname, &pSysversion);
-	}
-	if (pSysname != nullptr)
-	{
-		if (output_platform)
+		WineGetVersionFunction pWineGetVersion = reinterpret_cast<WineGetVersionFunction>(reinterpret_cast<void*>(GetProcAddress(hntdll, "wine_get_version")));
+		WineGetHostVersionFunction pWineGetHostVersion = reinterpret_cast<WineGetHostVersionFunction>(reinterpret_cast<void*>(GetProcAddress(hntdll, "wine_get_host_version")));
+
+		if (pWineGetVersion == nullptr && pWineGetHostVersion == nullptr && !bResult)
 		{
-			(*output_platform) = pSysname;
+			auto ptrWineSetUnixEnv = GetProcAddress(hntdll, "__wine_set_unix_env");
+			if (ptrWineSetUnixEnv != NULL)
+			{
+				bResult = true;
+			}
 		}
-		resultWineVersionInfo += std::string(" (under ") + pSysname;
-		if (pSysversion != nullptr)
+
+		if (pWineGetVersion != nullptr)
 		{
-			resultWineVersionInfo += std::string(" ") + pSysversion;
+			bResult = true;
+			const char* pWineVer = pWineGetVersion();
+			if (pWineVer != nullptr)
+			{
+				resultWineVersionInfo += std::string(" ") + pWineVer;
+			}
+		}
+
+		const char* pSysname = nullptr;
+		const char* pSysversion = nullptr;
+		if (pWineGetHostVersion != nullptr)
+		{
+			bResult = true;
+			pWineGetHostVersion(&pSysname, &pSysversion);
+		}
+		if (pSysname != nullptr)
+		{
+			if (output_platform)
+			{
+				(*output_platform) = pSysname;
+			}
+			resultWineVersionInfo += std::string(" (under ") + pSysname;
+			if (pSysversion != nullptr)
+			{
+				resultWineVersionInfo += std::string(" ") + pSysversion;
+			}
+			resultWineVersionInfo += ")";
 		}
-		resultWineVersionInfo += ")";
 	}
 
-	if (output_wineinfostr)
+	if (!bResult)
 	{
-		(*output_wineinfostr) = resultWineVersionInfo;
+		HMODULE hKernel32 = GetModuleHandleW(L"kernel32.dll");
+		if (hKernel32 != NULL)
+		{
+			auto wineUnixFileNameFuncPt = GetProcAddress(hKernel32, "wine_get_unix_file_name");
+			if (wineUnixFileNameFuncPt != NULL)
+			{
+				bResult = true;
+			}
+			else
+			{
+				auto wineDosFileNameFuncPt = GetProcAddress(hKernel32, "wine_get_dos_file_name");
+				if (wineDosFileNameFuncPt != NULL)
+				{
+					bResult = true;
+				}
+			}
+		}
 	}
 
-	return true;
+	if (bResult)
+	{
+		if (output_wineinfostr)
+		{
+			(*output_wineinfostr) = resultWineVersionInfo;
+		}
+	}
+
+	return bResult;
 #else
 	return false;
 #endif /* WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP) */
 }
 
+#define WINEOPENURLWRAPPER(url, cl) \
+openURLInBrowser(url); \
+if (cl) { wzQuit(0); }
+
 void osSpecificPostInit_Win()
 {
 	std::string wineInfoStr;
@@ -1471,22 +1574,45 @@ void osSpecificPostInit_Win()
 	if (winCheckIfRunningUnderWine(&wineInfoStr, &wineHostPlatform))
 	{
 		const char* pWineNativeAvailableMsg = "You are running the Windows version of Warzone 2100 under Wine.\n\nA native version for your platform is likely available (and will perform better).\n\nPlease visit: https://wz2100.net";
+		std::string platformSimple;
 		// Display a messagebox that a native version is available for this platform
 		if (!wineHostPlatform.empty())
 		{
 			if (strncasecmp(wineHostPlatform.c_str(), "Darwin", std::min<size_t>(wineHostPlatform.size(), strlen("Darwin"))) == 0)
 			{
 				// macOS
+				platformSimple = "macOS";
 				pWineNativeAvailableMsg = "You are running the Windows version of Warzone 2100 under Wine.\n\nA native version for macOS is available.\n\nPlease visit: https://wz2100.net";
 			}
 			if (strncasecmp(wineHostPlatform.c_str(), "Linux", std::min<size_t>(wineHostPlatform.size(), strlen("Linux"))) == 0)
 			{
 				// Linux
+				platformSimple = "Linux";
 				pWineNativeAvailableMsg = "You are running the Windows version of Warzone 2100 under Wine.\n\nNative builds for Linux are available.\n\nPlease visit: https://wz2100.net";
 			}
 		}
 
 		wzDisplayDialog(Dialog_Information, "Warzone 2100 under Wine", pWineNativeAvailableMsg);
+
+		std::string url = "https://warzone2100.github.io/update-data/redirect/wine.html";
+		std::string queryString;
+		if (!platformSimple.empty())
+		{
+			queryString += (queryString.empty()) ? "?" : "&";
+			queryString += std::string("platform=") + platformSimple;
+		}
+		std::string variant;
+		if ((GetEnvironmentVariableW(L"STEAM_COMPAT_APP_ID", NULL, 0) > 0) || (GetEnvironmentVariableW(L"SteamAppId", NULL, 0) > 0))
+		{
+			variant = "Proton";
+		}
+		if (!variant.empty())
+		{
+			queryString += (queryString.empty()) ? "?" : "&";
+			queryString += std::string("variant=") + variant;
+		}
+		url += queryString;
+		WINEOPENURLWRAPPER(url.c_str(), !variant.empty())
 	}
 }
 #endif /* defined(WZ_OS_WIN) */
@@ -1581,19 +1707,104 @@ static void wzCmdInterfaceShutdown()
 	stdInThreadShutdown();
 }
 
+static void cleanupOldLogFiles()
+{
+	const int MAX_OLD_LOGS = war_getOldLogsLimit();
+	if (MAX_OLD_LOGS < 0)
+	{
+		// skip cleanup
+		return;
+	}
+
+	ASSERT_OR_RETURN(, PHYSFS_isInit() != 0, "PhysFS isn't initialized");
+	// safety check to ensure we *never* delete the current log file
+	WzString fullCurrentLogFileName = WzString::fromUtf8(getDefaultLogFilePath("/"));
+	if (fullCurrentLogFileName.startsWith("logs/"))
+	{
+		fullCurrentLogFileName.remove(0, 5);
+	}
+
+	CleanupFileEnumFilterFunctions filterFuncs;
+	constexpr std::chrono::hours MinDeletableAge(24 * 7); // never delete logs that are less than a week old
+	auto currentTime = std::chrono::system_clock::now();
+	filterFuncs.fileNameFilterFunction = [](const char *fileName) -> bool {
+		return filenameEndWithExtension(fileName, ".txt") || filenameEndWithExtension(fileName, ".log");
+	};
+	filterFuncs.fileLastModifiedFilterFunction = [currentTime, MinDeletableAge](time_t fileLastModified) -> bool {
+		return std::chrono::duration_cast<std::chrono::hours>(currentTime - std::chrono::system_clock::from_time_t(fileLastModified)) >= MinDeletableAge;
+	};
+
+	// clean up old log .txt / .log files
+	WZ_PHYSFS_cleanupOldFilesInFolder("logs", filterFuncs, MAX_OLD_LOGS, [fullCurrentLogFileName](const char *fileName){
+		if (fullCurrentLogFileName.compare(fileName) == 0)
+		{
+			// skip
+			return true;
+		}
+		if (PHYSFS_delete(fileName) == 0)
+		{
+			debug(LOG_ERROR, "Failed to delete old log file: %s", fileName);
+			return false;
+		}
+		return true;
+	});
+}
+
 // for backend detection
 extern const char *BACKEND;
 
+static int utfargc = 0;
+static char **utfargv = nullptr;
+
+void mainShutdown()
+{
+	ActivityManager::instance().preSystemShutdown();
+
+	switch (GetGameMode())
+	{
+		case GS_NORMAL:
+			// if running a game while quitting, stop the game loop
+			// (currently required for some cleanup) (should modelShutdown() be added to systemShutdown?)
+			stopGameLoop();
+			break;
+		case GS_TITLE_SCREEN:
+			// if showing the title / menus while quitting, stop the title loop
+			// (currently required for some cleanup)
+			stopTitleLoop();
+			break;
+		default:
+			break;
+	}
+	saveConfig();
+	writeFavoriteStructsFile(FavoriteStructuresPath);
+#if defined(ENABLE_DISCORD)
+	discordRPCShutdown();
+#endif
+	wzCmdInterfaceShutdown();
+	urlRequestShutdown();
+	cleanupOldLogFiles();
+	systemShutdown();
+#ifdef WZ_OS_WIN	// clean up the memory allocated for the command line conversion
+	for (int i = 0; i < utfargc; i++)
+	{
+		char *** const utfargvF = &utfargv;
+		free((void *)(*utfargvF)[i]);
+	}
+	free(utfargv);
+#endif
+	ActivityManager::instance().shutdown();
+}
+
 int realmain(int argc, char *argv[])
 {
-	int utfargc = argc;
-	char **utfargv = (char **)argv;
+	utfargc = argc;
+	utfargv = (char **)argv;
 
 	osSpecificFirstChanceProcessSetup();
 
 	debug_init();
 	debug_register_callback(debug_callback_stderr, nullptr, nullptr, nullptr);
-#if defined(WZ_OS_WIN) && defined(DEBUG_INSANE)
+#if defined(_WIN32) && defined(DEBUG_INSANE)
 	debug_register_callback(debug_callback_win32debug, NULL, NULL, NULL);
 #endif // WZ_OS_WIN && DEBUG_INSANE
 
@@ -1605,15 +1816,22 @@ int realmain(int argc, char *argv[])
 		return EXIT_FAILURE;
 	}
 
+	// find debugging flags extra early
+	if (!ParseCommandLineDebugFlags(utfargc, utfargv))
+	{
+		return EXIT_FAILURE;
+	}
+
 	/*** Initialize PhysicsFS ***/
 	initialize_PhysicsFS(utfargv[0]);
 
 	/** Initialize crash-handling provider, if configured */
 	/** NOTE: Should come as early as possible in process init, but needs to be after initialize_PhysicsFS because we need the platform pref dir for storing temporary crash files... */
-	bool bCrashHandlingProvider = useCrashHandlingProvider(utfargc, utfargv);
+	bool debugCrashHandler = false;
+	bool bCrashHandlingProvider = useCrashHandlingProvider(utfargc, utfargv, debugCrashHandler);
 	if (bCrashHandlingProvider)
 	{
-		bCrashHandlingProvider = initCrashHandlingProvider(getWzPlatformPrefDir(), getDefaultLogFilePath(PHYSFS_getDirSeparator()));
+		bCrashHandlingProvider = initCrashHandlingProvider(getWzPlatformPrefDir(), getDefaultLogFilePath(PHYSFS_getDirSeparator()), debugCrashHandler);
 	}
 	auto shutdown_crash_handling_provider_on_return = gsl::finally([bCrashHandlingProvider] { if (bCrashHandlingProvider) { shutdownCrashHandlingProvider(); } });
 
@@ -1725,6 +1943,7 @@ int realmain(int argc, char *argv[])
 
 	/* Put in the writedir root */
 	sstrcpy(KeyMapPath, "keymap.json");
+	sstrcpy(FavoriteStructuresPath, "favoriteStructures.json");
 
 	// initialise all the command line states
 	war_SetDefaultStates();
@@ -1732,6 +1951,7 @@ int realmain(int argc, char *argv[])
 	debug(LOG_MAIN, "initializing");
 
 	loadConfig();
+	loadFavoriteStructsFile(FavoriteStructuresPath);
 
 	// parse the command line
 	if (!ParseCommandLine(utfargc, utfargv))
@@ -1840,7 +2060,7 @@ int realmain(int argc, char *argv[])
 	{
 		gfxbackend = war_getGfxBackend();
 	}
-	if (!wzMainScreenSetup(gfxbackend, war_getAntialiasing(), war_getWindowMode(), war_GetVsync()))
+	if (!wzMainScreenSetup(gfxbackend, war_getAntialiasing(), war_getWindowMode(), war_GetVsync(), war_getLODDistanceBiasPercentage()))
 	{
 		saveConfig(); // ensure any setting changes are persisted on failure
 		return EXIT_FAILURE;
@@ -1863,9 +2083,9 @@ int realmain(int argc, char *argv[])
 	ssprintf(buf, "Video Mode %d x %d (%s)", w, h, to_display_string(war_getWindowMode()).c_str());
 	addDumpInfo(buf);
 
-	float horizScaleFactor, vertScaleFactor;
-	wzGetGameToRendererScaleFactor(&horizScaleFactor, &vertScaleFactor);
-	debug(LOG_WZ, "Game to Renderer Scale Factor (w x h): %f x %f", horizScaleFactor, vertScaleFactor);
+	unsigned int horizScaleFactor, vertScaleFactor;
+	wzGetGameToRendererScaleFactorInt(&horizScaleFactor, &vertScaleFactor);
+	debug(LOG_WZ, "Game to Renderer Scale Factor (w x h): %u%% x %u%%", horizScaleFactor, vertScaleFactor);
 
 	debug(LOG_MAIN, "Final initialization");
 	if (!frameInitialise())
@@ -1881,23 +2101,15 @@ int realmain(int argc, char *argv[])
 		return EXIT_FAILURE;
 	}
 
-	if (!headlessGameMode())
-	{
-		unsigned int windowWidth = 0, windowHeight = 0;
-		wzGetWindowResolution(nullptr, &windowWidth, &windowHeight);
-		war_SetWidth(windowWidth);
-		war_SetHeight(windowHeight);
-	}
-
 	bool fogConfigOption = pie_GetFogEnabled();
 	pie_SetFogStatus(false);
-	pie_ScreenFlip(CLEAR_BLACK);
 
 	pal_Init();
 
 	pie_LoadBackDrop(SCREEN_RANDOMBDROP);
 	pie_SetFogStatus(false);
-	pie_ScreenFlip(CLEAR_BLACK);
+
+	pie_ScreenFrameRenderEnd();
 
 	if (!systemInitialise(horizScaleFactor, vertScaleFactor))
 	{
@@ -1961,40 +2173,8 @@ int realmain(int argc, char *argv[])
 
 	osSpecificPostInit();
 
-	wzMainEventLoop();
-	ActivityManager::instance().preSystemShutdown();
+	wzMainEventLoop(mainShutdown);
 
-	switch (GetGameMode())
-	{
-		case GS_NORMAL:
-			// if running a game while quitting, stop the game loop
-			// (currently required for some cleanup) (should modelShutdown() be added to systemShutdown?)
-			stopGameLoop();
-			break;
-		case GS_TITLE_SCREEN:
-			// if showing the title / menus while quitting, stop the title loop
-			// (currently required for some cleanup)
-			stopTitleLoop();
-			break;
-		default:
-			break;
-	}
-	saveConfig();
-#if defined(ENABLE_DISCORD)
-	discordRPCShutdown();
-#endif
-	wzCmdInterfaceShutdown();
-	urlRequestShutdown();
-	systemShutdown();
-#ifdef WZ_OS_WIN	// clean up the memory allocated for the command line conversion
-	for (int i = 0; i < argc; i++)
-	{
-		char *** const utfargvF = &utfargv;
-		free((void *)(*utfargvF)[i]);
-	}
-	free(utfargv);
-#endif
-	ActivityManager::instance().shutdown();
 	int exitCode = wzGetQuitExitCode();
 	wzShutdown();
 	debug(LOG_MAIN, "Completed shutting down Warzone 2100");
