@@ -35,6 +35,7 @@
 static PointTree *gridPointTree = nullptr;  // A quad-tree-like object.
 static PointTree::Filter *gridFiltersUnseen;
 static PointTree::Filter *gridFiltersDroidsByPlayer;
+static PointTree::Filter *gridFiltersDroidsRepairCandidates;
 
 // initialise the grid system
 bool gridInitialise()
@@ -43,6 +44,7 @@ bool gridInitialise()
 	gridPointTree = new PointTree;
 	gridFiltersUnseen = new PointTree::Filter[MAX_PLAYERS];
 	gridFiltersDroidsByPlayer = new PointTree::Filter[MAX_PLAYERS];
+	gridFiltersDroidsRepairCandidates = new PointTree::Filter[MAX_PLAYERS];
 
 	return true;  // Yay, nothing failed!
 }
@@ -78,6 +80,7 @@ void gridReset()
 	{
 		gridFiltersUnseen[player].reset(*gridPointTree);
 		gridFiltersDroidsByPlayer[player].reset(*gridPointTree);
+		gridFiltersDroidsRepairCandidates[player].reset(*gridPointTree);
 	}
 }
 
@@ -126,10 +129,10 @@ static GridList const &gridStartIterateFiltered(int32_t x, int32_t y, uint32_t r
 		}
 	}
 	gridPointTree->lastQueryResults.erase(w, i);  // Erase all points that were a bit too far.
-	/*
+
 	// In case you are curious.
-	debug(LOG_WARNING, "gridStartIterateFiltered(%d, %d, %u) found %u objects", x, y, radius, (unsigned)gridPointTree->lastQueryResults.size());
-	*/
+	//debug(LOG_WARNING, "gridStartIterateFiltered(%d, %d, %u) found %u objects", x, y, radius, (unsigned)gridPointTree->lastQueryResults.size());
+	
 	static GridList gridList;
 	gridList.resize(gridPointTree->lastQueryResults.size());
 	for (unsigned n = 0; n < gridList.size(); ++n)
@@ -186,6 +189,28 @@ GridList const &gridStartIterateDroidsByPlayer(int32_t x, int32_t y, uint32_t ra
 	return gridStartIterateFiltered(x, y, radius, &gridFiltersDroidsByPlayer[player], ConditionDroidsByPlayer(player));
 }
 
+struct ConditionDroidCandidateForRepair
+{
+	ConditionDroidCandidateForRepair(int32_t player_) : player(player_) {}
+	bool test(BASE_OBJECT *obj) const
+	{
+		if (obj->type != OBJ_DROID) return false;
+		const DROID *psDroid = (const DROID*) obj;
+		const bool isOwnOrAlly = psDroid->player == player && aiCheckAlliances(psDroid->player, player);
+		const bool isVTOL = asPropulsionStats[psDroid->asBits[COMP_PROPULSION]].propulsionType == PROPULSION_TYPE_LIFT;
+		// either it's a ground unit, or it's a VTOL on ground
+		const bool isOnGround = (!isVTOL) || (isVTOL && (psDroid->sMove.Status == MOVEINACTIVE && psDroid->sMove.iVertSpeed == 0));
+		// Note: no check for droidIsDamaged(psDroid) this is intentional
+		return !psDroid->died && isOwnOrAlly && isOnGround;
+	}
+	int player;
+};
+
+GridList const &gridStartIterateRepairCandidates(int32_t x, int32_t y, uint32_t radius, int player)
+{
+	return gridStartIterateFiltered(x, y, radius, &gridFiltersDroidsRepairCandidates[player], ConditionDroidCandidateForRepair(player));
+}
+
 struct ConditionUnseen
 {
 	ConditionUnseen(int32_t player_) : player(player_) {}
