@@ -29,6 +29,7 @@
 #include "lib/framework/file.h"
 #include "lib/framework/wzapp.h"
 #include "lib/framework/physfs_ext.h"
+#include "lib/framework/frameresource.h"
 
 #include "lib/ivis_opengl/piepalette.h" // for pal_Init()
 #include "lib/ivis_opengl/piestate.h"
@@ -65,6 +66,8 @@
 #include "activity.h"
 #include "warzoneconfig.h"
 
+#define MAX_STRUCTURE_LIMITS 4096 // Set a high (but explicit) maximum for the number of structure limits supported
+
 // send complete game info set!
 void sendOptions()
 {
@@ -122,6 +125,11 @@ void sendOptions()
 
 	// Send the number of structure limits to expect
 	uint32_t numStructureLimits = static_cast<uint32_t>(ingame.structureLimits.size());
+	if (numStructureLimits > MAX_STRUCTURE_LIMITS)
+	{
+		debug(LOG_ERROR, "Number of structure limits (%" PRIu32") exceeds maximum supported - truncating", numStructureLimits);
+		numStructureLimits = MAX_STRUCTURE_LIMITS;
+	}
 	NETuint32_t(&numStructureLimits);
 	debug(LOG_NET, "(Host) Structure limits to process on client is %zu", ingame.structureLimits.size());
 	// Send the structures changed
@@ -138,6 +146,8 @@ void sendOptions()
 	ActivityManager::instance().updateMultiplayGameData(game, ingame, NETGameIsLocked());
 }
 
+std::vector<MULTISTRUCTLIMITS> oldStructureLimits;
+
 // ////////////////////////////////////////////////////////////////////////////
 // options for a game. (usually recvd in frontend)
 // returns: false if the options should be considered invalid and the client should disconnect
@@ -204,6 +214,9 @@ bool recvOptions(NETQUEUE queue)
 	}
 	netPlayersUpdated = true;
 
+	// Make a copy of old structure limits to see what got changed
+	oldStructureLimits = ingame.structureLimits;
+
 	// Free any structure limits we may have in-place
 	ingame.structureLimits.clear();
 
@@ -211,21 +224,99 @@ bool recvOptions(NETQUEUE queue)
 	uint32_t numStructureLimits = 0;
 	NETuint32_t(&numStructureLimits);
 	debug(LOG_NET, "Host is sending us %u structure limits", numStructureLimits);
+	if (numStructureLimits > MAX_STRUCTURE_LIMITS)
+	{
+		debug(LOG_POPUP, "Number of structure limits (%" PRIu32") exceeds maximum supported. Incompatible host.", numStructureLimits);
+		NETend();
+		return false;
+	}
 	// If there were any changes allocate memory for them
 	if (numStructureLimits)
 	{
 		ingame.structureLimits.resize(numStructureLimits);
+		// If host have limits changed everyone should load limits too
+		// Do not load limits if mods present because mods are not loaded yet
+		if (!modHashesSize && !bLimiterLoaded)
+		{
+			initLoadingScreen(true);
+			if (!resLoad("wrf/limiter_data.wrf", 503))
+			{
+				debug(LOG_INFO, "Unable to load limiter_data during recvOptions!");
+			}
+			else
+			{
+				bLimiterLoaded = true;
+				closeLoadingScreen();
+			}
+		}
 	}
 
+	int nondefaultlimitsize = 0;
 	for (i = 0; i < numStructureLimits; i++)
 	{
 		NETuint32_t(&ingame.structureLimits[i].id);
 		NETuint32_t(&ingame.structureLimits[i].limit);
+		if (bLimiterLoaded && ingame.structureLimits[i].id < numStructureStats)
+		{
+			ASSERT(asStructureStats != nullptr, "numStructureStats > 0, but asStructureStats is null??");
+			if (asStructureStats[ingame.structureLimits[i].id].upgrade[0].limit != ingame.structureLimits[i].limit)
+			{
+				nondefaultlimitsize++;
+			}
+		}
 	}
 	NETuint8_t(&ingame.flags);
 
 	NETend();
 
+	// Do not print limits information if we don't have them loaded
+	if (bLimiterLoaded)
+	{
+		// Check if those vectors are different
+		bool structurelimitsUpdated = (oldStructureLimits.size() != ingame.structureLimits.size()) || (oldStructureLimits != ingame.structureLimits);
+	
+		// Notify if structure limits were changed
+		if (structurelimitsUpdated)
+		{
+			if (nondefaultlimitsize)
+			{
+				addConsoleMessage(astringf(_("Changed structure limits [%d]:"), nondefaultlimitsize).c_str(), DEFAULT_JUSTIFY, SYSTEM_MESSAGE);
+				int changedNum = 1;
+				for (i = 0; i < numStructureLimits; i++)
+				{
+					if (ingame.structureLimits[i].id < numStructureStats)
+					{
+						if (asStructureStats[ingame.structureLimits[i].id].upgrade[0].limit != ingame.structureLimits[i].limit)
+						{
+							WzString structname = asStructureStats[ingame.structureLimits[i].id].name;
+							std::string tmpConsoleMsgStr = astringf(_("[%d] Limit [%s]: %u (default: %u)"), changedNum, structname.toUtf8().c_str(), ingame.structureLimits[i].limit, asStructureStats[ingame.structureLimits[i].id].upgrade[0].limit);
+							addConsoleMessage(tmpConsoleMsgStr.c_str(), DEFAULT_JUSTIFY, SYSTEM_MESSAGE);
+							changedNum++;
+						}
+					}
+					else
+					{
+						std::string tmpConsoleMsgStr = astringf(_("[%d] Limit that is bigger than numStructureStats (%u): %u"), i, ingame.structureLimits[i].id, ingame.structureLimits[i].limit);
+						addConsoleMessage(tmpConsoleMsgStr.c_str(), DEFAULT_JUSTIFY, SYSTEM_MESSAGE);
+					}
+				}
+			}
+			else
+			{
+				addConsoleMessage(_("Limits were reset to default."), DEFAULT_JUSTIFY, SYSTEM_MESSAGE);
+			}
+		}
+	}
+	else
+	{
+		if (numStructureLimits)
+		{
+			std::string tmpConsoleMsgStr = astringf(_("Host initialized %u limits, unable to show them due to mods"), numStructureLimits);
+			addConsoleMessage(tmpConsoleMsgStr.c_str(), DEFAULT_JUSTIFY, SYSTEM_MESSAGE);
+		}
+	}
+
+
 	bool bRebuildMapList = strcmp(game.map, priorGameInfo.map) != 0 || game.hash != priorGameInfo.hash || game.modHashes != priorGameInfo.modHashes;
 	if (bRebuildMapList)
 	{
@@ -385,7 +476,7 @@ bool hostCampaign(const char *SessionName, char *hostPlayerName, bool spectatorH
 	}
 
 	NetPlay.players[selectedPlayer].ready = false;
-	sstrcpy(NetPlay.players[selectedPlayer].name, hostPlayerName);
+	setPlayerName(selectedPlayer, hostPlayerName);
 
 	ingame.localJoiningInProgress = true;
 	ingame.JoiningInProgress[selectedPlayer] = true;
@@ -432,6 +523,8 @@ bool multiShutdown()
 	debug(LOG_MAIN, "free game data (structure limits)");
 	ingame.structureLimits.clear();
 
+	clearDisplayMultiJoiningStatusCache();
+
 	return true;
 }
 
