@@ -25,11 +25,12 @@
  * load / update / store multiplayer statistics for league tables etc...
  */
 
-#include <3rdparty/json/json.hpp> // Must come before WZ includes
+#include <nlohmann/json.hpp> // Must come before WZ includes
 
 #include "lib/framework/file.h"
 #include "lib/framework/frame.h"
 #include "lib/framework/wzapp.h"
+#include "lib/framework/i18n.h"
 #include "lib/netplay/nettypes.h"
 
 #include "activity.h"
@@ -38,12 +39,13 @@
 #include "mission.h" // for cheats
 #include "multistat.h"
 #include "urlrequest.h"
+#include "stdinreader.h"
 
 #include <utility>
 #include <memory>
+#include <chrono>
 #include <SQLiteCpp/SQLiteCpp.h>
 
-
 // ////////////////////////////////////////////////////////////////////////////
 // STATS STUFF
 // ////////////////////////////////////////////////////////////////////////////
@@ -68,6 +70,7 @@ static void NETauto(PLAYERSTATS::Autorating &ar)
 		NETauto(ar.level);
 		NETauto(ar.elo);
 		NETauto(ar.autohoster);
+		NETauto(ar.details);
 	}
 }
 
@@ -82,6 +85,7 @@ PLAYERSTATS::Autorating::Autorating(nlohmann::json const &json)
 		level = json["level"].get<uint8_t>();
 		elo = json["elo"].get<std::string>();
 		autohoster = json["autohoster"].get<bool>();
+		details = json["details"].get<std::string>();
 		valid = true;
 	} catch (const std::exception &e) {
 		debug(LOG_WARNING, "Error parsing rating JSON: %s", e.what());
@@ -96,20 +100,30 @@ void lookupRatingAsync(uint32_t playerIndex)
 	}
 
 	auto hash = playerStats[playerIndex].identity.publicHashString();
-	if (hash.empty())
+	auto key = playerStats[playerIndex].identity.publicKeyHexString();
+	if (hash.empty() || key.empty())
 	{
 		return;
 	}
 
-	std::string url = autoratingUrl(hash);
+	std::string url = getAutoratingUrl();
 	if (url.empty())
+	{
+		setAutoratingUrl(WZ_DEFAULT_PUBLIC_RATING_LOOKUP_SERVICE_URL);
+		url = WZ_DEFAULT_PUBLIC_RATING_LOOKUP_SERVICE_URL;
+	}
+
+	if (!getAutoratingEnable())
 	{
 		return;
 	}
 
 	URLDataRequest req;
 	req.url = url;
-	debug(LOG_INFO, "Requesting \"%s\"", req.url.c_str());
+	req.setRequestHeader("WZ-Player-Hash", hash);
+	req.setRequestHeader("WZ-Player-Key", key);
+	req.setRequestHeader("WZ-Locale", getLanguage());
+	debug(LOG_INFO, "Requesting \"%s\" for player %d (%.32s) (%s)", req.url.c_str(), playerIndex, NetPlay.players[playerIndex].name, hash.c_str());
 	req.onSuccess = [playerIndex, hash](std::string const &url, HTTPResponseDetails const &response, std::shared_ptr<MemoryStruct> const &data) {
 		long httpStatusCode = response.httpStatusCode();
 		std::string urlCopy = url;
@@ -130,9 +144,11 @@ void lookupRatingAsync(uint32_t playerIndex)
 			}
 			try {
 				playerStats[playerIndex].autorating = nlohmann::json::parse(dataCopy->memory, dataCopy->memory + dataCopy->size);
+				playerStats[playerIndex].autoratingFrom = RATING_SOURCE_LOCAL;
 				if (playerStats[playerIndex].autorating.valid)
 				{
-					setMultiStats(playerIndex, playerStats[playerIndex], false);
+					setMultiStats(playerIndex, playerStats[playerIndex], !NetPlay.isHost);
+					netPlayersUpdated = true;
 				}
 			}
 			catch (const std::exception &e) {
@@ -220,7 +236,9 @@ void recvMultiStats(NETQUEUE queue)
 		return;
 	}
 
-	NETauto(playerStats[playerIndex].autorating);
+	PLAYERSTATS::Autorating receivedAutorating;
+	NETauto(receivedAutorating);
+	bool processAutoratingData = false;
 
 	// we don't what to update ourselves, we already know our score (FIXME: rewrite setMultiStats())
 	if (!myResponsibility(playerIndex))
@@ -251,14 +269,46 @@ void recvMultiStats(NETQUEUE queue)
 		{
 			ingame.PingTimes[playerIndex] = PING_LIMIT;
 			ingame.VerifiedIdentity[playerIndex] = false;
+
+			if (!ingame.muteChat[playerIndex])
+			{
+				// check if the new identity was previously muted
+				auto playerOptions = getStoredPlayerOptions(NetPlay.players[playerIndex].name, playerStats[playerIndex].identity);
+				if (playerOptions.has_value() && playerOptions.value().mutedTime.has_value())
+				{
+					ingame.muteChat[playerIndex] = (playerOptions.value().mutedTime.value().time_since_epoch().count() > 0);
+				}
+			}
+
+			// Output to stdinterface, if enabled
+			std::string senderPublicKeyB64 = base64Encode(playerStats[playerIndex].identity.toBytes(EcKey::Public));
+			std::string senderIdentityHash = playerStats[playerIndex].identity.publicHashString();
+			wz_command_interface_output("WZEVENT: player identity UNVERIFIED: %" PRIu32 " %s %s\n", playerIndex, senderPublicKeyB64.c_str(), senderIdentityHash.c_str());
+
+			processAutoratingData = true;
 		}
 	}
-	NETend();
-
-	if (NetPlay.isHost && !playerStats[playerIndex].autorating.valid)
+	else
 	{
-		lookupRatingAsync(playerIndex);
+		processAutoratingData = true;
 	}
+
+	if (processAutoratingData)
+	{
+		if (getAutoratingEnable())
+		{
+			playerStats[playerIndex].autorating.valid = false;
+			playerStats[playerIndex].autoratingFrom = RATING_SOURCE_LOCAL;
+			lookupRatingAsync(playerIndex);
+		}
+		else
+		{
+			playerStats[playerIndex].autorating = receivedAutorating;
+			playerStats[playerIndex].autoratingFrom = RATING_SOURCE_HOST;
+		}
+	}
+
+	NETend();
 }
 
 // ////////////////////////////////////////////////////////////////////////////
@@ -346,13 +396,6 @@ bool loadMultiStats(char *sPlayerName, PLAYERSTATS *st)
 	st->recentScore = 0;
 	st->recentPowerLost = 0;
 
-	// clear any skirmish stats.
-	for (size_t size = 0; size < MAX_PLAYERS; size++)
-	{
-		ingame.skScores[size][0] = 0;
-		ingame.skScores[size][1] = 0;
-	}
-
 	return true;
 }
 
@@ -393,21 +436,16 @@ void updateMultiStatsDamage(UDWORD attacker, UDWORD defender, UDWORD inflicted)
 
 	if (NetPlay.bComms)
 	{
-		// killing and getting killed by scavengers does not influence scores in MP games
-		if (attacker != scavengerSlot() && defender != scavengerSlot())
+		if ((attacker == scavengerSlot()) || (defender == scavengerSlot()))
 		{
-			// FIXME: Why in the world are we using two different structs for stats when we can use only one?
-			playerStats[attacker].totalScore  += 2 * inflicted;
-			playerStats[attacker].recentScore += 2 * inflicted;
-			playerStats[defender].totalScore  -= inflicted;
-			playerStats[defender].recentScore -= inflicted;
+			return; // damaging and getting damaged by scavengers does not influence scores in MP games
 		}
+		playerStats[attacker].totalScore  += 2 * inflicted;
+		playerStats[defender].totalScore  -= inflicted;
 	}
-	else
-	{
-		ingame.skScores[attacker][0] += 2 * inflicted;  // increment skirmish players rough score.
-		ingame.skScores[defender][0] -= inflicted;  // increment skirmish players rough score.
-	}
+
+	playerStats[attacker].recentScore += 2 * inflicted;
+	playerStats[defender].recentScore -= inflicted;
 }
 
 // update games played.
@@ -464,27 +502,28 @@ static inline uint32_t calcObjectCost(const BASE_OBJECT *psObj)
 // update kills
 void updateMultiStatsKills(BASE_OBJECT *psKilled, UDWORD player)
 {
-	if (player < MAX_PLAYERS)
+	if (player >= MAX_PLAYERS)
 	{
-		if (NetPlay.bComms)
-		{
-			// killing scavengers does not count in MP games
-			if (psKilled != nullptr && psKilled->player != scavengerSlot())
-			{
-				// FIXME: Why in the world are we using two different structs for stats when we can use only one?
-				++playerStats[player].totalKills;
-				++playerStats[player].recentKills;
-				if (psKilled->player < MAX_PLAYERS)
-				{
-					playerStats[psKilled->player].recentPowerLost += static_cast<uint64_t>(calcObjectCost(psKilled));
-				}
-			}
-		}
-		else
-		{
-			ingame.skScores[player][1]++;
-		}
+		return;
 	}
+	if (NetPlay.bComms)
+	{
+		if (psKilled != nullptr)
+		{
+			if (psKilled->player == scavengerSlot())
+			{
+				return; // killing scavengers does not count in MP games
+			}
+			if (psKilled->player < MAX_PLAYERS)
+			{
+				playerStats[psKilled->player].recentPowerLost += static_cast<uint64_t>(calcObjectCost(psKilled));
+			}
+			++playerStats[player].totalKills;
+			++playerStats[player].recentKills;
+		}
+		return;
+	}
+	++playerStats[player].recentKills;
 }
 
 class KnownPlayersDB {
@@ -495,6 +534,13 @@ public:
 		EcKey::Key pk;
 	};
 
+	struct PlayerOptions {
+		std::string name;
+		EcKey::Key pk;
+		optional<std::chrono::system_clock::time_point> mutedTime;
+		optional<std::chrono::system_clock::time_point> bannedTime;
+	};
+
 public:
 	// Caller is expected to handle thrown exceptions
 	KnownPlayersDB(const std::string& knownPlayersDBPath)
@@ -502,13 +548,16 @@ public:
 		db = std::unique_ptr<SQLite::Database>(new SQLite::Database(knownPlayersDBPath, SQLite::OPEN_READWRITE | SQLite::OPEN_CREATE));
 		db->exec("PRAGMA journal_mode=WAL");
 		createKnownPlayersDBTables();
-		query_findPlayerByName = std::unique_ptr<SQLite::Statement>(new SQLite::Statement(*db, "SELECT local_id, name, pk FROM known_players WHERE name = ?"));
+		query_findPlayerIdentityByName = std::unique_ptr<SQLite::Statement>(new SQLite::Statement(*db, "SELECT local_id, name, pk FROM known_players WHERE name = ?"));
 		query_insertNewKnownPlayer = std::unique_ptr<SQLite::Statement>(new SQLite::Statement(*db, "INSERT OR IGNORE INTO known_players(name, pk) VALUES(?, ?)"));
 		query_updateKnownPlayerKey = std::unique_ptr<SQLite::Statement>(new SQLite::Statement(*db, "UPDATE known_players SET pk = ? WHERE name = ?"));
+		query_findPlayerOptionsByPK = std::unique_ptr<SQLite::Statement>(new SQLite::Statement(*db, "SELECT name, muted, banned FROM player_options WHERE pk = ?"));
+		query_insertNewPlayerOptions = std::unique_ptr<SQLite::Statement>(new SQLite::Statement(*db, "INSERT OR IGNORE INTO player_options(pk, name, muted, banned) VALUES(?, ?, ?, ?)"));
+		query_updatePlayerOptionsMuted = std::unique_ptr<SQLite::Statement>(new SQLite::Statement(*db, "UPDATE player_options SET muted = ? WHERE pk = ? AND name = ?"));
 	}
 
 public:
-	optional<PlayerInfo> findPlayerByName(const std::string& name)
+	optional<PlayerInfo> findPlayerIdentityByName(const std::string& name)
 	{
 		if (name.empty())
 		{
@@ -522,13 +571,13 @@ public:
 		}
 		optional<PlayerInfo> result;
 		try {
-			query_findPlayerByName->bind(1, name);
-			if (query_findPlayerByName->executeStep())
+			query_findPlayerIdentityByName->bind(1, name);
+			if (query_findPlayerIdentityByName->executeStep())
 			{
 				PlayerInfo data;
-				data.local_id = query_findPlayerByName->getColumn(0).getInt64();
-				data.name = query_findPlayerByName->getColumn(1).getString();
-				std::string publicKeyb64 = query_findPlayerByName->getColumn(2).getString();
+				data.local_id = query_findPlayerIdentityByName->getColumn(0).getInt64();
+				data.name = query_findPlayerIdentityByName->getColumn(1).getString();
+				std::string publicKeyb64 = query_findPlayerIdentityByName->getColumn(2).getString();
 				data.pk = base64Decode(publicKeyb64);
 				result = data;
 			}
@@ -538,7 +587,7 @@ public:
 			result = nullopt;
 		}
 		try {
-			query_findPlayerByName->reset();
+			query_findPlayerIdentityByName->reset();
 		}
 		catch (const std::exception& e) {
 			debug(LOG_ERROR, "Failed to reset prepared statement; error: %s", e.what());
@@ -549,7 +598,7 @@ public:
 	}
 
 	// Note: May throw on database error!
-	void addKnownPlayer(std::string const &name, EcKey const &key, bool overrideCurrentKey)
+	void addKnownPlayerIdentity(std::string const &name, EcKey const &key, bool overrideCurrentKey)
 	{
 		if (key.empty())
 		{
@@ -582,14 +631,118 @@ public:
 		transaction.commit();
 	}
 
+public:
+	std::vector<PlayerOptions> findPlayerOptions(const std::string& name, EcKey const &key)
+	{
+		if (key.empty())
+		{
+			return {};
+		}
+
+		std::string publicKeyb64 = base64Encode(key.toBytes(EcKey::Public));
+
+		std::vector<PlayerOptions> result;
+		try {
+			query_findPlayerOptionsByPK->bind(1, publicKeyb64);
+			while (query_findPlayerOptionsByPK->executeStep())
+			{
+				PlayerOptions data;
+				data.name = query_findPlayerOptionsByPK->getColumn(0).getString();
+				data.pk = key.toBytes(EcKey::Public);
+				auto mutedTime = std::chrono::seconds(query_findPlayerOptionsByPK->getColumn(1).getInt64());
+				if (mutedTime.count() > 0)
+				{
+					data.mutedTime = std::chrono::system_clock::time_point(mutedTime);
+				}
+				auto bannedTime = std::chrono::seconds(query_findPlayerOptionsByPK->getColumn(2).getInt64());
+				if (bannedTime.count() > 0)
+				{
+					data.bannedTime = std::chrono::system_clock::time_point(bannedTime);
+				}
+				if (data.name == name)
+				{
+					// if we find an exact match (for both pk *and* name), use it!
+					result.clear();
+					result.push_back(data);
+					break;
+				}
+				result.push_back(data);
+			}
+		}
+		catch (const std::exception& e) {
+			debug(LOG_ERROR, "Failure to query database for player; error: %s", e.what());
+			result.clear();
+		}
+		try {
+			query_findPlayerOptionsByPK->reset();
+		}
+		catch (const std::exception& e) {
+			debug(LOG_ERROR, "Failed to reset prepared statement; error: %s", e.what());
+		}
+
+		return result;
+	}
+
+	// Note: May throw on database error!
+	void setPlayerMuted(std::string const &name, EcKey const &key, optional<std::chrono::system_clock::time_point> mutedTime)
+	{
+		std::string publicKeyb64;
+		if (!key.empty())
+		{
+			publicKeyb64 = base64Encode(key.toBytes(EcKey::Public));
+		}
+
+		// Begin transaction
+		SQLite::Transaction transaction(*db);
+
+		auto mutedTimeValue = mutedTime.has_value() ? mutedTime.value().time_since_epoch().count() : 0;
+
+		query_insertNewPlayerOptions->bind(1, publicKeyb64);
+		query_insertNewPlayerOptions->bind(2, name);
+		query_insertNewPlayerOptions->bind(3, mutedTimeValue);
+		query_insertNewPlayerOptions->bind(4, -1); // default "never set" is -1, not 0
+		if (query_insertNewPlayerOptions->exec() == 0)
+		{
+			query_updatePlayerOptionsMuted->bind(1, mutedTimeValue);
+			query_updatePlayerOptionsMuted->bind(2, publicKeyb64);
+			query_updatePlayerOptionsMuted->bind(3, name);
+			if (query_updatePlayerOptionsMuted->exec() == 0)
+			{
+				debug(LOG_WARNING, "Failed to update player_options (%s)", name.c_str());
+			}
+			query_updatePlayerOptionsMuted->reset();
+		}
+		query_insertNewPlayerOptions->reset();
+
+		// Commit transaction
+		transaction.commit();
+	}
+
 private:
 	void createKnownPlayersDBTables()
 	{
 		SQLite::Transaction transaction(*db);
 		if (!db->tableExists("known_players"))
 		{
+			// used to store association of name and known player key
 			db->exec("CREATE TABLE known_players (local_id INTEGER PRIMARY KEY, name TEXT UNIQUE, pk TEXT)");
 		}
+		int dbVersion = db->execAndGet("PRAGMA user_version").getInt();
+		// database schema upgrades
+		switch (dbVersion)
+		{
+			case 0:
+				// used to store player_options that may be associated with a player identity
+				// NOTE: In this case, "name" does *NOT* have a unique constraint
+				db->exec("CREATE TABLE player_options (local_id INTEGER PRIMARY KEY, pk TEXT NOT NULL, name TEXT NOT NULL, muted INTEGER, banned INTEGER)");
+				db->exec("CREATE UNIQUE INDEX idx_player_options_pk_name ON player_options (pk, name)");
+				db->exec("PRAGMA user_version = 1");
+				dbVersion = 1;
+				// fall-through
+			default:
+				// done
+				break;
+		}
 		transaction.commit();
 	}
 
@@ -605,10 +758,13 @@ private:
 
 private:
 	std::unique_ptr<SQLite::Database> db; // Must be the first-listed member variable so it is destructed last
-	std::unique_ptr<SQLite::Statement> query_findPlayerByName;
+	std::unique_ptr<SQLite::Statement> query_findPlayerIdentityByName;
 	std::unique_ptr<SQLite::Statement> query_insertNewKnownPlayer;
 	std::unique_ptr<SQLite::Statement> query_updateKnownPlayerKey;
 	std::unordered_map<std::string, std::pair<optional<PlayerInfo>, UDWORD>> findPlayerCache;
+	std::unique_ptr<SQLite::Statement> query_findPlayerOptionsByPK;
+	std::unique_ptr<SQLite::Statement> query_insertNewPlayerOptions;
+	std::unique_ptr<SQLite::Statement> query_updatePlayerOptionsMuted;
 	UDWORD lastCacheClean = 0;
 };
 
@@ -644,7 +800,7 @@ bool isLocallyKnownPlayer(std::string const &name, EcKey const &key)
 	{
 		return false;
 	}
-	auto result = knownPlayersDB->findPlayerByName(name);
+	auto result = knownPlayersDB->findPlayerIdentityByName(name);
 	if (!result.has_value())
 	{
 		return false;
@@ -661,26 +817,71 @@ void addKnownPlayer(std::string const &name, EcKey const &key, bool override)
 	ASSERT_OR_RETURN(, knownPlayersDB.operator bool(), "knownPlayersDB is uninitialized");
 
 	try {
-		knownPlayersDB->addKnownPlayer(name, key, override);
+		knownPlayersDB->addKnownPlayerIdentity(name, key, override);
 	}
 	catch (const std::exception& e) {
 		debug(LOG_ERROR, "Failed to add known_player with error: %s", e.what());
 	}
 }
 
+optional<StoredPlayerOptions> getStoredPlayerOptions(std::string const &name, EcKey const &key)
+{
+	ASSERT_OR_RETURN(nullopt, knownPlayersDB.operator bool(), "knownPlayersDB is uninitialized");
+	auto results = knownPlayersDB->findPlayerOptions(name, key);
+	if (results.empty())
+	{
+		return nullopt;
+	}
+	// findPlayerOptions may have returned more than one result, so sort through and find the "newest" for each setting
+	StoredPlayerOptions playerOptions;
+	for (const auto& result : results)
+	{
+		if (result.mutedTime.has_value())
+		{
+			playerOptions.mutedTime = std::max(result.mutedTime.value(), playerOptions.mutedTime.value_or(std::chrono::system_clock::time_point()));
+		}
+		if (result.bannedTime.has_value())
+		{
+			playerOptions.bannedTime = std::max(result.bannedTime.value(), playerOptions.bannedTime.value_or(std::chrono::system_clock::time_point()));
+		}
+	}
+	return playerOptions;
+}
+
+void storePlayerMuteOption(std::string const &name, EcKey const &key, bool muted)
+{
+	ASSERT_OR_RETURN(, knownPlayersDB.operator bool(), "knownPlayersDB is uninitialized");
+	optional<std::chrono::system_clock::time_point> mutedTime;
+	if (muted)
+	{
+		mutedTime = std::chrono::system_clock::now();
+	}
+	try {
+		knownPlayersDB->setPlayerMuted(name, key, mutedTime);
+	}
+	catch (const std::exception& e) {
+		debug(LOG_ERROR, "Failed to store player mute option with error: %s", e.what());
+	}
+}
+
 uint32_t getMultiPlayUnitsKilled(uint32_t player)
 {
-	// Let's use the real score for MP games
-	// FIXME: Why in the world are we using two different structs for stats when we can use only one?
-	if (NetPlay.bComms)
-	{
-		return getMultiStats(player).recentKills;
-	}
-	else
-	{
-		// estimated kills
-		return static_cast<uint32_t>(ingame.skScores[player][1]);
-	}
+	return getMultiStats(player).recentKills;
+}
+
+void setMultiPlayUnitsKilled(uint32_t player, uint32_t kills)
+{
+	playerStats[player].recentKills = kills;
+}
+
+uint32_t getMultiPlayRecentScore(uint32_t player)
+{
+	return getMultiStats(player).recentScore;
+}
+
+void setMultiPlayRecentScore(uint32_t player, uint32_t score)
+{
+	playerStats[player].recentScore = score;
 }
 
 uint32_t getSelectedPlayerUnitsKilled()
@@ -695,6 +896,18 @@ uint32_t getSelectedPlayerUnitsKilled()
 	}
 }
 
+void resetRecentScoreData()
+{
+	for (unsigned int i = 0; i < MAX_CONNECTED_PLAYERS; ++i)
+	{
+		playerStats[i].recentKills = 0;
+		playerStats[i].recentScore = 0;
+		playerStats[i].recentPowerLost = 0;
+		playerStats[i].identity.clear();
+		playerStats[i].autorating = PLAYERSTATS::Autorating();
+	}
+}
+
 // MARK: -
 
 inline void to_json(nlohmann::json& j, const EcKey& k) {
@@ -774,12 +987,5 @@ bool loadMultiStatsFromJSON(const nlohmann::json& json)
 		playerStats[idx] = json.at(idx).get<PLAYERSTATS>();
 	}
 
-	// clear any skirmish stats.
-	for (size_t size = 0; size < MAX_PLAYERS; size++)
-	{
-		ingame.skScores[size][0] = 0;
-		ingame.skScores[size][1] = 0;
-	}
-
 	return true;
 }
