@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2019  Warzone 2100 Project
+	Copyright (C) 2005-2020  Warzone 2100 Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -56,6 +56,7 @@
 #include "multistat.h"
 #include "mapgrid.h"
 #include "random.h"
+#include "display3d.h"
 
 #include <algorithm>
 #include <functional>
@@ -76,14 +77,27 @@ struct INTERVAL
 	int begin, end;  // Time 1 = 0, time 2 = 1024. Or begin >= end if empty.
 };
 
+struct DAMAGE
+{
+	PROJECTILE *psProjectile;
+	BASE_OBJECT *psDest;
+	unsigned damage;
+	WEAPON_CLASS weaponClass;
+	WEAPON_SUBCLASS weaponSubClass;
+	unsigned impactTime;
+	bool isDamagePerSecond;
+	int minDamage;
+};
+
 // Watermelon:they are from droid.c
 /* The range for neighbouring objects */
 #define PROJ_NEIGHBOUR_RANGE (TILE_UNITS*4)
 // used to create a specific ID for projectile objects to facilitate tracking them.
-static const UDWORD ProjectileTrackerID =	0xdead0000;
+static const uint32_t ProjectileTrackerID = 0xdead0000;
+static uint32_t projectileTrackerIDIncrement = 0;
 
 /* The list of projectiles in play */
-static std::vector<PROJECTILE *> psProjectileList;
+static std::vector<PROJECTILE*> psProjectileList;
 
 /* The next projectile to give out in the proj_First / proj_Next methods */
 static ProjectileIterator psProjectileNext;
@@ -99,7 +113,7 @@ static void	proj_ImpactFunc(PROJECTILE *psObj);
 static void	proj_PostImpactFunc(PROJECTILE *psObj);
 static void proj_checkPeriodicalDamage(PROJECTILE *psProj);
 
-static int32_t objectDamage(BASE_OBJECT *psObj, unsigned damage, WEAPON_CLASS weaponClass, WEAPON_SUBCLASS weaponSubClass, unsigned impactTime, bool isDamagePerSecond, int minDamage);
+static int32_t objectDamage(DAMAGE *psDamage);
 
 
 static inline void setProjectileDestination(PROJECTILE *psProj, BASE_OBJECT *psObj)
@@ -140,7 +154,7 @@ bool gfxVisible(PROJECTILE *psObj)
 	    && psObj->psSource->player != selectedPlayer
 	    && (psObj->psDest == nullptr
 	        || psObj->psDest->died
-	        || !psObj->psDest->visible[selectedPlayer]))
+	        || !psObj->psDest->visibleForLocalDisplay()))
 	{
 		return false;
 	}
@@ -151,7 +165,7 @@ bool gfxVisible(PROJECTILE *psObj)
 	    && psObj->psDest->type == OBJ_STRUCTURE
 	    && psObj->psDest->player != selectedPlayer
 	    && (psObj->psSource == nullptr
-	        || !psObj->psSource->visible[selectedPlayer]))
+	        || !psObj->psSource->visibleForLocalDisplay()))
 	{
 		return false;
 	}
@@ -159,7 +173,7 @@ bool gfxVisible(PROJECTILE *psObj)
 	// You can see the source
 	if (psObj->psSource != nullptr
 	    && !psObj->psSource->died
-	    && psObj->psSource->visible[selectedPlayer])
+	    && psObj->psSource->visibleForLocalDisplay())
 	{
 		return true;
 	}
@@ -167,7 +181,7 @@ bool gfxVisible(PROJECTILE *psObj)
 	// You can see the destination
 	if (psObj->psDest != nullptr
 	    && !psObj->psDest->died
-	    && psObj->psDest->visible[selectedPlayer])
+	    && psObj->psDest->visibleForLocalDisplay())
 	{
 		return true;
 	}
@@ -186,6 +200,7 @@ proj_InitSystem()
 	{
 		experienceGain[x] = 100;
 	}
+	projectileTrackerIDIncrement = 0;
 	return true;
 }
 
@@ -196,6 +211,10 @@ proj_InitSystem()
 void
 proj_FreeAllProjectiles()
 {
+	for (auto proj: psProjectileList)
+	{
+		delete proj;
+	}
 	psProjectileList.clear();
 	psProjectileNext = psProjectileList.end();
 }
@@ -257,29 +276,24 @@ int getExpGain(int player)
 	return experienceGain[player];
 }
 
-// update the kills after a target is damaged/destroyed
-static void proj_UpdateKills(PROJECTILE *psObj, int32_t experienceInc)
+DROID *getDesignatorAttackingObject(int player, BASE_OBJECT *target)
+{
+	DROID* psCommander = cmdDroidGetDesignator(player);
+
+	return psCommander != nullptr && psCommander->action == DACTION_ATTACK && psCommander->psActionTarget[0] == target
+		? psCommander
+		: nullptr;
+}
+
+
+// update the source experience after a target is damaged/destroyed
+static void proj_UpdateExperience(PROJECTILE *psObj, uint32_t experienceInc)
 {
 	DROID	        *psDroid;
 	BASE_OBJECT     *psSensor;
 
 	CHECK_PROJECTILE(psObj);
 
-	if (psObj->psSource == nullptr || (psObj->psDest && psObj->psDest->type == OBJ_FEATURE)
-	    || (psObj->psDest && psObj->psSource->player == psObj->psDest->player))	// no exp for friendly fire
-	{
-		return;
-	}
-
-	// If experienceInc is negative then the target was killed
-	if (bMultiPlayer && experienceInc < 0)
-	{
-		updateMultiStatsKills(psObj->psDest, psObj->psSource->player);
-	}
-
-	// Since we are no longer interested if it was killed or not, abs it
-	experienceInc = abs(experienceInc) * getExpGain(psObj->psSource->player) / 100;
-
 	if (psObj->psSource->type == OBJ_DROID)			/* update droid kills */
 	{
 		psDroid = (DROID *) psObj->psSource;
@@ -294,10 +308,10 @@ static void proj_UpdateKills(PROJECTILE *psObj, int32_t experienceInc)
 			experienceInc = (uint64_t)experienceInc * qualityFactor(psDroid, (DROID *)psObj->psDest) / 65536;
 		}
 
-		ASSERT_OR_RETURN(, 0 <= experienceInc && experienceInc < (int)(2.1 * 65536), "Experience increase out of range");
+		ASSERT_OR_RETURN(, experienceInc < (int)(2.1 * 65536), "Experience increase out of range");
 
 		psDroid->experience += experienceInc;
-		cmdDroidUpdateKills(psDroid, experienceInc);
+		cmdDroidUpdateExperience(psDroid, experienceInc);
 
 		psSensor = orderStateObj(psDroid, DORDER_FIRESUPPORT);
 		if (psSensor
@@ -308,14 +322,11 @@ static void proj_UpdateKills(PROJECTILE *psObj, int32_t experienceInc)
 	}
 	else if (psObj->psSource->type == OBJ_STRUCTURE)
 	{
-		ASSERT_OR_RETURN(, 0 <= experienceInc && experienceInc < (int)(2.1 * 65536), "Experience increase out of range");
+		ASSERT_OR_RETURN(, experienceInc < (int)(2.1 * 65536), "Experience increase out of range");
 
-		// See if there was a command droid designating this target
-		psDroid = cmdDroidGetDesignator(psObj->psSource->player);
+		psDroid = getDesignatorAttackingObject(psObj->psSource->player, psObj->psDest);
 
-		if (psDroid != nullptr
-		    && psDroid->action == DACTION_ATTACK
-		    && psDroid->psActionTarget[0] == psObj->psDest)
+		if (psDroid != nullptr)
 		{
 			psDroid->experience += experienceInc;
 		}
@@ -413,7 +424,7 @@ bool proj_SendProjectileAngled(WEAPON *psWeap, SIMPLE_OBJECT *psAttacker, int pl
 	ASSERT_OR_RETURN(false, psStats != nullptr, "Invalid weapon stats");
 	ASSERT_OR_RETURN(false, psTarget == nullptr || !psTarget->died, "Aiming at dead target!");
 
-	PROJECTILE *psProj = new PROJECTILE(ProjectileTrackerID | (realTime >> 4), player);
+	PROJECTILE *psProj = new PROJECTILE(ProjectileTrackerID + ++projectileTrackerIDIncrement, player);
 
 	/* get muzzle offset */
 	if (psAttacker == nullptr)
@@ -874,8 +885,8 @@ static void proj_InFlightFunc(PROJECTILE *psProj)
 		}
 		setProjectileDestination(psProj, closestCollisionObject);  // We hit something.
 
-		// Buildings and terrain cannot be penetrated and we need a penetrating weapon.
-		if (closestCollisionObject != nullptr && closestCollisionObject->type == OBJ_DROID && psStats->penetrate)
+		// Buildings and terrain cannot be penetrated and we need a penetrating weapon, and projectile should not have already travelled further than 1.25 * maximum range.
+		if (closestCollisionObject != nullptr && closestCollisionObject->type == OBJ_DROID && psStats->penetrate && currentDistance < static_cast<int>(1.25 * proj_GetLongRange(psStats, psProj->player)))
 		{
 			WEAPON asWeap;
 			asWeap.nStat = psStats - asWeaponStats;
@@ -891,7 +902,7 @@ static void proj_InFlightFunc(PROJECTILE *psProj)
 		return;
 	}
 
-	if (currentDistance * 100u >= proj_GetLongRange(psStats, psProj->player) * psStats->distanceExtensionFactor)
+	if (currentDistance * 100 >= proj_GetLongRange(psStats, psProj->player) * psStats->distanceExtensionFactor)
 	{
 		// We've travelled our maximum range.
 		psProj->state = PROJ_IMPACT;
@@ -998,6 +1009,11 @@ static void proj_ImpactFunc(PROJECTILE *psObj)
 			position.z = psObj->pos.y;  // z = y [sic] intentional
 			position.y = map_Height(position.x, position.z);
 			addEffect(&position, EFFECT_SAT_LASER, SAT_LASER_STANDARD, false, nullptr, 0, psObj->time);
+
+			if (clipXY(psObj->pos.x, psObj->pos.y))
+			{
+				shakeStart(1800); // takes out lots of stuff so shake length is greater
+			}
 		}
 	}
 
@@ -1121,11 +1137,19 @@ static void proj_ImpactFunc(PROJECTILE *psObj)
 			debug(LOG_NEVER, "Damage to object %d, player %d\n",
 			      psObj->psDest->id, psObj->psDest->player);
 
-			// Damage the object
-			relativeDamage = objectDamage(psObj->psDest, damage, psStats->weaponClass, psStats->weaponSubClass,
-			                              psObj->time, false, psStats->upgrade[psObj->player].minimumDamage);
+			struct DAMAGE sDamage = {
+				psObj,
+				psObj->psDest,
+				damage,
+				psStats->weaponClass,
+				psStats->weaponSubClass,
+				psObj->time,
+				false,
+				(int)psStats->upgrade[psObj->player].minimumDamage
+			};
 
-			proj_UpdateKills(psObj, relativeDamage);
+			// Damage the object
+			relativeDamage = objectDamage(&sDamage);
 
 			if (relativeDamage >= 0)	// So long as the target wasn't killed
 			{
@@ -1155,8 +1179,13 @@ static void proj_ImpactFunc(PROJECTILE *psObj)
 		/* Note when it exploded for the explosion effect */
 		psObj->born = gameTime;
 
+		// If projectile impacts a droid start the splash damage from the center of it, else use whatever location the projectile impacts at as the splash center.
+		auto destDroid = castDroid(psObj->psDest);
+		Vector3i targetPos = (destDroid != nullptr) ? destDroid->pos : psObj->pos;
+
 		static GridList gridList;  // static to avoid allocations.
-		gridList = gridStartIterate(psObj->pos.x, psObj->pos.y, psStats->upgrade[psObj->player].radius);
+		gridList = gridStartIterate(targetPos.x, targetPos.y, psStats->upgrade[psObj->player].radius);
+
 		for (GridIterator gi = gridList.begin(); gi != gridList.end(); ++gi)
 		{
 			BASE_OBJECT *psCurr = *gi;
@@ -1202,7 +1231,7 @@ static void proj_ImpactFunc(PROJECTILE *psObj)
 			{
 				continue;  // Target in air, and can't shoot at air, or target on ground, and can't shoot at ground.
 			}
-			if (useSphere && !Vector3i_InSphere(psCurr->pos, psObj->pos, psStats->upgrade[psObj->player].radius))
+			if (useSphere && !Vector3i_InSphere(psCurr->pos, targetPos, psStats->upgrade[psObj->player].radius))
 			{
 				continue;  // Target out of range.
 			}
@@ -1213,9 +1242,19 @@ static void proj_ImpactFunc(PROJECTILE *psObj)
 			{
 				updateMultiStatsDamage(psObj->psSource->player, psCurr->player, damage);
 			}
-			int relativeDamage = objectDamage(psCurr, damage, psStats->weaponClass, psStats->weaponSubClass,
-			                                  psObj->time, false, psStats->upgrade[psObj->player].minimumDamage);
-			proj_UpdateKills(psObj, relativeDamage);
+
+			struct DAMAGE sDamage = {
+				psObj,
+				psCurr,
+				damage,
+				psStats->weaponClass,
+				psStats->weaponSubClass,
+				psObj->time,
+				false,
+				(int)psStats->upgrade[psObj->player].minimumDamage
+			};
+
+			objectDamage(&sDamage);
 		}
 	}
 
@@ -1283,7 +1322,7 @@ void PROJECTILE::update()
 		setProjectileDestination(psObj, nullptr);
 	}
 	// Remove dead objects from psDamaged.
-	psDamaged.erase(std::remove_if(psDamaged.begin(), psDamaged.end(), std::ptr_fun(&::isDead)), psDamaged.end());
+	psDamaged.erase(std::remove_if(psDamaged.begin(), psDamaged.end(), [](const BASE_OBJECT *psObj) { return ::isDead(psObj); }), psDamaged.end());
 
 	// This extra check fixes a crash in cam2, mission1
 	if (worldOnMap(psObj->pos.x, psObj->pos.y) == false)
@@ -1328,10 +1367,10 @@ void proj_UpdateAll()
 	std::vector<PROJECTILE *> psProjectileListOld = psProjectileList;
 
 	// Update all projectiles. Penetrating projectiles may add to psProjectileList.
-	std::for_each(psProjectileListOld.begin(), psProjectileListOld.end(), std::mem_fun(&PROJECTILE::update));
+	std::for_each(psProjectileListOld.begin(), psProjectileListOld.end(), std::mem_fn(&PROJECTILE::update));
 
 	// Remove and free dead projectiles.
-	psProjectileList.erase(std::remove_if(psProjectileList.begin(), psProjectileList.end(), std::mem_fun(&PROJECTILE::deleteIfDead)), psProjectileList.end());
+	psProjectileList.erase(std::remove_if(psProjectileList.begin(), psProjectileList.end(), std::mem_fn(&PROJECTILE::deleteIfDead)), psProjectileList.end());
 }
 
 /***************************************************************************/
@@ -1381,10 +1420,18 @@ static void proj_checkPeriodicalDamage(PROJECTILE *psProj)
 		unsigned damageRate = calcDamage(weaponPeriodicalDamage(psStats, psProj->player), psStats->periodicalDamageWeaponEffect, psCurr);
 		debug(LOG_NEVER, "Periodical damage of %d per second to object %d, player %d\n", damageRate, psCurr->id, psCurr->player);
 
-		int relativeDamage = objectDamage(psCurr, damageRate, psStats->periodicalDamageWeaponClass,
-		                                  psStats->periodicalDamageWeaponSubClass, gameTime - deltaGameTime / 2 + 1, true,
-		                                  psStats->upgrade[psProj->player].minimumDamage);
-		proj_UpdateKills(psProj, relativeDamage);
+		struct DAMAGE sDamage = {
+			psProj,
+			psCurr,
+			damageRate,
+			psStats->periodicalDamageWeaponClass,
+			psStats->periodicalDamageWeaponSubClass,
+			gameTime - deltaGameTime / 2 + 1,
+			true,
+			(int)psStats->upgrade[psProj->player].minimumDamage
+		};
+
+		objectDamage(&sDamage);
 	}
 }
 
@@ -1410,21 +1457,27 @@ bool proj_Direct(const WEAPON_STATS *psStats)
 
 /***************************************************************************/
 
+#define ASSERT_PLAYER_OR_RETURN(retVal, player) \
+	ASSERT_OR_RETURN(retVal, player >= 0 && player < MAX_PLAYERS, "Invalid player: %" PRIu32 "", player);
+
 // return the maximum range for a weapon
 int proj_GetLongRange(const WEAPON_STATS *psStats, int player)
 {
+	ASSERT_PLAYER_OR_RETURN(0, player);
 	return psStats->upgrade[player].maxRange;
 }
 
 // return the minimum range for a weapon
 int proj_GetMinRange(const WEAPON_STATS *psStats, int player)
 {
+	ASSERT_PLAYER_OR_RETURN(0, player);
 	return psStats->upgrade[player].minRange;
 }
 
 // return the short range for a weapon
 int proj_GetShortRange(const WEAPON_STATS *psStats, int player)
 {
+	ASSERT_PLAYER_OR_RETURN(0, player);
 	return psStats->upgrade[player].shortRange;
 }
 
@@ -1477,6 +1530,11 @@ ObjectShape establishTargetShape(BASE_OBJECT *psTarget)
 structure strength*/
 UDWORD	calcDamage(UDWORD baseDamage, WEAPON_EFFECT weaponEffect, BASE_OBJECT *psTarget)
 {
+	if (baseDamage == 0)
+	{
+		return 0;
+	}
+
 	UDWORD	damage = baseDamage * 100;
 
 	if (psTarget->type == OBJ_STRUCTURE)
@@ -1491,13 +1549,8 @@ UDWORD	calcDamage(UDWORD baseDamage, WEAPON_EFFECT weaponEffect, BASE_OBJECT *ps
 		damage += baseDamage * (asWeaponModifierBody[weaponEffect][body] - 100);
 	}
 
-	// A little fail safe!
-	if (damage == 0 && baseDamage != 0)
-	{
-		return 1;
-	}
-
-	return damage / 100;
+	//Always do at least one damage.
+	return MAX(damage / 100, 1);
 }
 
 /*
@@ -1515,36 +1568,88 @@ UDWORD	calcDamage(UDWORD baseDamage, WEAPON_EFFECT weaponEffect, BASE_OBJECT *ps
  *    multiplied by -1, resulting in a negative number. Killed features do not
  *    result in negative numbers.
  */
-static int32_t objectDamage(BASE_OBJECT *psObj, unsigned damage, WEAPON_CLASS weaponClass, WEAPON_SUBCLASS weaponSubClass, unsigned impactTime, bool isDamagePerSecond, int minDamage)
+static int32_t objectDamageDispatch(DAMAGE *psDamage)
 {
-	switch (psObj->type)
+	switch (psDamage->psDest->type)
 	{
 	case OBJ_DROID:
-		return droidDamage((DROID *)psObj, damage, weaponClass, weaponSubClass, impactTime, isDamagePerSecond, minDamage);
+		return droidDamage((DROID *)psDamage->psDest, psDamage->damage, psDamage->weaponClass, psDamage->weaponSubClass, psDamage->impactTime, psDamage->isDamagePerSecond, psDamage->minDamage);
 		break;
 
 	case OBJ_STRUCTURE:
-		return structureDamage((STRUCTURE *)psObj, damage, weaponClass, weaponSubClass, impactTime, isDamagePerSecond, minDamage);
+		return structureDamage((STRUCTURE *)psDamage->psDest, psDamage->damage, psDamage->weaponClass, psDamage->weaponSubClass, psDamage->impactTime, psDamage->isDamagePerSecond, psDamage->minDamage);
 		break;
 
 	case OBJ_FEATURE:
-		return featureDamage((FEATURE *)psObj, damage, weaponClass, weaponSubClass, impactTime, isDamagePerSecond, minDamage);
+		return featureDamage((FEATURE *)psDamage->psDest, psDamage->damage, psDamage->weaponClass, psDamage->weaponSubClass, psDamage->impactTime, psDamage->isDamagePerSecond, psDamage->minDamage);
 		break;
 
 	case OBJ_PROJECTILE:
-		ASSERT(!"invalid object type: bullet", "invalid object type: OBJ_PROJECTILE (id=%d)", psObj->id);
-		break;
-
-	case OBJ_TARGET:
-		ASSERT(!"invalid object type: target", "invalid object type: OBJ_TARGET (id=%d)", psObj->id);
+		ASSERT(!"invalid object type: bullet", "invalid object type: OBJ_PROJECTILE (id=%d)", psDamage->psDest->id);
 		break;
 
 	default:
-		ASSERT(!"unknown object type", "unknown object type %d, id=%d", psObj->type, psObj->id);
+		ASSERT(!"unknown object type", "unknown object type %d, id=%d", psDamage->psDest->type, psDamage->psDest->id);
 	}
 	return 0;
 }
 
+static bool isFriendlyFire(DAMAGE* psDamage)
+{
+	return psDamage->psProjectile->psDest && psDamage->psProjectile->psSource->player == psDamage->psProjectile->psDest->player;
+}
+
+static bool shouldIncreaseExperience(DAMAGE *psDamage)
+{
+	return psDamage->psProjectile->psSource && !isFeature(psDamage->psProjectile->psDest) && !isFriendlyFire(psDamage);
+}
+
+static void updateKills(DAMAGE* psDamage)
+{
+	if (bMultiPlayer)
+	{
+		updateMultiStatsKills(psDamage->psDest, psDamage->psProjectile->psSource->player);
+	}
+
+	if (psDamage->psProjectile->psSource->type == OBJ_DROID)
+	{
+		DROID *psDroid = (DROID *) psDamage->psProjectile->psSource;
+
+		psDroid->kills++;
+
+		if (hasCommander(psDroid))
+		{
+			DROID *psCommander = psDroid->psGroup->psCommander;
+			psCommander->kills++;
+		}
+	}
+	else if (psDamage->psProjectile->psSource->type == OBJ_STRUCTURE)
+	{
+		DROID *psCommander = getDesignatorAttackingObject(psDamage->psProjectile->psSource->player, psDamage->psProjectile->psDest);
+
+		if (psCommander != nullptr) {
+			psCommander->kills++;
+		}
+	}
+}
+
+static int32_t objectDamage(DAMAGE *psDamage)
+{
+	int32_t relativeDamage = objectDamageDispatch(psDamage);
+
+	if (shouldIncreaseExperience(psDamage)) {
+		proj_UpdateExperience(psDamage->psProjectile, abs(relativeDamage) * getExpGain(psDamage->psProjectile->psSource->player) / 100);
+
+		bool isTargetDestroyed = relativeDamage < 0;
+
+		if (isTargetDestroyed) {
+			updateKills(psDamage);
+		}
+	}
+
+	return relativeDamage;
+}
+
 /* Returns true if an object has just been hit by an electronic warfare weapon*/
 static bool	justBeenHitByEW(BASE_OBJECT *psObj)
 {
