@@ -270,7 +270,7 @@ bool bInTutorial = false;
 // ----------------------------------------------------------
 
 Vector2i positions[MAX_PLAYERS];
-std::vector<Vector2i> derricks;
+static std::unordered_set<uint16_t> derricks;
 
 void scriptSetStartPos(int position, int x, int y)
 {
@@ -281,8 +281,13 @@ void scriptSetStartPos(int position, int x, int y)
 
 void scriptSetDerrickPos(int x, int y)
 {
-	Vector2i pos(x, y);
-	derricks.push_back(pos);
+	const auto mx = map_coord(x);
+	const auto my = map_coord(y);
+	// MAX_TILE_TEXTURES is 255, so 2 bytes are enough
+	// to describe a map position
+	static_assert(MAX_TILE_TEXTURES <= 255, "If MAX_TILE_TEXTURES is raised above 255, this code will need to change");
+	uint16_t out = (mx << 8) | my;
+	derricks.insert(out);
 }
 
 bool scriptInit()
@@ -306,16 +311,18 @@ Vector2i getPlayerStartPosition(int player)
 nlohmann::json scripting_engine::constructDerrickPositions()
 {
 	// Static map knowledge about start positions
-	//== * ```derrickPositions``` An array of derrick starting positions on the current map. Each item in the array is an
+	//== * ```derrickPositions``` A set of derrick starting positions on the current map. Each item in the set is an
 	//== object containing the x and y variables for a derrick.
 	nlohmann::json derrickPositions = nlohmann::json::array(); //engine->newArray(derricks.size());
-	for (int i = 0; i < derricks.size(); i++)
+	for (uint16_t pos: derricks)
 	{
 		nlohmann::json vector = nlohmann::json::object();
-		vector["x"] = map_coord(derricks[i].x);
-		vector["y"] = map_coord(derricks[i].y);
+		const auto mx = (pos & static_cast<uint16_t>(0xff00)) >> 8;
+		const auto my = pos & (static_cast<uint16_t>(0x00ff));
+		vector["x"] = mx;
+		vector["y"] = my;
 		vector["type"] = SCRIPT_POSITION;
-		derrickPositions.push_back(vector);
+		derrickPositions.push_back(std::move(vector));
 	}
 	return derrickPositions;
 }
@@ -550,7 +557,7 @@ wzapi::scripting_instance* scripting_engine::loadPlayerScript(const WzString& pa
 
 
 	//== * ```difficulty``` The currently set campaign difficulty, or the current AI's difficulty setting. It will be one of
-	//== ```EASY```, ```MEDIUM```, ```HARD``` or ```INSANE```.
+	//== ```SUPEREASY``` (campaign only), ```EASY```, ```MEDIUM```, ```HARD``` or ```INSANE```.
 	if (game.type == LEVEL_TYPE::SKIRMISH)
 	{
 		globalVars["difficulty"] = static_cast<int8_t>(difficulty);
@@ -885,9 +892,9 @@ std::vector<scripting_engine::timerNodeSnapshot> scripting_engine::debug_GetTime
 	return debug_timer_snapshot;
 }
 
-void jsAutogameSpecific(const WzString &name, int player)
+void jsAutogameSpecific(const WzString &name, int player, AIDifficulty difficulty)
 {
-	wzapi::scripting_instance* instance = loadPlayerScript(name, player, AIDifficulty::MEDIUM);
+	wzapi::scripting_instance* instance = loadPlayerScript(name, player, difficulty);
 	if (!instance)
 	{
 		console(_("Failed to load selected AI! Check your logs to see why."));
@@ -974,42 +981,43 @@ void jsShowDebug()
 //__
 //__ ## eventDesignBody()
 //__
-//__An event that is run when current user picks a body in the design menu.
+//__ An event that is run when current user picks a body in the design menu.
 //__
 //__ ## eventDesignPropulsion()
 //__
-//__An event that is run when current user picks a propulsion in the design menu.
+//__ An event that is run when current user picks a propulsion in the design menu.
 //__
 //__ ## eventDesignWeapon()
 //__
-//__An event that is run when current user picks a weapon in the design menu.
+//__ An event that is run when current user picks a weapon in the design menu.
 //__
 //__ ## eventDesignCommand()
 //__
-//__An event that is run when current user picks a command turret in the design menu.
+//__ An event that is run when current user picks a command turret in the design menu.
 //__
 //__ ## eventDesignSystem()
 //__
-//__An event that is run when current user picks a system other than command turret in the design menu.
+//__ An event that is run when current user picks a system other than command turret in the design menu.
 //__
 //__ ## eventDesignQuit()
 //__
-//__An event that is run when current user leaves the design menu.
+//__ An event that is run when current user leaves the design menu.
 //__
 //__ ## eventMenuBuildSelected()
 //__
-//__An event that is run when current user picks something new in the build menu.
+//__ An event that is run when current user picks something new in the build menu.
 //__
 //__ ## eventMenuBuild()
 //__
-//__An event that is run when current user opens the build menu.
+//__ An event that is run when current user opens the build menu.
 //__
 //__ ## eventMenuResearch()
 //__
-//__An event that is run when current user opens the research menu.
+//__ An event that is run when current user opens the research menu.
 //__
 //__ ## eventMenuManufacture()
-//__An event that is run when current user opens the manufacture menu.
+//__
+//__ An event that is run when current user opens the manufacture menu.
 //__
 bool triggerEvent(SCRIPT_TRIGGER_TYPE trigger, BASE_OBJECT *psObj)
 {
@@ -1119,7 +1127,7 @@ bool triggerEvent(SCRIPT_TRIGGER_TYPE trigger, BASE_OBJECT *psObj)
 	if ((trigger == TRIGGER_START_LEVEL || trigger == TRIGGER_GAME_LOADED) && !saveandquit_enabled().empty())
 	{
 		saveGame(saveandquit_enabled().c_str(), GTYPE_SAVE_START);
-		exit(0);
+		wzQuit(0);
 	}
 
 	return true;
@@ -1584,7 +1592,10 @@ bool triggerEventAllianceOffer(uint8_t from, uint8_t to)
 //__
 bool triggerEventAllianceAccepted(uint8_t from, uint8_t to)
 {
-	ASSERT(scriptsReady, "Scripts not initialized yet");
+	if (!scriptsReady)
+	{
+		return false; //silently ignore
+	}
 	for (auto *instance : scripts)
 	{
 		instance->handle_eventAllianceAccepted(from, to);
@@ -1598,7 +1609,10 @@ bool triggerEventAllianceAccepted(uint8_t from, uint8_t to)
 //__
 bool triggerEventAllianceBroken(uint8_t from, uint8_t to)
 {
-	ASSERT(scriptsReady, "Scripts not initialized yet");
+	if (!scriptsReady)
+	{
+		return false; //silently ignore
+	}
 	for (auto *instance : scripts)
 	{
 		instance->handle_eventAllianceBroken(from, to);
@@ -2010,9 +2024,18 @@ bool scripting_engine::loadLabels(const char *filename)
 		}
 		else if (list[i].startsWith("object"))
 		{
-			p.id = ini.value("id").toInt();
+			auto id = ini.value("id").toInt();
+			const auto player = ini.value("player").toInt();
+			const auto it = moduleToBuilding[player].find(id);
+			if (it != moduleToBuilding[player].end())
+			{
+				// replace moduleId with its building id
+				debug(LOG_NEVER, "replaced with %i;%i", id, it->second);
+				id = it->second;
+			}
+			p.id = id;
 			p.type = ini.value("type").toInt();
-			p.player = ini.value("player").toInt();
+			p.player = player;
 			p.triggered = ini.value("triggered", -1).toInt(); // deactivated by default
 			p.subscriber = ini.value("subscriber", ALL_PLAYERS).toInt();
 			labels[label] = p;
@@ -2106,9 +2129,11 @@ bool scripting_engine::writeLabels(const char *filename)
 		}
 		else
 		{
+			auto id = l.id;
+			auto player = l.player;
 			ini.beginGroup("object_" + WzString::number(c[4]++));
-			ini.setValue("id", l.id);
-			ini.setValue("player", l.player);
+			ini.setValue("id", id);
+			ini.setValue("player", player);
 			ini.setValue("type", l.type);
 			ini.setValue("label", WzString::fromUtf8(key));
 			ini.setValue("triggered", l.triggered);
@@ -2142,7 +2167,9 @@ bool scripting_engine::writeLabels(const char *filename)
 //-- Optionally add a filter on it in the second parameter, which can
 //-- be a specific player to watch for, or ```ALL_PLAYERS``` by default.
 //-- This is a fast operation of O(log n) algorithmic complexity. (3.2+ only)
+//--
 //-- ## resetArea(labelName[, playerFilter])
+//--
 //-- Reset the trigger on an area. Next time a unit enters the area, it will trigger
 //-- an area event. Optionally add a filter on it in the second parameter, which can
 //-- be a specific player to watch for, or ```ALL_PLAYERS``` by default.
@@ -2375,6 +2402,7 @@ optional<std::string> scripting_engine::_findMatchingLabel(wzapi::game_object_id
 	value.id = obj_id.id;
 	value.player = obj_id.player;
 	value.type = obj_id.type;
+	debug(LOG_NEVER, "looking for label %i;%i;%i", obj_id.id, obj_id.player, obj_id.type);
 	std::string label;
 	for (const auto &it : labels)
 	{
