@@ -94,6 +94,7 @@ static void replaceStructureComponent(STRUCTURE *pList, UDWORD oldType, UDWORD o
                                       UDWORD newCompInc, UBYTE player);
 static void switchComponent(DROID *psDroid, UDWORD oldType, UDWORD oldCompInc,
                             UDWORD newCompInc);
+
 static void replaceTransDroidComponents(DROID *psTransporter, UDWORD oldType,
                                         UDWORD oldCompInc, UDWORD newCompInc);
 
@@ -909,6 +910,37 @@ static void eventResearchedHandleUpgrades(const RESEARCH *psResearch, const STRU
 	}
 }
 
+static void makeComponentRedundant(UBYTE &state)
+{
+	switch (state)
+	{
+	case AVAILABLE:
+		state = REDUNDANT;
+		break;
+	case UNAVAILABLE:
+		state = REDUNDANT_UNAVAILABLE;
+		break;
+	case FOUND:
+		state = REDUNDANT_FOUND;
+		break;
+	}
+}
+
+static void makeComponentAvailable(UBYTE &state)
+{
+	switch (state)
+	{
+	case UNAVAILABLE:
+	case FOUND:
+		state = AVAILABLE;
+		break;
+	case REDUNDANT_UNAVAILABLE:
+	case REDUNDANT_FOUND:
+		state = REDUNDANT;
+		break;
+	}
+}
+
 /* process the results of a completed research topic */
 void researchResult(UDWORD researchIndex, UBYTE player, bool bDisplay, STRUCTURE *psResearchFacility, bool bTrigger)
 {
@@ -927,16 +959,13 @@ void researchResult(UDWORD researchIndex, UBYTE player, bool bDisplay, STRUCTURE
 	//check for structures to be made available
 	for (unsigned short pStructureResult : pResearch->pStructureResults)
 	{
-		if (apStructTypeLists[player][pStructureResult] != REDUNDANT)
-		{
-			apStructTypeLists[player][pStructureResult] = AVAILABLE;
-		}
+		makeComponentAvailable(apStructTypeLists[player][pStructureResult]);
 	}
 
 	//check for structures to be made redundant
 	for (unsigned short pRedStruct : pResearch->pRedStructs)
 	{
-		apStructTypeLists[player][pRedStruct] = REDUNDANT;
+		makeComponentRedundant(apStructTypeLists[player][pRedStruct]);
 	}
 
 	//check for component replacement
@@ -946,7 +975,7 @@ void researchResult(UDWORD researchIndex, UBYTE player, bool bDisplay, STRUCTURE
 		{
 			COMPONENT_STATS *pOldComp = ri.pOldComponent;
 			replaceComponent(ri.pNewComponent, pOldComp, player);
-			apCompLists[player][pOldComp->compType][pOldComp->index] = REDUNDANT;
+			makeComponentRedundant(apCompLists[player][pOldComp->compType][pOldComp->index]);
 		}
 	}
 
@@ -957,10 +986,7 @@ void researchResult(UDWORD researchIndex, UBYTE player, bool bDisplay, STRUCTURE
 		COMPONENT_TYPE type = componentResult->compType;
 		//set the component state to AVAILABLE
 		int compInc = componentResult->index;
-		if (apCompLists[player][type][compInc] != REDUNDANT)
-		{
-			apCompLists[player][type][compInc] = AVAILABLE;
-		}
+		makeComponentAvailable(apCompLists[player][type][compInc]);
 		//check for default sensor
 		if (type == COMP_SENSOR && (asSensorStats + compInc)->location == LOC_DEFAULT)
 		{
@@ -983,7 +1009,7 @@ void researchResult(UDWORD researchIndex, UBYTE player, bool bDisplay, STRUCTURE
 	for (auto &pRedArtefact : pResearch->pRedArtefacts)
 	{
 		COMPONENT_TYPE type = pRedArtefact->compType;
-		apCompLists[player][type][pRedArtefact->index] = REDUNDANT;
+		makeComponentRedundant(apCompLists[player][type][pRedArtefact->index]);
 	}
 
 	//Add message to player's list if Major Topic
@@ -1131,7 +1157,7 @@ void cancelResearch(STRUCTURE *psBuilding, QUEUE_MODE mode)
 	UDWORD              topicInc;
 	PLAYER_RESEARCH	    *pPlayerRes;
 
-	ASSERT_OR_RETURN(, psBuilding->pStructureType->type == REF_RESEARCH, "Structure not a research facility");
+	ASSERT_OR_RETURN(, psBuilding->pStructureType && psBuilding->pStructureType->type == REF_RESEARCH, "Structure not a research facility");
 
 	RESEARCH_FACILITY *psResFac = &psBuilding->pFunctionality->researchFacility;
 	if (!(RESEARCH *)psResFac->psSubject)
@@ -1396,9 +1422,7 @@ static void replaceComponent(COMPONENT_STATS *pNewComponent, COMPONENT_STATS *pO
 	replaceDroidComponent(apsDroidLists[player], oldType, oldCompInc, newCompInc);
 	replaceDroidComponent(mission.apsDroidLists[player], oldType, oldCompInc, newCompInc);
 	replaceDroidComponent(apsLimboDroids[player], oldType, oldCompInc, newCompInc);
-
-	//check thru the templates
-	enumerateTemplates(player, [oldType, oldCompInc, newCompInc](DROID_TEMPLATE* psTemplates) {
+	const auto replaceComponentInTemplate = [oldType, oldCompInc, newCompInc](DROID_TEMPLATE* psTemplates) {
 		switch (oldType)
 		{
 		case COMP_BODY:
@@ -1428,8 +1452,23 @@ static void replaceComponent(COMPONENT_STATS *pNewComponent, COMPONENT_STATS *pO
 			return true;
 		}
 		return true;
-	});
-
+	};
+	//check thru the templates
+	enumerateTemplates(player, replaceComponentInTemplate);
+	// also check build queues
+	STRUCTURE *psNBuilding = nullptr;
+	for (STRUCTURE *psCBuilding = apsStructLists[player]; psCBuilding != nullptr; psCBuilding = psNBuilding)
+	{
+		/* Copy the next pointer - not 100% sure if the structure could get destroyed but this covers us anyway */
+		psNBuilding = psCBuilding->psNext;
+		if ((psCBuilding->pStructureType->type == STRUCTURE_TYPE::REF_FACTORY ||
+			psCBuilding->pStructureType->type == STRUCTURE_TYPE::REF_CYBORG_FACTORY ||
+			psCBuilding->pStructureType->type == STRUCTURE_TYPE::REF_VTOL_FACTORY) &&
+			psCBuilding->pFunctionality->factory.psSubject != nullptr)
+		{
+			replaceComponentInTemplate(psCBuilding->pFunctionality->factory.psSubject);
+		}
+	}
 	replaceStructureComponent(apsStructLists[player], oldType, oldCompInc, newCompInc, player);
 	replaceStructureComponent(mission.apsStructLists[player], oldType, oldCompInc, newCompInc, player);
 }
@@ -1492,7 +1531,7 @@ void researchReward(UBYTE losingPlayer, UBYTE rewardPlayer)
 			if (psFacility->psBestTopic)
 			{
 				topicIndex = ((RESEARCH *)psFacility->psBestTopic)->ref - STAT_RESEARCH;
-				if (topicIndex)
+				if (topicIndex && !IsResearchCompleted(&asPlayerResList[rewardPlayer][topicIndex]))
 				{
 					//if it cost more - it is better (or should be)
 					if (researchPoints < asResearch[topicIndex].researchPoints)
