@@ -17,7 +17,7 @@
 	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
-#include <3rdparty/json/json.hpp> // Must come before WZ includes
+#include <nlohmann/json.hpp> // Must come before WZ includes
 using json = nlohmann::json;
 
 #include "version.h"
@@ -47,8 +47,15 @@ using json = nlohmann::json;
 # pragma GCC diagnostic push
 # pragma GCC diagnostic ignored "-Wmaybe-uninitialized" // Ignore on GCC 4.7+
 #endif
+#if defined(__GNUC__) && !defined(__INTEL_COMPILER) && !defined(__clang__) && (12 <= __GNUC__)
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wstringop-overflow" // Ignore on GCC 12+`
+#endif
 #define ONLY_C_LOCALE 1
 #include <date/date.h>
+#if defined(__GNUC__) && !defined(__INTEL_COMPILER) && !defined(__clang__) && (12 <= __GNUC__)
+# pragma GCC diagnostic pop
+#endif
 #if defined(__GNUC__) && !defined(__INTEL_COMPILER) && !defined(__clang__) && ((4 < __GNUC__) || ((4 == __GNUC__) && (7 <= __GNUC_MINOR__)))
 # pragma GCC diagnostic pop
 #endif
@@ -654,6 +661,7 @@ static void fetchLatestData(const std::vector<std::string> &updateDataUrls, Proc
 	std::vector<std::string> additionalUrls(updateDataUrls.begin() + 1, updateDataUrls.end());
 	pRequest->onSuccess = [additionalUrls, processDataFunc, outputPaths](const std::string& url, const HTTPResponseDetails& responseDetails, const std::shared_ptr<MemoryStruct>& data) {
 
+		std::string urlCopy = url;
 		long httpStatusCode = responseDetails.httpStatusCode();
 		if (httpStatusCode != 200)
 		{
@@ -672,8 +680,8 @@ static void fetchLatestData(const std::vector<std::string> &updateDataUrls, Proc
 		}
 		catch (const std::exception& e) {
 			std::string errorStr = e.what();
-			wzAsyncExecOnMainThread([url, errorStr]{
-				debug(LOG_NET, "%s; %s", errorStr.c_str(), url.c_str());
+			wzAsyncExecOnMainThread([urlCopy, errorStr]{
+				debug(LOG_INFO, "Failed to verify signature: %s; %s", errorStr.c_str(), urlCopy.c_str());
 			});
 			fetchLatestData(additionalUrls, processDataFunc, outputPaths);
 			return;
@@ -686,8 +694,8 @@ static void fetchLatestData(const std::vector<std::string> &updateDataUrls, Proc
 		}
 		catch (const std::exception &e) {
 			std::string errorStr = e.what();
-			wzAsyncExecOnMainThread([url, errorStr]{
-				debug(LOG_NET, "%s; %s", errorStr.c_str(), url.c_str());
+			wzAsyncExecOnMainThread([urlCopy, errorStr]{
+				debug(LOG_INFO, "Failed to parse JSON: %s; %s", errorStr.c_str(), urlCopy.c_str());
 			});
 			fetchLatestData(additionalUrls, processDataFunc, outputPaths);
 			return;
@@ -794,12 +802,12 @@ static void fetchLatestData(const std::vector<std::string> &updateDataUrls, Proc
 				tryNextUrl = true;
 				break;
 			case URLRequestFailureType::CANCELLED:
-				wzAsyncExecOnMainThread([url]{
+				wzAsyncExecOnMainThread([]{
 					debug(LOG_INFO, "Update check was cancelled");
 				});
 				break;
 			case URLRequestFailureType::CANCELLED_BY_SHUTDOWN:
-				wzAsyncExecOnMainThread([url]{
+				wzAsyncExecOnMainThread([]{
 					debug(LOG_WARNING, "Update check was cancelled by application shutdown");
 				});
 				break;
