@@ -35,6 +35,9 @@
 #include "radar.h"
 #include "activity.h"
 
+#define MAX_REPLAY_FILES 36
+constexpr int MAX_OLD_LOGS = 50;
+
 /***************************************************************************/
 
 struct WARZONE_GLOBALS
@@ -66,8 +69,19 @@ struct WARZONE_GLOBALS
 	bool autoAdjustDisplayScale = true;
 	int autoLagKickSeconds = 60;
 	bool disableReplayRecording = false;
+	int maxReplaysSaved = MAX_REPLAY_FILES;
+	int oldLogsLimit = MAX_OLD_LOGS;
 	uint32_t MPinactivityMinutes = 5;
 	uint8_t MPopenSpectatorSlots = 0;
+	int fogStart = 4000;
+	int fogEnd = 8000;
+	int lodDistanceBiasPercentage = WZ_LODDISTANCEPERCENTAGE_HIGH; // default to "High" to best match prior version behavior
+	int minimizeOnFocusLoss = -1; // see enum class MinimizeOnFocusLossBehavior
+	// fullscreen mode settings
+	UDWORD fullscreenModeWidth = 0; // current display default
+	UDWORD fullscreenModeHeight = 0; // current display default
+	int fullscreenModeScreen = -1;
+	int toggleFullscreenMode = 0; // 0 = the backend default
 };
 
 static WARZONE_GLOBALS warGlobs;
@@ -376,6 +390,28 @@ void war_SetRadarJump(bool radarJump)
 	ActivityManager::instance().changedSetting("radarJump", std::to_string(radarJump));
 }
 
+int war_getLODDistanceBiasPercentage()
+{
+	return warGlobs.lodDistanceBiasPercentage;
+}
+
+void war_setLODDistanceBiasPercentage(int bias)
+{
+	if (bias > 200) { bias = 200; }
+	if (bias < -200) { bias = -200; }
+	warGlobs.lodDistanceBiasPercentage = bias;
+}
+
+int war_getMinimizeOnFocusLoss()
+{
+	return warGlobs.minimizeOnFocusLoss;
+}
+
+void war_setMinimizeOnFocusLoss(int val)
+{
+	warGlobs.minimizeOnFocusLoss = val;
+}
+
 video_backend war_getGfxBackend()
 {
 	return warGlobs.gfxBackend;
@@ -431,6 +467,26 @@ void war_setDisableReplayRecording(bool disable)
 	warGlobs.disableReplayRecording = disable;
 }
 
+int war_getMaxReplaysSaved()
+{
+	return warGlobs.maxReplaysSaved;
+}
+
+void war_setMaxReplaysSaved(int maxReplaysSaved)
+{
+	warGlobs.maxReplaysSaved = maxReplaysSaved;
+}
+
+int war_getOldLogsLimit()
+{
+	return warGlobs.oldLogsLimit;
+}
+
+void war_setOldLogsLimit(int oldLogsLimit)
+{
+	warGlobs.oldLogsLimit = oldLogsLimit;
+}
+
 uint32_t war_getMPInactivityMinutes()
 {
 	return warGlobs.MPinactivityMinutes;
@@ -455,3 +511,62 @@ void war_setMPopenSpectatorSlots(uint16_t spectatorSlots)
 	spectatorSlots = std::min<uint16_t>(spectatorSlots, MAX_SPECTATOR_SLOTS);
 	warGlobs.MPopenSpectatorSlots = spectatorSlots;
 }
+
+int war_getFogEnd()
+{
+	return warGlobs.fogEnd;
+}
+
+int war_getFogStart()
+{
+	return warGlobs.fogStart;
+}
+
+void war_setFogEnd(int end)
+{
+	 warGlobs.fogEnd = end;
+}
+
+void war_setFogStart(int start)
+{
+	 warGlobs.fogStart = start;
+}
+
+void war_SetFullscreenModeWidth(UDWORD width)
+{
+	warGlobs.fullscreenModeWidth = width;
+}
+
+UDWORD war_GetFullscreenModeWidth()
+{
+	return warGlobs.fullscreenModeWidth;
+}
+
+void war_SetFullscreenModeHeight(UDWORD height)
+{
+	warGlobs.fullscreenModeHeight = height;
+}
+
+UDWORD war_GetFullscreenModeHeight()
+{
+	return warGlobs.fullscreenModeHeight;
+}
+void war_SetFullscreenModeScreen(int screen)
+{
+	warGlobs.fullscreenModeScreen = screen;
+}
+
+int war_GetFullscreenModeScreen()
+{
+	return warGlobs.fullscreenModeScreen;
+}
+
+void war_setToggleFullscreenMode(int mode)
+{
+	warGlobs.toggleFullscreenMode = mode;
+}
+
+int war_getToggleFullscreenMode()
+{
+	return warGlobs.toggleFullscreenMode;
+}
