@@ -36,6 +36,8 @@
 
 #define MIN_MPINACTIVITY_MINUTES 4
 
+#define WZ_LODDISTANCEPERCENTAGE_HIGH -50
+
 /***************************************************************************/
 /*
  *	Global Definitions
@@ -80,12 +82,22 @@ void war_SetVsync(int value);
 int war_GetVsync();
 void war_SetDisplayScale(unsigned int scale);
 unsigned int war_GetDisplayScale();
+// non-fullscreen window sizes / screen
 void war_SetWidth(UDWORD width);
 UDWORD war_GetWidth();
 void war_SetScreen(int screen);
 int war_GetScreen();
 void war_SetHeight(UDWORD height);
 UDWORD war_GetHeight();
+// fullscreen display mode + screen
+void war_SetFullscreenModeWidth(UDWORD width);
+UDWORD war_GetFullscreenModeWidth();
+void war_SetFullscreenModeScreen(int screen);
+int war_GetFullscreenModeScreen();
+void war_SetFullscreenModeHeight(UDWORD height);
+UDWORD war_GetFullscreenModeHeight();
+void war_setToggleFullscreenMode(int mode);
+int war_getToggleFullscreenMode();
 void war_SetVideoBufferDepth(UDWORD videoBufferDepth);
 UDWORD war_GetVideoBufferDepth();
 void war_SetPauseOnFocusLoss(bool enabled);
@@ -120,10 +132,22 @@ int war_getAutoLagKickSeconds();
 void war_setAutoLagKickSeconds(int seconds);
 bool war_getDisableReplayRecording();
 void war_setDisableReplayRecording(bool disable);
+int war_getMaxReplaysSaved();
+void war_setMaxReplaysSaved(int maxReplaysSaved);
+int war_getOldLogsLimit();
+void war_setOldLogsLimit(int oldLogsLimit);
 uint32_t war_getMPInactivityMinutes();
 void war_setMPInactivityMinutes(uint32_t minutes);
 uint16_t war_getMPopenSpectatorSlots();
 void war_setMPopenSpectatorSlots(uint16_t spectatorSlots);
+int war_getFogEnd();
+int war_getFogStart();
+void war_setFogEnd(int end);
+void war_setFogStart(int start);
+int war_getLODDistanceBiasPercentage();
+void war_setLODDistanceBiasPercentage(int bias);
+int war_getMinimizeOnFocusLoss();
+void war_setMinimizeOnFocusLoss(int val);
 
 /**
  * Enable or disable sound initialization
