@@ -91,7 +91,7 @@ static void setupLoadingScreen()
 	barLeftX = barRightX - boxWidth;
 	barLeftY = barRightY - boxHeight;
 
-	starsNum = boxWidth / boxHeight;
+	starsNum = boxWidth / std::max<int>(boxHeight, 1);
 	starHeight = static_cast<int>(2.0 * h / 640.0);
 
 	if (!stars)
@@ -107,7 +107,7 @@ static void setupLoadingScreen()
 
 bool recalculateEffectiveHeadlessValue()
 {
-	if (hostlaunch == HostLaunch::Skirmish || hostlaunch == HostLaunch::Autohost || autogame_enabled())
+	if (hostlaunch == HostLaunch::Skirmish || hostlaunch == HostLaunch::Autohost || hostlaunch == HostLaunch::LoadReplay || autogame_enabled())
 	{
 		// only support headless mode if hostlaunch is --skirmish or --autogame
 		return bHeadlessAutoGameModeCLIOption;
@@ -218,7 +218,6 @@ TITLECODE titleLoop()
 	audio_Update();
 
 	pie_SetFogStatus(false);
-	pie_ScreenFlip(CLEAR_BLACK);//title loop
 
 	if ((keyDown(KEY_LALT) || keyDown(KEY_RALT)) && keyPressed(KEY_RETURN))
 	{
@@ -242,6 +241,7 @@ void loadingScreenCallback()
 	{
 		return;
 	}
+
 	lastTick = currTick;
 
 	/* Draw the black rectangle at the bottom, with a two pixel border */
@@ -265,7 +265,9 @@ void loadingScreenCallback()
 		}
 	}
 
-	pie_ScreenFlip(CLEAR_OFF_AND_NO_BUFFER_DOWNLOAD);//loading callback		// don't clear.
+	pie_ScreenFrameRenderEnd();
+	pie_ScreenFrameRenderBegin();
+
 	audio_Update();
 
 	wzPumpEventsWhileLoading();
@@ -274,6 +276,7 @@ void loadingScreenCallback()
 // fill buffers with the static screen
 void initLoadingScreen(bool drawbdrop)
 {
+	pie_ScreenFrameRenderBegin(); // start a frame *if one isn't yet started*
 	setupLoadingScreen();
 	wzShowMouse(false);
 	pie_SetFogStatus(false);
@@ -293,10 +296,6 @@ void initLoadingScreen(bool drawbdrop)
 	{
 		screen_StopBackDrop();
 	}
-
-	// Start with two cleared buffers as the hacky loading screen code re-uses old buffers to create its effect.
-	pie_ScreenFlip(CLEAR_BLACK);
-	pie_ScreenFlip(CLEAR_BLACK);
 }
 
 // shut down the loading screen
@@ -308,7 +307,6 @@ void closeLoadingScreen()
 		stars = nullptr;
 	}
 	resSetLoadCallback(nullptr);
-	pie_ScreenFlip(CLEAR_BLACK);
 }
 
 
